#ifndef DECODEMANAGER_H
#define DECODEMANAGER_H

#include <QObject>
#include <QMap>
#include <QMutex>
#include <QTimer>
#include <QVector>
#include <QList>

class QByteArray;
class QTime;

#include "adcm_raw.h"
//class FrameList; // adcm_raw.h
//class AdcmDataBlockDecoder; // adcm_raw.h
class AdcConfig;
class AdcmEvent;
class CalAvgConfig;
class CalZeroConfig;
class adcm_counters_t;
class adcm_decstat_t;
class adcm_el_t;
#include "adcm_el.h"
class conf_mem_t;
class DecoderConfig;
class DecodeWorker;
class HistManager;
class HistList;
class QSemaphore;
class storage;
class StorageConfig;
class EventListFiller;


struct WorkerJob
{
    uint workerIndex;
    EventListIt startPos;
    EventListIt endPos;
    int size;
    bool completed;
    WorkerJob(int index, EventListIt start,EventListIt end, int s):
        workerIndex(index),
        startPos(start),
        endPos(end),
        size(s),
        completed(false)
    {}
};

class DecodeManager : public QObject
{
    Q_OBJECT
public:
    DecodeManager(QObject *parent);
    ~DecodeManager();
    EventListFiller * createFiller();
    void pause();
    void resume();
    void setAdcConfMem(const conf_mem_t &value); // (Re-)Initialize
    void reset(); // Calls from filler
    void clear(); // Clears hist & stats (worker+mngr), storage & counters(mngr)
public Q_SLOTS:
    // Slots connected from MainWindow
    void loop();
    void setAdcConfig(const AdcConfig &r); // Any changing
    void setDecoderConfig(const DecoderConfig &r);
    void setStorageConfig(const StorageConfig &r);
    bool appendChannelCounters(const adcm_counters_t &cnt);
    bool endBurst();
    void clearHist(int);
    void setHistEnabled(bool);
    // Slot for internal timer
    // Slot for DecodeWorker feedback
    void addWorkerStatistics(const adcm_decstat_t &, int index=-1);
Q_SIGNALS:
    void decodedCalAvg(const CalAvgConfig &);
    void decodedCalZero(const CalZeroConfig &);
    void monitorEvent(const AdcmEvent &); // we should share monitorEventTime and include mutex

    void summedStatisticsUpdated(const adcm_decstat_t &);
    void histListModified(const HistList &);
    void decoderConfigUpdated();
    void startWorkers(int);
    void storageConfigUpdated(bool);

public:
    static const int numberWorkers;

private:
    void clearCounters();
    bool startJob();
    void clenUpCompletedJobs(int index=-1);
    void info();
    void checkEvListSize();
    DecodeWorker* getIdleWorker();
    void writeChMap();

    bool paused;
    uint channalNumber;
    AdcConfig * const adcConfig; // Uses in writeChMap to write triggers
    StorageConfig * const storageConfig;
    DecoderConfig * const decoderConfig; // Unique, created at DecodeManager (can be replaced with decoderConfig->mode)
    HistManager * const hmgr; // Unique created at DecodeManager
    bool histEnabled;
    adcm_counters_t * const counters;
    storage * const stor;

    QVector<QThread*> workerThreads;
    QVector<DecodeWorker*> workers;
    QList<WorkerJob> currentJobs;

    QSemaphore * const idleWorkers; // Worker releases (job done); Manager acquires (job assigned)
    QSemaphore * const freeEvents; // Manager releases (clean up job); Filler via eventList acquires (job creating)
    QSemaphore * const managerEvents; // Filler releases (job created); Manager acquires (job assigned)
    QSemaphore * const pauseSem; // Released when loop finaly stoped;

    uint lastEventListSize; // Debug value
    struct adcm_el_t * const eventList;

    QMap<int, adcm_decstat_t> workersDecStat;
    adcm_decstat_t * const summedDecStat;

    QTime * const lastHistSumTime;
    QTime * const workStatTimer;
};

#endif // DECODEMANAGER_H
