#ifndef _DECODEWORKER_H
#define _DECODEWORKER_H 1

#include <QObject>

class AdcmEvent;
//class adcm_el_t;
#include "adcmlib/adcm_el.h"
class adcm_decstat_t;

class HistManager;
class storage;

class FrameList;
class HistList;
class AdcmDataBlockDecoder;
class WaveformDecoder;
struct adcm_counters_t;
class conf_mem_t;
class AdcConfig;
class DecoderConfig;
class StorageConfig;
class CalZeroConfig;
class CalAvgConfig;
class QTimer;
class QMutex;
class QSemaphore;

class DecodeWorker : public QObject
{
    Q_OBJECT

public:

    explicit DecodeWorker (int workerIndex,
                      QSemaphore * const _idleWorkers,
                      storage * const _stor,
                      DecoderConfig * const decConf,
                      adcm_el_t *,
                      bool storEnabled);
    virtual ~DecodeWorker ();

    void decoderConfigModified(const DecoderConfig &decConf);
    void setStorageConfig(bool);
    void setAdcConfig(const AdcConfig &r);
    void setAdcConfMem(const conf_mem_t &value);
    void clear(); // Clears stats & hist
    void reset();
    uint getWorkerIndex() const { return workerIndex; }
    bool isIdle();
    void lockWorker();
    void unlockWorker();
    HistManager *getHistManager() { return hmgr; }
    void setHistEnabled(bool);
    void pause();
    void resume();

public Q_SLOTS:
    void sendStatistics();
    void decodeEventList (EventListIt, EventListIt);

Q_SIGNALS:
    void monitorEvent(const AdcmEvent &); // we should share monitorEventTime and include mutex
    void statisticsUpdated(const adcm_decstat_t &, int index);
    void decodedCalZero(const CalZeroConfig &);
    void decodedCalAvg(const CalAvgConfig &);

private:
    bool getHistEnable(int index) const;
    int processEvent(AdcmEvent *ev);
    bool writeEvent(const AdcmEvent &ev);
    void makeCalZero(const AdcmEvent &ev);
    void makeCalAvg(const AdcmEvent &ev);

private Q_SLOTS:
    void sendCalResult();

private:
    const unsigned int workerIndex;
    QSemaphore * const idleWorkers;
    QMutex * const stateMutex; // Locked when a job assigned; unlocked when it is idle
    storage * const stor; // Unique created at DecodeManager
    HistManager * const hmgr; // Separated
    bool histEnabled;
    struct adcm_el_t * const eventList;
    adcm_decstat_t * const decst; // Can be independent and be joined in DM with adcm_decstat_t::add

    WaveformDecoder * const decoder; // Separated
    QTime * const monitorEventTime;

    QTimer * const statisticsTimer; // Call updateStatistics()
    QTimer * const calResultTimer;
    int decodeMode;
    bool storageEnabled;
    conf_mem_t * const conf_mem; // Uses conf_mem->nch in makeCalZero,makeCalAvg and hmgr,decoder reconfiguration. Thus we can store just conf_mem->nch
    CalAvgConfig * const newCalAvgConfig;  // CalAvgConfig/CalZeroConfig should store separated and collected in DecodeManager
    CalZeroConfig * const newCalZeroConfig;// CalAvgConfig/CalZeroConfig should store separated and collected in DecodeManager
    quint64 prevEvTs;
};

#endif /* _DECODEWORKER_H */
