//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <QCheckBox>
#include <QComboBox>
#include <QDateTime>
#include <QPushButton>
#include <QListView>
#include <QDebug>

#include "HistExplorerView.h"
#include "histdialog.h"
#include "histplot.h"
#include "qutil.h"
#include "th1.h"
#include "dsplib/dsplib.h"

HistExplorerView::HistExplorerView(QWidget *parent)
    : QWidget(parent)
{
    setupUi(this);
    //    hists.setAutoDelete(true);
}

void HistExplorerView::showhist (int id)
{
    bool found = false;
    for (HistDialogList::ConstIterator it = hists.begin(); it != hists.end(); ++it) {
        HistDialog * p = *it;
        if (p->getid () == id) {
            p->show ();
            p->raise ();
            found = true;
            break;
        }
    }
    if (!found) {
        qWarning() << QString ("showhist(%1): no hist found!").arg(id);
    }
}

void HistExplorerView::recreateHistograms (const HistList &hl)
{
    ScopedWidgetDisabler cb_hist_inhibit(cb_hist);
    //  qDebug() << "hl=" << hl;

    // save histogram window state
    QMap<QString, MyWidgetState> histWidgetState;
    Q_FOREACH(const HistDialog * h, hists) {
        MyWidgetState ws;
        //    ws.rect = h->rect();
        ws.pos = h->pos();
        ws.size = h->size();
        ws.shown = h->isShown();
        histWidgetState[h->getName()] = ws;
        delete h;
    }
    //  // create list of existing histogram names
    //  QStringList origNameList;
    //  foreach_const(hists, it) {
    //    origNameList.push_back((*it)->getName());
    //  }
    //
    //  // create list of new histogram names
    //  QStringList newNameList;
    //  for(HistList::ConstIterator it = hl.begin(); it!=hl.end(); ++it) {
    //    const HistPtr h1 = it.data();
    //    const QString name(h1->getname());
    //    newNameList.push_back(name);
    //  }
    //  // list of histograms to delete (exist in orig but not in new)
    //  QStringList toDeleteNames;
    //  foreach(origNameList, it) {
    //    const QString &name = *it;
    //    if (!newNameList.contains(name)) {
    //      toDeleteNames.push_back(name);
    //    }
    //  }
    hists.clear();

    QStringList strList;
    for(HistList::const_iterator histIt=hl.begin();histIt != hl.end(); histIt++) {
        th1Ptr h1 = histIt.data();
        const QString name(h1->getname());

        //    if (origNameList.contains(name)) {
        //      foreach(hists, it) {
        //        hist *h = *it;
        //        if (h->getName() == name) {
        //          h->set_h(h1);
        //          break;
        //        }
        //      }
        //    } else {
        HistDialog * h = new HistDialog (this, h1);
        h->setid (histIt.key());
        hists.push_back (h);
        if (histWidgetState.contains(name)) {
            const MyWidgetState &ws = histWidgetState[name];
            //        h->setGeometry(ws.rect);
            h->resize(ws.size);
            h->move(ws.pos);
            h->setShown(ws.shown);
        }
        //    }
        strList.append(name);
    }

    //  // delete unused histograms
    //  for(hist* h = hists.first(); h; h = hists.next()) {
    //    const QString name(h->getName());
    //    if (toDeleteNames.contains(name)) {
    //      hists.remove(h); // iterator will point to next
    //    }
    //  }

    // update ComboBox
    QStringList origList;
    for(int i=0; i<cb_hist->count(); i++)
        origList.push_back(cb_hist->text(i));
    // no ComboBox update if contents unchanged
    if (origList == strList)
        return;
    QString selectionText = cb_hist->currentText();
    int selectionIndex = 0;
    if (int n = strList.findIndex(selectionText) != -1)
        selectionIndex = n;
    cb_hist->clear();
    cb_hist->insertStringList(strList);
    cb_hist->setCurrentItem (selectionIndex);
    cb_hist->adjustSize();
    //  qDebug() << "strList=" << strList;
    //  qDebug() << "------------------------------------------";
}

HistDialog * HistExplorerView::findhist (int id)
{
    for (HistDialogList::ConstIterator it = hists.begin(); it != hists.end(); ++it) {
        HistDialog * p = *it;
        if (p->getid () == id)
            return (p);
    }
    return NULL;
}

const HistDialog * HistExplorerView::const_findhist (int id) const
{
    Q_FOREACH(HistDialog * h, hists) {
        if (h->getid () == id)
            return h;
    }
    return NULL;
}

void HistExplorerView::on_cb_hist_activated( int )
{
    updateView();
}

HistDialog * HistExplorerView::current_hist()
{
    int id = cb_hist->currentItem()-1;
    return findhist (id);
}

const HistDialog * HistExplorerView::const_current_hist() const
{
    int id = cb_hist->currentItem()-1;
    return const_findhist (id);
}

QString HistExplorerView::currentHistName() const
{
    const HistDialog * cur = const_current_hist();
    if (cur)
        return cur->getName();
    else
        return QString();
}

void HistExplorerView::updateView()
{
    update_mainhist_controls();
    update_current_main_hist();
}

void HistExplorerView::update_mainhist_controls()
{
    update_checkBox_autoRange();
    update_checkBox_autoFit();
    //  update_current_main_hist(); -- don't!!!
}

void HistExplorerView::update_current_main_hist()
{

    if (!isVisible()) return;

    for(int n=0; n < cb_hist->count(); n++)
    {
        //          bool enable = true;
        //	  if (decodethread)
        //		enable = decodethread->get_hist_enable (n);

        //          Q3ListBox *listbox = cb_hist_1->listBox(); // TODO
        //          Q3ListBoxItem *item = listbox->item(n);    // TODO
        //          if (item) item->setSelectable(enable);     // TODO
    }

    HistDialog * histDialog = current_hist();
    HistPlot *hp = 0;
    if (!histDialog) {
        clear_current_main_hist();
        return;
    }

    hp = histDialog->getplot();
    const th1Ptr th = histDialog->get_h();
    if (!hp || !th) {
        clear_current_main_hist();
        return;
    }

    histDialog->update();

    histPlot->setpoints(hp->getpointsx(), hp->getpointsy());
//        hist_plot_1->set_fit_points(hp->getfitpointsx(), hp->getfitpointsy());
    double ar_min=0, ar_max=0;
    th->update_autorange();
    //	th->update_autofit();
    th->getrange(&ar_min, &ar_max);
    histPlot->set_x_range(ar_min, ar_max);
    histPlot->replot();
    bool autofit = th->fit_enabled();
    if (autofit) {
        bool ok;
        double v, m, s, n;
        histDialog->get_fit_param(&ok, &v, &m, &s, &n);
        bool fit_res = th->get_options() & th1::FIT_RESOL;
        QString str;
        if (ok) {
            str = QString("Fit: value=%1, mean=%2, sigma=%3, FWHM=%4").arg(v).arg(m).arg(s).arg(s*GausFWHM);
            if (fit_res)
                str += QString(", resolution=%1%%").arg(100.*s*GausFWHM/m);
        } else
            str = "No fit parameters available";
        setStatus(str);
    } else {
        setStatus("");
    }
    update_current_main_hist_bh(true);
}

void HistExplorerView::clear_current_main_hist()
{
    histPlot->clear_points();
    histPlot->clear_fit_points();
    histPlot->replot();
    update_current_main_hist_bh(false);
}

void HistExplorerView::update_current_main_hist_bh(bool enable_controls)
{
    histPlot->setEnabled(enable_controls);
    pbShow->setEnabled(enable_controls);
    checkBox_autoRange->setEnabled(enable_controls);
    checkBox_autoFit->setEnabled(enable_controls);
}

void HistExplorerView::update_checkBox_autoRange()
{
    const HistDialog * h = current_hist();
    if (!h) return;
    const th1Ptr th = h->get_h();
    if (!th) return;
    bool state = th->autorange_enabled();
    //  std::cout<<"update "<<th->getname()<<" autoRange: "<<state<<std::endl;
    checkBox_autoRange->setChecked(state);
}

void HistExplorerView::on_checkBox_autoRange_toggled( bool state )
{
    const HistDialog * h = current_hist();
    if (!h) return;
    const th1Ptr th = h->get_h();
    if (!th) return;
    th->set_autorange_enable(state);
    //  std::cout<<"set "<<th->getname()<<" autoRange: "<<state<<std::endl;
    update_current_main_hist();
}

void HistExplorerView::update_checkBox_autoFit()
{
    const HistDialog * h = current_hist();
    if (!h) return;
    const th1Ptr th = h->get_h();
    if (!th) return;
    bool state = th->fit_enabled();
    checkBox_autoFit->setChecked(state);
    //  std::cout<<"update "<<th->getname()<<" autoFit: "<<state<<std::endl;
}

void HistExplorerView::on_checkBox_autoFit_toggled( bool state )
{
    const HistDialog * h = current_hist();
    if (!h) return;
    const th1Ptr th = h->get_h();
    if (!th) return;
    //  std::cout<<"set "<<th->getname()<<" autoFit: "<<state<<std::endl;
    th->set_fit_enable(state);
    update_current_main_hist();
}

void HistExplorerView::on_pbHideAll_clicked()
{
    Q_FOREACH(HistDialog * h, hists) {
        h->hide();
    }
}

void HistExplorerView::on_pbShow_clicked()
{
    int id = cb_hist->currentItem()-1;
    showhist(id);
}

void HistExplorerView::setStatus(const QString &s)
{
    Q_UNUSED(s);
    // TODO
}

void HistExplorerView::on_pbhclearthis_clicked()
{
    const HistDialog * cur = const_current_hist();
    if(cur)
        emit clearHist(cur->getid());
}


void HistExplorerView::on_pbhclear_clicked()
{
    emit clearHist(-1);
}


void HistExplorerView::on_pbhsave_clicked()
{
    const HistDialog * histDialog = const_current_hist();
    if (histDialog){
        const th1Ptr histPtr = histDialog->get_h();
        if (!histPtr.isNull())
            histPtr->save_to_file(this);
    }
}


void HistExplorerView::on_pbhsaveall_clicked()
{
    QString subdir = QDateTime::currentDateTime().toString("yyyy-MM-dd_hh-mm-ss");
    Q_FOREACH(HistDialog * h, hists) {
        const th1Ptr hist = h->get_h();
        hist->save_to_file(this, subdir);
    }
}
