//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef HISTEXPLORERVIEW_H
#define HISTEXPLORERVIEW_H

#include <QSharedPointer>
#include <QList>

#include "ui_HistExplorerViewUi.h"
#include "histmanager.h"
#include "histdialog.h"


typedef QSharedPointer<HistDialog> HistDialogPtr;

typedef QList<HistDialog *> HistDialogList;

class HistExplorerView : public QWidget, public Ui::HistExplorerViewUi
{
    Q_OBJECT
public:
    explicit HistExplorerView(QWidget *parent = 0);
    void recreateHistograms (const HistList &hl);
    QString currentHistName() const;

public slots:
    void updateView();
signals:
    void clearHist(int index);

protected:
    void showhist(int id);
    HistDialog *findhist(int id);
    const HistDialog *const_findhist(int id) const;
    HistDialog *current_hist();
    const HistDialog *const_current_hist() const;
    void update_mainhist_controls();
    void update_current_main_hist();
    void update_checkBox_autoRange();
    void update_checkBox_autoFit();
    void setStatus(const QString &s);
    void update_current_main_hist_bh(bool enable_controls);
    void clear_current_main_hist();

private slots:
    void on_cb_hist_activated( int );
    void on_checkBox_autoRange_toggled( bool state );
    void on_checkBox_autoFit_toggled( bool state );
    void on_pbHideAll_clicked();
    void on_pbShow_clicked();
    void on_pbhclearthis_clicked();
    void on_pbhclear_clicked();
    void on_pbhsave_clicked();
    void on_pbhsaveall_clicked();

private:
    HistDialogList hists;

};

#endif // HISTEXPLORERVIEW_H
