
#include <QDir>
#include <QDebug>

#include "RawDataWriter.h"
#include "adchardw.h"
#include "adcm_raw.h"
#include "storage.h"

#define PREFIX "RawDataWriter: "
#define WRITER_DEBUG_LEVEL 1
#define WDL WRITER_DEBUG_LEVEL

RawDataWriter::RawDataWriter(QObject *parent)
    : QObject(parent),
      stor(new storage(this))
{
}

void RawDataWriter::reset()
{
    stor->reset ();
}

void RawDataWriter::endBurst()
{
    stor->flush ();
}

void RawDataWriter::writeFrameList(const FrameList &list)
{
    if (list.shouldDrop()) {
        qDebug() << QString("RawDataWriter dropped %1 frames").arg(list.size());
        return;
    }
    Q_FOREACH(const QByteArray &buf, list)
        stor->write(buf.constData(), buf.size());
}

void RawDataWriter::setStorageConfig(const StorageConfig &r)
{
    storageConfig = r;
    const QString fullFileName = storageConfig.bn + QDir::separator() + storageConfig.fn;
//    qDebug() << QString("filename='%1'").arg(fullFileName);
    stor->setFileName(fullFileName);
    stor->setopt(storageConfig.enable, storageConfig.mode);
    stor->reset();
}
