//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef SETTINGSDIALOG_H
#define SETTINGSDIALOG_H

#include "ui_SettingsDialogUi.h"
#include "settings_data.h"

class conf_mem_t;

class SettingsDialog : public QDialog, public Ui::SettingsDialogUi
{
    Q_OBJECT
public:
    explicit SettingsDialog(const conf_mem_t &i_conf_mem, QWidget *parent = 0);
    void setData( const SettingsData &in );
    SettingsData getData() const;

signals:
    void applied();

public slots:
    virtual void accept();

protected:
    void updateLimits();
    void updateEnabledStatus();
    void apply();
    void restoreDefaults();

protected:
    void writeSettings();
private:
    const conf_mem_t &conf_mem;
    SettingsData data;

private slots:
    void on_buttonBox_clicked(QAbstractButton* button);
    void on_spinBox_samples_base_valueChanged( int base );
    void on_spinBox_samples_S_valueChanged( int );
    void on_spinBox_samples_M_valueChanged( int val );
    void on_spinBox_trig_min_valueChanged( int val );
    void on_spinBox_trig_max_valueChanged( int val );
    void on_doubleSpinBoxTime_valueChanged( double );
    void on_spinBoxEventCount_valueChanged( int );
    void on_checkBoxEventCount_toggled( bool );
    void on_checkBoxTime_toggled( bool );
    void on_groupBoxStopCondition_toggled( bool );
    void on_radioButtonTimestampUTC_toggled( bool );
};

#endif // SETTINGSDIALOG_H
