#ifndef ADCINST_H
#define ADCINST_H

#include <QObject>
#include <QByteArray>
#include "adcm_df.h"

namespace mlink {
class RegIOBase;
}
class AdcmDevice;
class QTimer;
class QUrl;
class FrameFinder;
class FrameList;
class AllConfig;
class conf_mem_t;
class StopWatchTimer;
class AdcHwCounters;

class AdcInstance : public QObject
{
    Q_OBJECT
public:
    enum State {
        UnconnectedState = 0,
        ConnectingState = 2,
        ConnectedState = 3
    };
    explicit AdcInstance(QObject *parent = 0);
    virtual ~AdcInstance();
    AdcmDevice *dev() { return adcmDevice; }
    float getMinGain() const;
    float getMaxGain() const;
    float getGainStep() const;
    bool hasGainControl() const;
    bool hasTriggerControl() const;

    bool waitForConnected(int msecs = 30000);
    State state() const { return p_state; }
    double runtime () const;
    QString errorString() const { return p_errorString; }
    QStringList getDeviceIdStrList() const;

Q_SIGNALS:
    void confMemModified(const conf_mem_t &);
    void error();
//    void bytesReceived(QByteArray);
    void frameReceived(const FrameList &r);
    void connected();
    void disconnected();
    void timeElapsed(double seconds);
    void byteCountChanged(quint64 bytes);
    void hitCountsChanged(adcm_counters_t);
    void tempChanged(QVector<double>);

public Q_SLOTS:
    void connectToDevice(const QUrl &url);
    void startReadout();
    void stopReadout();
    void setup(const AllConfig &cfg);
    void reset();
    void dataProgress(quint64 value);

protected:
    void setErrorString(const QString &str) { p_errorString = str; }
    void emitError(const QString &str);

private:
//    void read_adc_setup_memory(AllConfig *cfg);
    float adc_gain_val_to_db(int value) const;
    quint64 queueSize() const;
    bool isQueueFull() const;
    adcm_counters_t makeAdcmCounters(const AdcHwCounters &adcHwCounters) const;

    float min_gain_db;
    float max_gain_db;
    float gain_step_db;

    mlink::RegIOBase * const adcmLink;
    AdcmDevice * adcmDevice;
    QTimer * const pollTimer;
    QTimer * const hitcountsTimer;
    FrameFinder * const frameFinder;
    quint64 prev_pos;
    quint64 decoder_pos;
    State p_state;
    bool dma_running;

    StopWatchTimer * const runTimer;
    quint64 numBytesReceived;
    adcm_counters_t adcm_counters;
//    QByteArray rawdata;
    QString p_errorString;
    quint64 lastSentSeq;
    quint64 lastReceivedSeq;
    // statistics
    quint64 statsNumBytesReceived;
    double statsTimestamp;
    AdcHwCounters * const prevAdcHwCounters;

private Q_SLOTS:
    void poll();
    void readHitCounts();
    void on_error();
    void on_connected();
    void on_disconnected();
    void updateStatistics();
};

#endif /* ADCINST_H */
