//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#include <cstring>
#include <cmath>

#include <QTimer>
#include <QAbstractTableModel>
#include <QMap>

#include "config/adcconfig.h"
#include "config/calavgconfig.h"
#include "calavgdialog.h"

enum {
    table_col_count = 0,
    table_col_offset = 1,
    table_col_preint = 2,
    table_col_int = 3,
    table_col_postint = 4,
    table_n_cols = 5
};

class CalAvgModel: public QAbstractTableModel
{
public:
    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const;
    void setCustomData(const AdcConfig &adcConfig, const CalAvgConfig &calAvgConfig);
private:
    QMap<int, QMap<int, QString> > d;
    QMap<int, bool> d_enable;
};

int CalAvgModel::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return d[table_col_count].size();
}

int CalAvgModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return table_n_cols;
}

QVariant CalAvgModel::data(const QModelIndex &index, int role) const
{
    if (role == Qt::DisplayRole) {
        return d[index.column()][index.row()];
    }
    return QVariant();
}

QVariant CalAvgModel::headerData(int section, Qt::Orientation orientation,
                                 int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();
    if (orientation == Qt::Horizontal) {
        switch(section) {
        case table_col_count : return "Count";
        case table_col_offset : return "Offset";
        case table_col_preint : return "Pre-Int";
        case table_col_int : return "Integral";
        case table_col_postint : return "Post-Int";
        }
    }
    if (orientation == Qt::Vertical) {
        return QString::number(section);
    }
    return QVariant();
}

void CalAvgModel::setCustomData(const AdcConfig &adcConfig, const CalAvgConfig &calAvgConfig)
{
    QMap<int, QMap<int, QString> > newMap;
    QMap<int, bool> newEnable;
    for(int i=0; i<calAvgConfig.nent.size(); i++) {
        const bool enable = adcConfig.ch_enabled_shape_calibration(i);
        const int canent = calAvgConfig.nent[i];
//        const double base    = cfg.calavg_baseline(i);
//        const double inte    = cfg.calavg_integral(i);
//        const double preint  = cfg.calavg_preintegral(i);
//        const double postint = cfg.calavg_postintegral(i);

        newMap[table_col_count][i] = QString("%1").arg(canent);
//        newMap[table_col_offset][i] = QString("%1").arg(base, 0, 'f', 1);
//        newMap[table_col_int][i] = QString("%1").arg(inte, 0, 'f', 1);
//        newMap[table_col_preint][i] = QString("%1").arg(preint, 0, 'f', 1);
//        newMap[table_col_postint][i] = QString("%1").arg(postint, 0, 'f', 1);
        newEnable[i] = enable;
    }
    if (d != newMap || d_enable != newEnable) {
        d = newMap;
        d_enable = newEnable;
        reset();
    }
}


CalAvgDialog::CalAvgDialog (QWidget * parent)
    : CalibrationDialog(parent),
      d_model(new CalAvgModel)
{
    setupUi(this);
    tableView->setModel(d_model);
}

CalAvgDialog::~CalAvgDialog ()
{
    delete d_model;
}

void CalAvgDialog::update ()
{
    reinterpret_cast<CalAvgModel*>(d_model)->setCustomData(*adcConfig, *calAvg);
}

