//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "calibrationdialog.h"

#include <QTimer>

#include "adcmraw/adcm_mem.h"
#include "config/adcconfig.h"
#include "config/calavgconfig.h"
#include "config/calzeroconfig.h"

CalibrationDialog::CalibrationDialog(QWidget *parent) :
    QDialog(parent),
    conf_mem(new conf_mem_t),
    adcConfig(new AdcConfig),
    calAvg(new CalAvgConfig),
    calZero(new CalZeroConfig),
    pixmap_ok(new QPixmap(":/icons/icon_16x16_ok.png")),
    pixmap_error(new QPixmap(":/icons/icon_16x16_cancel.png"))
{
    QTimer *d_updatetimer(new QTimer(this));
    connect(d_updatetimer, SIGNAL(timeout()), SLOT(update()));
    d_updatetimer->start(300, FALSE);
}

CalibrationDialog::~CalibrationDialog ()
{
    delete conf_mem;
    delete adcConfig;
    delete calZero;
    delete pixmap_ok;
    delete pixmap_error;
}

void CalibrationDialog::setAdcConfMem(const conf_mem_t &r)
{
    *conf_mem = r;
    update();
}

void CalibrationDialog::setAdcConfig(const AdcConfig &r)
{
    *adcConfig = r;
    update();
}

void CalibrationDialog::setCalAvgConfig(const CalAvgConfig &r)
{
    *calAvg = r;
    update();
}

void CalibrationDialog::setCalZeroConfig(const CalZeroConfig &r)
{
    *calZero = r;
    update();
}
