
#include <QStyle>
#include <QCheckBox>
#include <QPainter>
#include <QApplication>
#include <QStyleOption>

#include "cti.h"

MyCheckTableItem::MyCheckTableItem( Q3Table *table )
    : Q3TableItem( table, WhenCurrent ), cb(0), checked( FALSE )
{
}

QWidget *MyCheckTableItem::createEditor() const
{
    // create an editor - a combobox in our case
    ( (MyCheckTableItem*)this )->cb = new QCheckBox( table()->viewport(), "qt_editor_checkbox" );
    cb->setChecked( checked );
    cb->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    cb->setBackgroundColor( table()->viewport()->backgroundColor() );
    QObject::connect( cb, SIGNAL( toggled(bool) ), table(), SLOT( doValueChanged() ) );
    return cb;
}

void MyCheckTableItem::setContentFromEditor( QWidget *w )
{
    QCheckBox *cb = qobject_cast<QCheckBox*>(w);
    if ( cb )
        checked = cb->isChecked();
}

void MyCheckTableItem::paint( QPainter *p, const QColorGroup &cg,
                             const QRect &cr, bool selected )
{
    p->fillRect( 0, 0, cr.width(), cr.height(),
                selected ? cg.brush( QColorGroup::Highlight )
                         : cg.brush( QColorGroup::Base ) );

    //    int w = cr.width();
    //    int h = cr.height();
    QSize sz = QSize( table()->style()->pixelMetric( QStyle::PM_IndicatorWidth ),
                     table()->style()->pixelMetric( QStyle::PM_IndicatorHeight ) );
    QColorGroup c( cg );
    c.setBrush( QColorGroup::Background, c.brush( QColorGroup::Base ) );
    QStyle::State flags = QStyle::State_None;
    if(isEnabled())
        flags |= QStyle::State_Enabled;
    if ( checked )
        flags |= QStyle::State_On;
    else
        flags |= QStyle::State_Off;
    if ( isEnabled() && table()->isEnabled() )
        flags |= QStyle::State_Enabled;

    QStyleOptionButton qso;
    qso.state  = flags;
    qso.rect = QRect( 0, ( cr.height() - sz.height() ) / 2, sz.width(), sz.height() );
    (table()->style())->drawPrimitive( QStyle::PE_IndicatorCheckBox, &qso, p );
    /*
    int x = sz.width() + 6;
    w = w - x;
    if ( selected )
        p->setPen( cg.highlightedText() );
    else
        p->setPen( cg.text() );
    p->drawText( x, 0, w, h, wordWrap() ? ( alignment() | WordBreak ) : alignment(), text() );
*/
}

void MyCheckTableItem::setChecked( bool b )
{
    checked = b;
    table()->updateCell( row(), col() );
    QWidget *w = table()->cellWidget( row(), col() );
    QCheckBox *cb = qobject_cast<QCheckBox*>(w);
    if ( cb )
        cb->setChecked( b );
}

bool MyCheckTableItem::isChecked() const
{
    // #### why was this next line here. It must not be here, as
    // #### people want to call isChecked() from within paintCell()
    // #### and end up in an infinite loop that way
    // table()->updateCell( row(), col() );
    QWidget *w = table()->cellWidget( row(), col() );
    QCheckBox *cb = qobject_cast<QCheckBox*>(w);
    if ( cb )
        return cb->isChecked();
    return checked;
}

int MyCheckTableItem::RTTI = 2;

int MyCheckTableItem::rtti() const
{
    return RTTI;
}
