//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DCCONF_H
#define DCCONF_H

#include <vector>
#include "ui_dcconfBase.h"

#include "config/decoderconfig.h"

class AdcConfig;
//class DecoderConfig;
class conf_mem_t;
class RFFTEngine;

class dcconf : public QDialog, public Ui::dcconfBase
{
    Q_OBJECT
public:
    explicit dcconf(QWidget* parent, const conf_mem_t &i_conf_mem, const AdcConfig &i_adcConfig, const DecoderConfig &i_decoderConfig);
    virtual ~dcconf();

signals:
    void decoderConfigModified(const DecoderConfig &);

private slots:
    void on_comboBoxWindowingFunction_activated( int );
    void on_sbfirnp_valueChanged( int );
    void on_sbsnf_valueChanged( int ); // Noise floor of source
    void on_sbrnf_valueChanged( int ); // Noise floor of target
    void on_sbput_valueChanged( int );
    void on_sbspumd_valueChanged( int  );
    void on_sbi1_valueChanged( int ); // Start of signal's interval
    void on_sbi2_valueChanged( int ); // End of signal's interval
    void on_sbblr1_valueChanged( int ); // Start of baseline's interval
    void on_sbblr2_valueChanged( int ); // End of baseline's interval
    void on_sbpos_valueChanged(int);
    void on_sbwid_valueChanged(int);
    void on_spinBoxChannel_valueChanged ( int inch);
    void on_sbexp_valueChanged( int );
    void on_sbblrmd_valueChanged( int );
    void on_pushButtonLoadShape_clicked();
    void on_pushButtonSaveShape_clicked();
    void on_pushButtonGenerateFir_clicked();
    void on_pushButtonSaveFir_clicked();
    void on_pushButtonLoadFir_clicked();
    void on_buttonBox_clicked(QAbstractButton* button);

protected:
    void setButtons ();
    bool modified() const;
    void apply();
    void restoreDefaults();
    void replotINP();
    void replotINPFFT();
    void replotFIR();
    void replotFIRFFT();
    void replotTEST();
    void Restore();
    void Redraw();
    double reqf( int i ) const;
    double reqfunscaled( int i ) const;
    double ca( unsigned int ch, int i ) const;
    double getTimeStepInUs(unsigned int ch_num) const;
    double getTimeStepInNs(int ch_num) const;
    int data_latency() const;
    std::vector<double> getCalAvgY(int channel) const;
    std::vector<double> getCalAvgXInUs(int channel) const;
    std::vector<double> getRefY(int channel, double amplitudeScaler) const;
    std::vector<double> getRefY(int channel) const;
    std::vector<double> getRefXInUs(int channel) const;
    double getCalAvgAmplitude(int channel) const;

private:
    dcconfBase * const ui;
    int ch;
    int restoring;
    int win_type;
    int disable_control_slots;
    const conf_mem_t &conf_mem;
    const AdcConfig &adcConfig;
    DecoderConfig currentDecoderConfig;
    DecoderConfig newDecoderConfig;
    RFFTEngine * const fft;
};

#endif // DCCONF_H
