#ifndef HISTMGR_H
#define HISTMGR_H

#include <QString>
#include <QMap>
#include <QSharedPointer>
#include <QMetaType>
#include <QMutex>
#include "config/allconfig.h"
#include "th1.h"

//class th1;
class AllConfig;
typedef QSharedPointer<th1> th1Ptr;
class HistList: public QMap<int, th1Ptr> {};

Q_DECLARE_METATYPE(HistList)

class AdcmEvent;
class conf_mem_t;

class HistManager: public QObject
{
    Q_OBJECT
public:
    explicit HistManager(QObject *parent);
    ~HistManager();
    int fill (const AdcmEvent &ev);
    int fillEvDt (const AdcmEvent &ev, quint64 prevEvTs);
    int fill_status (const struct adcm_decstat_t &st);
//    const HistList &getHistList () const {return hl;}
    void print() const;
    HistList getHistList() { QMutexLocker locker(mutex); return hl; }
    void addHist(HistManager *hmgr);
    void lock();
    void unlock();

Q_SIGNALS:
    void histListModified(const HistList &);

public Q_SLOTS:
    void setAdcConfig(const AdcConfig &r);
    void setAdcConfMem(const conf_mem_t &confMem);
    void clear (int index=-1);

protected:
    void create_histograms();
    void delete_histograms();
    //  th1Ptr findHist
    int hist_nch;

private:

    bool get_hist_enable(int index) const;
    int nch() const;

    HistList hl;
    int h_dT_index; // Time Ts-Tm fine
    int h_dTmax_index; // Time Ts-Tm coarse
    int h_Tmr_index; // Rec. quality Tm
    int h_Tsr_index; // Rec. quality Ts
    int h_Ts_index; //Time Ts
    int h_wm_index; // Filt. pulse width Wm
    int h_ws_index; // Filt. pulse width Ws
    int h_slideev_index; // Raw pulses / event
    int h_pulsev_index; // Fitted pulses / event
    int h_niter_index; // Fitter iterations / event
    int h_evdist_index; // Channel time, rel. Spill
    int h_decodetime_index; // Event decode time
    int h_T_index; // Ch Time rel. Master
    int h_Int_index; // Integral
    int h_bl_index; // BaseLine
    int h_dT_Ev_index; // Time Ev(i)-Ev(i-1)
    int h_n_total;
    AdcConfig adcConfig;
    conf_mem_t * const conf_mem;
    QMutex *mutex; // Blocks fill, fillEvDt, clear
};

#endif /* HISTMGR_H */
