
#include <qwt_math.h>
#include <qwt_scale_engine.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_plot_marker.h>

#include "histplot.h"
#include "histplot_p.h"

const int znp=2;
const double zpx[znp] = {0,1};
const double zpy[znp] = {0,0};

HistPlot::HistPlot (QWidget * p)
    : QwtPlot (p), d(new HistPlotPrivate)
{
    //  setAutoLegend (FALSE);

    // grid
    d->grid.attach(this);
    d->grid.setMajPen(QPen(Qt::black, 0, Qt::DotLine));

    d->curv.attach(this);
    d->curv.setTitle("Data");
    d->curv.setPen (QPen (Qt::black, 2));
    d->curv.setStyle (QwtPlotCurve::Steps);
    d->curv.setBrush (Qt::blue);
    //  curv->setSymbol(QwtSymbol(QwtSymbol::XCross, black, black, QSize(2,2)));


    // fit function
    d->curv_f.attach(this);
    d->curv_f.setTitle("Fit");
    d->curv_f.setPen (QPen (Qt::black, 1));
    d->curv_f.setStyle (QwtPlotCurve::Lines);
    //	curv_f->setCurveFitter(QwtSplineCurveFitter);

    //  Insert zero line at y = 0
    d->mark_y0.attach(this);
    d->mark_y0.setYValue(0.0);
    d->mark_y0.setLineStyle(QwtPlotMarker::HLine);
    d->mark_y0.setLabelAlignment(Qt::AlignRight | Qt::AlignBottom);
    d->mark_y0.setLinePen(QPen(Qt::green, 0, Qt::DashDotLine));

    //  long mY = insertLineMarker ("", QwtPlot::yLeft);
    //  setMarkerYPos (mY, 0.0);

    // Set axis titles
    //    setAxisTitle(QwtPlot::xBottom, "Time/seconds");
    //    setAxisTitle(QwtPlot::yLeft, "Values");

    //    setAxisOptions(QwtPlot::yLeft, QwtAutoScale::Floating | QwtAutoScale::Logarithmic);

}

HistPlot::~HistPlot()
{
    d->grid.detach();
    d->curv.detach();
    d->curv_f.detach();
    d->mark_y0.detach();
    delete d;
}

void HistPlot::setpoints (const std::vector<double> &inx, const std::vector<double> &iny)
{
    const size_t len = std::min(inx.size(), iny.size());
    d->px = inx;
    d->py = iny;
#if QWT_VERSION >= 0x060000
    d->curv.setSamples (&inx[0], &iny[0], len); // qwt 6
#else
    d->curv.setData (&inx[0], &iny[0], len); // qwt 5
#endif
}

void HistPlot::set_fit_points (const std::vector<double> &inx, const std::vector<double> &iny)
{
    const size_t len = std::min(inx.size(), iny.size());
    d->fpx = inx;
    d->fpy = iny;
#if QWT_VERSION >= 0x060000
    d->curv.setSamples (&inx[0], &iny[0], len); // qwt 6
#else
    d->curv.setData (&inx[0], &iny[0], len); // qwt 5
#endif
}

void HistPlot::clear_points()
{
    setpoints(std::vector<double>(), std::vector<double>());
}

void HistPlot::clear_fit_points()
{
    set_fit_points(std::vector<double>(), std::vector<double>());
}

std::vector<double> HistPlot::getpointsx() const {return d->px;}
std::vector<double> HistPlot::getpointsy() const {return d->py;}
std::vector<double> HistPlot::getfitpointsx() const {return d->fpx;}
std::vector<double> HistPlot::getfitpointsy() const {return d->fpy;}
int HistPlot::getpointslen() const {return std::min(d->px.size(), d->py.size());}
int HistPlot::getfitpointslen() const {return std::min(d->fpx.size(), d->fpy.size());}

/*
void HistPlot::autorange() {

  if (!px||!py||(len<=0)) return;

  double maxvalue = 0;
  for (int i = 0; i < len; i++) {
    if (py[i] > maxvalue) maxvalue = py[i];
  }

  double threshold = maxvalue * 0.025;
  if (threshold < 2) threshold = 2;

    double pxmin = px[0], pxmax = px[len-1];
    for (int i = 1; i < len; i++)
      if (py[i] > threshold) {
        pxmin = px[i-1];
        break;
      }
    for (int i = len - 3; i > 0; i--)
      if (py[i] > threshold) {
        pxmax = px[i+2];
        break;
      }
    if (pxmin<pxmax) {
          ar_xmin = pxmin;
          ar_xmax = pxmax;
      setAxisScale (QwtPlot::xBottom, pxmin, pxmax, 0);
        }
}
*/

void HistPlot::set_x_range(double xmin, double xmax) {
    setAxisScale (QwtPlot::xBottom, xmin, xmax, 0);
}
/*
void HistPlot::autorange() {
  double ar_min=0, ar_max=0;
  h->autorange();
  h->getrange(&ar_min, &ar_max);
  set_x_range(ar_min, ar_max);
}
*/
