
#include <cstdlib>
#include <iostream>
#include <exception>
#include <stdexcept>

#include <QApplication>
#include <QMessageBox>
#include <QTextCodec>
#include <QSplashScreen>
#include <QtCore>

#include "mlink/exc.h"
#include "mainwindow.h"
#include "qlog/qlog.h"
#include "mlink/colorlog/log.h"

namespace { const char rcs_id[] = "$Id: main.cpp 15864 2013-11-14 10:22:11Z noafi $"; }


int main( int argc, char ** argv )
{
    logger.setDebugLevel(DEBUG_STARTUP_DEBUG_LEVEL);
    colorlog::set_debug_level(DEBUG_STARTUP_DEBUG_LEVEL);

    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("utf8"));
    qDebug(QString("Starting %1").arg(argv[0]));
    //    qInstallMsgHandler(myMessageOutput);

    QApplication a( argc, argv );
    QSplashScreen splash(QPixmap(":/icons/adcm-splash.png"));
    splash.show();
    splash.showMessage(QString("Loading translations"));
    a.processEvents();
    QCoreApplication::setOrganizationName("AFI Electronics");
    QCoreApplication::setOrganizationDomain("local");
    QCoreApplication::setApplicationName("ADCM");

    // translation file for Qt
    QTranslator qt( 0 );
    qt.load( QString( "qt_" ) + QTextCodec::locale(), "." );
    a.installTranslator( &qt );


    // translation file for application strings
    QTranslator myapp( 0 );
    myapp.load( QString( "adcm_" ) + QTextCodec::locale(), "." );
    a.installTranslator( &myapp );

    MainWindow *w = NULL;
    try {
        splash.showMessage(qApp->tr("Starting threads"));
        a.processEvents();
        w = new MainWindow;
        splash.showMessage(qApp->tr("Starting GUI"));
        a.processEvents();
        w->initialize(argc, argv);
        a.setMainWidget(w);
        w->show();
        splash.finish(w);
        w->initialize_gui(argc, argv);
        a.connect( &a, SIGNAL( lastWindowClosed() ), &a, SLOT( quit() ) );
        int ret = a.exec();
        delete w;
        return ret;
    }
    catch (mlink::exc &e) {
        QString str = QString("Application error: ")+"exception " + QString::fromLocal8Bit(e.what());
        qCritical() << str;
        QMessageBox::critical(w, "Error", str, QMessageBox::Cancel,  Qt::NoButton);
        return EXIT_FAILURE;
    }
    catch (std::runtime_error &e) {
        QString str = QString("Application error: ")+"std::runtime_error " + QString::fromLocal8Bit(e.what());
        qCritical() << str;
        QMessageBox::critical(w, "Error", str, QMessageBox::Cancel,  Qt::NoButton);
        return EXIT_FAILURE;
    }
    catch (std::exception &e) {
        QString str = QString("Application error: ")+"std::exception " + QString::fromLocal8Bit(e.what());
        qCritical() << str;
        QMessageBox::critical(w, "Error", str, QMessageBox::Cancel,  Qt::NoButton);
        return EXIT_FAILURE;
    }
}
