//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <getopt.h>

#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <cmath>
#include <iostream>
#include <stdexcept>

#include <QAction>
#include <QApplication>
#include <QCheckBox>
#include <QComboBox>
#include <QDebug>
#include <QDesktopServices>
#include <QFileDialog>
#include <q3listbox.h> // FIXME
#include <QList>
#include <QMainWindow>
#include <QMessageBox>
#include <QMutex>
#include <QProgressDialog>
#include <QPushButton>
#include <QRadioButton>
#include <QSettings>
#include <QSpinBox>
#include <QStatusBar>
#include <QTabWidget>
#include <QTextStream>
#include <QTimer>
#include <QToolTip>
#include <QThread>
#include <QValidator>

#include <qwt_scale_engine.h>

#include "regio/regiobase.h"
#include "mlink/regiomlink.h"

#include "ui_mainwindowbase.h"
#include "mainwindow.h"
#include "oemstrings.h"
#include "version.h"
//#include "rawio.h"
#include "adchardw.h"
#include "adcm_raw.h"
#include "adcm_mem.h"

#include "bitop.h"
#include "fittersettingsdialog.h"
#include "dcconf.h"
#include "runstats.h"
#include "dsplib/dsplib.h"
#include "dsplib/fir.h"
#include "SetupDataOutputDialog.h"
#include "TrigSetupDialog.h"
#include "SettingsDialog.h"
#include "StatusMessageBox.h"
#include "cti.h"
#include "settings_data.h"

#include "adcm_dec.h"
#include "adcm_dws.h"
#include "AdcmChbuf.h" // adcm_ev_t
#include "adcm_ev.h"
#include "allstats.h"
#include "ch_label.h"
#include "adcinstance.h"


//#include "regioemu.h"
#include "adcmdevice.h"

#include "sysmon.h"
#include "reports.h"
#include "util.h"
#include "sysinfo.h"

#include "calavgdialog.h"
#include "calzerodialog.h"
#include "StatusMessageBox.h"
#include "DecodeManager.h"
#include "EventListFiller.h"
#include "RawDataWriter.h"

#include "th1.h"
#include "runstats.h"
#include "daq_ipc.h"

#include "qutil.h"
//#include "mymutexlocker.h"

#include "runcontroladcm.h"
#include "waveformmodeladc.h"

#include "RunControlWidget.h"
//#include "qxw/WaveformView.h"
//#include "qxw/WaveformViewAdvanced.h"
#include "WaveformViewAdcm.h"
#include "HistExplorerView.h"

Q_DECLARE_METATYPE(adcm_counters_t)


#ifdef DEBUG
# undef MAINDEBUG
# define MAINDEBUG 1
#endif

#ifndef MAINDEBUG
# define MAINDEBUG 0            /* dummy */
#endif

#define PREFIX "main: "
static const int MAINWINDOW_DEBUG_LEVEL = 1;
static const int MWDL = MAINWINDOW_DEBUG_LEVEL; /* shortcut */

const double autosave_time = 5.0;
const double DMA_CYCLE_TIME = 10.0;   // seconds
const double DMA_RESTART_RATIO = 0.9;
const double update_display_time = 0.333;
//const double update_scope_time = 0.5;
const double update_livestat_time = 1;
const double update_histograms_time = 0.7; //TODO remove it
const double updateBaseStatTime = 1;
const double simulate_event_time = 1.0;

int disable_cfg_slots = 0;          // when setting GUI boxes

//quint64 save_decst_nw = 0;

enum { CONVLEN = ADCM_MAX_N_SAMP + FIRMAXLEN };

// Deprecated
//// Persistant decoder data
//const int save_ec_default = 0x5A5A5A5A;
//unsigned int save_ec = save_ec_default; // saved event number for display update

bool display_average_state = false;

unsigned long force_bufsize = 0;

enum {
    table_cc_col_En=0,
    table_cc_col_Zero=1,
    table_cc_col_Inv=2,
    table_cc_col_M=3,
    table_cc_col_S=4,
    table_cc_col_thr=5,
    table_cc_col_gain=6,
    table_cc_col_rate=7,
    table_cc_col_title=8,
    table_cc_col_number=9
};

class MainWindowPrivate
{
public:
    conf_mem_t adcConfMem;
    AllConfig cfg;
};


MainWindow::MainWindow(QWidget* parent)
    : QMainWindow(parent),
      idLabel(new QLabel(this)),
      temperatureLabel(new QLabel(this)),
      temperatureOverflow(0),
      defaultConfigFileName("adcm_config.ini"),
      defaultConfigDir(QDesktopServices::storageLocation(QDesktopServices::HomeLocation)),
      waveformModel_ADC(new WaveformModel_ADC(this)),
      timerAutoSave(new QTimer (this)),
      timerBaseStatistics(new QTimer (this)),
      timerRawWriter(new QTimer(this)),
      timerEventWriter(new QTimer(this)),
      timer_sim(new QTimer (this)),
      timerUpdateDisplay(new QTimer (this)),
      timerUpdateHistograms(new QTimer (this)), //TODO remove it
      timerUpdateLiveStatistics(new QTimer (this)),
      savedcfg(new AllConfig),
      lastAutoSaveConfig(new AllConfig),
      calAvgDialog(new CalAvgDialog(this)),
      calZeroDialog(new CalZeroDialog(this)),
//      allstats(new (struct allstats_t)),
      status_mbox(new StatusMessageBox(this)),
      adcInstance(new AdcInstance(NULL)),
      decodeManager(new DecodeManager(this)),
      filler(decodeManager->createFiller()),
      writeInstance(new RawDataWriter (NULL)),
      adcThread(new QThread(this)),
      decodethread(new QThread(this)),
      fillerThread(new QThread(this)),
      writethread(new QThread(this)),
      eventsProceed(0),
      runSeconds(0),
      runBytes(0),
      prev_mb(0),
      prev_tv(QTime::currentTime()),
      autoSaveLoaded(false),
      d_ptr(new MainWindowPrivate())
{
    setObjectName("main GUI thread");
    setupUi(this); // setupUi calls connectSlotsByName()
    statusBar()->addPermanentWidget(idLabel, 9);
    statusBar()->addPermanentWidget(temperatureLabel, 1);

    if (!DECODER_PROFILING)
        tabWidget2->removePage(tabPerformance);

    connectSignals();
}

MainWindow::~MainWindow()
{
    autoSave();
    stopThreads();
    delete decodethread;
    delete fillerThread;
    delete writethread;
    delete adcThread;
//    delete allstats;
    delete lastAutoSaveConfig;
    delete savedcfg;
    delete d_ptr;
}

void MainWindow::connectSignals()
{
    if (DECODER_PROFILING)
        connect(this, SIGNAL(performanceInfoModified(QString)), textBrowserPerformance, SLOT(setText(QString)));

    qRegisterMetaType<AllConfig>();
    qRegisterMetaType<AdcConfig>();
    qRegisterMetaType<DecoderConfig>();
    qRegisterMetaType<StorageConfig>();

    connect(adcInstance, SIGNAL(confMemModified(conf_mem_t)), this, SLOT(setAdcConfMem(conf_mem_t)));
    connect(adcInstance, SIGNAL(confMemModified(conf_mem_t)), filler, SLOT(setAdcConfMem(conf_mem_t)));
    connect(adcInstance, SIGNAL(confMemModified(conf_mem_t)), calZeroDialog, SLOT(setAdcConfMem(conf_mem_t)));
    connect(adcInstance, SIGNAL(confMemModified(conf_mem_t)), calAvgDialog, SLOT(setAdcConfMem(conf_mem_t)));

    connect(this, SIGNAL(adcConfigModified(AdcConfig)), this, SLOT(setAdcConfig(AdcConfig)));
    connect(this, SIGNAL(adcConfigModified(AdcConfig)), decodeManager, SLOT(setAdcConfig(AdcConfig)));
    connect(this, SIGNAL(adcConfigModified(AdcConfig)), calZeroDialog, SLOT(setAdcConfig(AdcConfig)));
    connect(this, SIGNAL(adcConfigModified(AdcConfig)), calAvgDialog, SLOT(setAdcConfig(AdcConfig)));

    connect(this, SIGNAL(decoderConfigModified(DecoderConfig)), this, SLOT(setDecoderConfig(DecoderConfig)));
    connect(this, SIGNAL(decoderConfigModified(DecoderConfig)), decodeManager, SLOT(setDecoderConfig(DecoderConfig)));

    connect(this, SIGNAL(evStorageConfigModified(StorageConfig)), decodeManager, SLOT(setStorageConfig(StorageConfig)));
    connect(this, SIGNAL(rawStorageConfigModified(StorageConfig)), writeInstance, SLOT(setStorageConfig(StorageConfig)));
    connect(this, SIGNAL(evStorageConfigModified(StorageConfig)), SLOT(updateEventWriterTimer(StorageConfig)));
    connect(this, SIGNAL(rawStorageConfigModified(StorageConfig)), SLOT(updateRawWriterTimer(StorageConfig)));
    connect(timerRawWriter, SIGNAL(timeout()), writeInstance, SLOT(endBurst()));
    connect(timerEventWriter, SIGNAL(timeout()), decodeManager, SLOT(endBurst()));

    connect(checkBoxHistogramFilling, SIGNAL(toggled(bool)), decodeManager, SLOT(setHistEnabled(bool)));

//    connect(this, SIGNAL(configModified(AllConfig)), writeInstance, SLOT(configModified(AllConfig)));
//    connect(this, SIGNAL(configModified(AllConfig)), SLOT(slot_configModified(AllConfig)));

    connect(status_mbox, SIGNAL(accepted()), this, SLOT(status_mbox_accept()));
    connect(status_mbox, SIGNAL(rejected()), this, SLOT(status_mbox_reject()));

    qRegisterMetaType<FrameList>();
    connect(adcInstance, SIGNAL(frameReceived(FrameList)), filler, SLOT(appendFrameList(FrameList)));
    connect(filler, SIGNAL(completed(quint64)), adcInstance, SLOT(dataProgress(quint64)));
    connect(adcInstance, SIGNAL(frameReceived(FrameList)), writeInstance, SLOT(writeFrameList(FrameList)));
    qRegisterMetaType<adcm_counters_t>();
    connect(adcInstance, SIGNAL(hitCountsChanged(adcm_counters_t)), decodeManager, SLOT(appendChannelCounters(adcm_counters_t)));
    connect(adcInstance, SIGNAL(hitCountsChanged(adcm_counters_t)), this, SLOT(onHitCountsChanged(adcm_counters_t)));
    connect(adcInstance, SIGNAL(connected()), this, SLOT(onDeviceConnected()));
    connect(adcInstance, SIGNAL(disconnected()), this, SLOT(onDeviceDisconnected()));
    connect(adcInstance, SIGNAL(timeElapsed(double)), this, SLOT(updateRuntime(double)));


    connect(adcInstance, SIGNAL(byteCountChanged(quint64)), this, SLOT(updateRunBytes(quint64)));
    connect(adcInstance, SIGNAL(tempChanged(QVector<double>)), this, SLOT(updateTemperature(QVector<double>)));

    qRegisterMetaType<CalAvgConfig>();
    connect(this, SIGNAL(calAvgConfigModified(CalAvgConfig)), calAvgDialog, SLOT(setCalAvgConfig(CalAvgConfig)));

    qRegisterMetaType<CalZeroConfig>();
    connect(this, SIGNAL(calZeroConfigModified(CalZeroConfig)), calZeroDialog, SLOT(setCalZeroConfig(CalZeroConfig)));

    connect(filler, SIGNAL(progressChanged(double)), SLOT(decoderProgress_changed(double)));
    connect(filler, SIGNAL(eventsProceedUpdated(quint64)), SLOT(eventsProceed_changed(quint64)));
    qRegisterMetaType<adcm_decstat_t>();
    connect(decodeManager, SIGNAL(summedStatisticsUpdated(adcm_decstat_t)), SLOT(onDecoderStatisticsUpdated(adcm_decstat_t)));
    qRegisterMetaType<HistList>();
    connect(decodeManager, SIGNAL(histListModified(HistList)), SLOT(onHistListModified(HistList)));

    // Run Control connections
    //connect(this, SIGNAL(transitionTaskComplete(daq_state_t)), &runControl, SLOT(transitionTaskComplete(daq_state_t))); //emit was commented
    connect(RunControlWidget1, SIGNAL(changeStateRequest(daq_state_t)), &runControl, SLOT(changeStateForced(daq_state_t)));
    connect(RunControlWidget1, SIGNAL(powerOff()), SLOT(powerOff()));
    connect(&runControl, SIGNAL(transitionTask(daq_state_t)), SLOT(transitionTask(daq_state_t)));
    connect(&runControl, SIGNAL(stateChanged(daq_state_t)), RunControlWidget1, SLOT(setState(daq_state_t)));
    connect(&runControl, SIGNAL(stateChangeEnabled(bool)), RunControlWidget1, SLOT(setEnabled(bool)));

    connect(histExplorerView, SIGNAL(clearHist(int)), decodeManager, SLOT(clearHist(int)));
}


void MainWindow::addCalZero(const CalZeroConfig &cal)
{
    Q_D(const MainWindow);
    CalZeroConfig newCalZero(d->cfg.dec.calzero);
    newCalZero += cal;
    setCalZeroConfig(newCalZero);
}

void MainWindow::addCalAvg(const CalAvgConfig &cal)
{
    Q_D(const MainWindow);
    CalAvgConfig newConfig(d->cfg.dec.calavg);
    newConfig += cal;
    setCalAvgConfig(newConfig);
}

void MainWindow::initialize(int argc, char **argv)
{
    parse_cmdline(argc, argv);

    if (!autoSaveLoaded) {
        loadAutoSave();
        autoSaveLoaded = true;
    }

    QFileInfo fi("/mnt/live/linuxrc");
    const bool is_live = fi.exists();
    RunControlWidget1->enablePowerOff(is_live);
    //  tlStatus->hide();
    //  tlSA->hide();
    //adjustSize();
    QTimer::singleShot(0, this, SLOT(startThreads()));
}

void MainWindow::loadAutoSave()
{
    bool configLoaded = false;
    if (!initialConfigFileName.isEmpty()) {
        if(QFile(initialConfigFileName).exists()) {
            // Load from initial config file
            QSettings settings(initialConfigFileName, QSettings::IniFormat);
            if (settings.status() == QSettings::NoError) {
                AllConfig tmpConfig;
                tmpConfig.readFromIni(&settings);
                if (!tmpConfig.isValid()) {
                    qCritical() << QString("Invalid config file '%1'")
                                   .arg(initialConfigFileName);
                } else {
                    readGuiSettings(initialConfigFileName);
                    *savedcfg = tmpConfig;
                    setAllConfig(tmpConfig);
                    autoSave();
                    configLoaded = true;
                    qDebug() << QString("Loaded specified config '%1'").arg(initialConfigFileName);
                }
            } else {
                qCritical() << QString("Can't read specified config file '%1'")
                               .arg(initialConfigFileName);
            }
        } else {
            qCritical() << QString("Can't find specified config file '%1'")
                           .arg(initialConfigFileName);
        }
    }
    if (!configLoaded) {
        // Load from autosave
        QSettings settings;
        AllConfig tmpConfig;
        tmpConfig.readFromIni(&settings);
        if (!tmpConfig.isValid()) {
            qWarning() << QString("Invalid autosave config");
        } else {
            *lastAutoSaveConfig = tmpConfig;
            setAllConfig(tmpConfig);
            configLoaded = true;
            qDebug() << QString("Loaded autosave config");
        }
        readGuiSettings();
    }
}

void MainWindow::initialize_gui(int argc, char *argv[])
{
//    Q_D(const MainWindow);
    clearStats ();
    disable_cfg_slots = 0;

//    config_modified(); // force re-read config.xml
    // set startup defaults

//    Q_ASSERT(d->cfg.isValid());

//    {
//        int wst1 = decodethread->get_req_seq();
//        int wst2=wst1;
//        //        qDebug() << QString("Starting decodethread (seq=%1)").arg(wst1);
//        decodethread->start ();
//        //        qDebug() << "Waiting decodethread to start...";
//        while(wst1 == (wst2=decodethread->get_req_seq())) {
//            qApp->processEvents();
//        }
//        //        qDebug() << QString("decodethread started (seq=%1)").arg(wst2);
//    }
//    {
//        int wst1 = writethread->get_req_seq();
//        int wst2=wst1;
//        //        qDebug() << QString("Starting writethread (seq=%1)").arg(wst1);
//        writethread->start ();
//        //        qDebug() << "Waiting writethread to start...";
//        while (wst1==(wst2=writethread->get_req_seq())) {
//            qApp->processEvents();
//        }
//        //        qDebug() << QString("writethread started (seq=%1)").arg(wst2);
//    }
    decodethread->start();
    writethread->start();
//    qDebug() << "Threads started";

    // update now
    //  update_mainhist_view();

    connect (timerAutoSave, SIGNAL (timeout ()), SLOT (autoSave ()));
    timerAutoSave->start ((int)(autosave_time*1e3), FALSE);

    connect (timerBaseStatistics, SIGNAL (timeout ()), SLOT (updateBaseStats ()));
    timerBaseStatistics->start ((int)(updateBaseStatTime*1e3), FALSE);

    connect (timerUpdateDisplay, SIGNAL (timeout ()), SLOT (update_display ()));
    timerUpdateDisplay->start ((int)(update_display_time*1e3));

    //    connect (timerUpdateScope, SIGNAL (timeout ()), SLOT (update_scope_event ()));
    //    timerUpdateScope->setSingleShot(true);
    //    timerUpdateScope->start ((int)(update_scope_time*1e3));

    connect (timer_sim, SIGNAL (timeout ()), SLOT (simulate_event ()));
    timer_sim->start ((int)(simulate_event_time*1e3), FALSE);

    connect (timerUpdateHistograms, SIGNAL (timeout ()), SLOT (update_mainhist_view ())); //TODO remove it
    timerUpdateHistograms->start ((int)(update_histograms_time*1e3)); //TODO remove it

    connect (timerUpdateLiveStatistics, SIGNAL (timeout ()), SLOT (display_refresh_live_statistics ()));
    timerUpdateLiveStatistics->start ((int)(update_livestat_time*1e3));

//    qDebug() << "Timers started";
    //  recreateHistograms ();
    //    qDebug() << "Histograms initialized";
//    display_refresh_run_statistics ();

    waveformViewAdcm->setModel(waveformModel_ADC);

    //    qDebug() << "MainWindow init complete";

    checkBoxWaveformEnableAverage->setEnabled(false);

    QSettings s;
    bool val = s.value("histogramFilling", true).toBool();
    checkBoxHistogramFilling->setChecked(val);
    decodeManager->setHistEnabled(val);

//    Q_ASSERT(d->cfg.isValid());
}

void MainWindow::status (const QString &str)
{
    QStatusBar *bar = statusBar ();
    if (!bar)
        return;
    bar->message (str);
}

QString MainWindow::getSaveFilename ()
{
    if (lastSaveFileName.isEmpty())
        lastSaveFileName = defaultConfigDir + QDir::separator() + defaultConfigFileName;

    const QString &newConfigFilePath = QFileDialog::getSaveFileName (
                this, "Save setup",
                lastSaveFileName, "INI files (*.ini);;All files (*)");

    if (!newConfigFilePath.isEmpty()) {
        QDir dir;
        lastSaveFileName = dir.filePath(newConfigFilePath);
    }
    return newConfigFilePath;
}

bool MainWindow::saveConfig(const QString &fileName)
{
    Q_D(const MainWindow);
    QSettings s(fileName, QSettings::IniFormat);
    s.remove(d->cfg.sectionName());
    d->cfg.writeToIni(&s);
    s.sync();
    if (s.status() != QSettings::NoError) {
        return false;
    }
    writeGuiSettings(fileName);
    *savedcfg = d->cfg;
    updateWindowTitle();
    return true;
}

bool MainWindow::isModified() const
{
    Q_D(const MainWindow);
    return !(d->cfg == *savedcfg);
}

void MainWindow::autoSave ()
{
    Q_D(const MainWindow);
    if (!autoSaveLoaded) return;
    writeGuiSettings();

    QSettings settings;
    if (!settings.value("autosave").toBool()) return;
    if (d->cfg == *lastAutoSaveConfig) return;

    settings.remove(d->cfg.sectionName());
    d->cfg.writeToIni(&settings);
    settings.sync();
    if (settings.status() != QSettings::NoError) {
        qWarning() << "AutoSave: Can't save settings";
    } else {
        *lastAutoSaveConfig = d->cfg;
    }

    if (false) { //test block
        AllConfig testCfg;
        testCfg.readFromIni(&settings);
        if(d->cfg != testCfg)
        {
            if(d->cfg.adc != testCfg.adc)
                qWarning("AdcConfig write/read test failed");
            //        const CalAvgConfig &v1 = d->cfg.dec.calavg;
            //        const CalAvgConfig &v2 = testCfg.dec.calavg;
            //        Q_ASSERT(v1 == v2); // may fail due to floating point comparison
            //        Q_ASSERT(d->cfg.dec.calavg == testCfg.dec.calavg);
            //        Q_ASSERT(d->cfg.dec.calzero == testCfg.dec.calzero);
            if(d->cfg.dec.fir != testCfg.dec.fir)
                qWarning("FIR digital filters write/read test failed");
            //        Q_ASSERT(d->cfg.dec == testCfg.dec);
            if(d->cfg.hist != testCfg.hist)
                qWarning("HistConfig write/read test failed");
            if(d->cfg.run != testCfg.run)
                qWarning("RunConfig write/read test failed");
            if(d->cfg.stor_raw != testCfg.stor_raw)
                qWarning("Raw StorageConfig write/read test failed");
            if(d->cfg.stor_dev != testCfg.stor_dev)
                qWarning("Dev StorageConfig write/read test failed");
            //        Q_ASSERT(d->cfg == testCfg);
            qWarning("AllConfig write/read test failed");
        }
    }
}

void MainWindow::doSaveWork()
{
    if (lastSaveFileName.isEmpty()) {
        QString s = getSaveFilename ();
        if (s.isEmpty())
            return;
        lastSaveFileName = s;
    }
    if (!saveConfig(lastSaveFileName)) {
        QMessageBox::warning (this, "Error writing file", "The file cannot be written", QMessageBox::Cancel, Qt::NoButton);
    }
}
///////////////////////// Start
//Public Slots
void MainWindow::startThreads()
{
    decodeManager->moveToThread(decodethread);
    decodethread->start();
    decodethread->setPriority(QThread::LowestPriority);

    QMetaObject::invokeMethod(decodeManager, "loop", Qt::QueuedConnection);

    {
        Q_D(const MainWindow);
        //presets AdcConfig
        decodeManager->setAdcConfig(d->cfg.adc);
//        calZeroDialog->setAdcConfig(d->cfg.adc);
//        calAvgDialog->setAdcConfig(d->cfg.adc);
        //presets AdcConfig
        filler->setAdcConfMem(d->adcConfMem);
//        calZeroDialog->setAdcConfMem(d->adcConfMem);
//        calAvgDialog->setAdcConfMem(d->adcConfMem);
    }

    filler->moveToThread(fillerThread);
    fillerThread->start();
    fillerThread->setPriority(QThread::LowPriority);

    writeInstance->moveToThread(writethread);
    writethread->start();
    writethread->setPriority(QThread::LowPriority);

    adcInstance->moveToThread(adcThread);
    adcThread->start();
    adcThread->setPriority(QThread::HighPriority);

    int waitMs = 10000;
    QProgressDialog progress(this);
    progress.setLabelText("Starting threads");
    progress.setMaximum(waitMs/1000);
    progress.show();
    QDateTime start(QDateTime::currentDateTime());
    while(!adcThread->isRunning() || !decodethread->isRunning() || !writethread->isRunning()) {
        QDateTime now(QDateTime::currentDateTime());
        int elapsed = now.toTime_t() - start.toTime_t();
        if (progress.wasCanceled() || elapsed > waitMs/1000) {
            qFatal("Timeout starting threads");
            return;
        }
        progress.setValue(elapsed);
        QApplication::processEvents(QEventLoop::AllEvents, 100);
    }
    progress.close();

    Q_D(MainWindow);
    setAllConfig(d->cfg);

    runControl.readSettings();
    runControl.enable();
    runControl.changeState(ST_INIT);
}

void MainWindow::stopThreads ()
{
    qDebug() << "MainWindow::stopThreads";
    stopall();

    adcThread->quit(); // ask to exit from run()
    if (!adcThread->wait(1000))
        qWarning("adcThread aborted");

    writethread->quit(); // ask to exit from run()
    if (!writethread->wait(1000))
        qWarning("writethread aborted");

    decodethread->quit(); // ask to exit from run()
    if (!decodethread->wait(2000))
        qWarning("decodthread aborted");

    fillerThread->quit(); // ask to exit from run()
    if (!fillerThread->wait(1000))
        qWarning("fillerThread aborted");
}

void MainWindow::gain_changed(int value)
{
    Q_D(const MainWindow);
    if (disable_cfg_slots)
        return;
    AdcConfig adcConfig(d->cfg.adc);
    bool found = false;
    for (int r=0;r<table_cc->numRows();r++) {
        if (table_cc->cellWidget(r, table_cc_col_gain) == QObject::sender()) {
            adcConfig.setGain(r, value);
            found = true;
            break;
        }
    }
    if (found)
        emit adcConfigModified(adcConfig);
    else
        qWarning() << "Unknown gain cell changed its value.";
}

void MainWindow::show_shape_cal_dialog ()
{
    calAvgDialog->show ();
    calAvgDialog->raise ();
}

void MainWindow::show_baseline_cal_dialog()
{
    calZeroDialog->show ();
    calZeroDialog->raise ();
}

void MainWindow::usage (const char *name)
{
    //#ifdef DEBUG
    //  strcpy(str, "[-d] ");
    //#else
    //  strcpy(str, "");
    //#endif
    printf ("\nUsage: %s [-t number]\n\n", name);
    printf ("   -h, --help              show this help\n");
    //#ifdef DEBUG
    printf ("   -d, --debug[=LEVEL]     set debug level to LEVEL, or increase by 1 if not specified\n");
    printf ("   -c, --confpath=<path>   set config directory to <path>\n");
    printf ("   -m, --dev <devname>     use device file <devname>\n");
    printf ("   -b, --bufsize <mb>      set buffer size to <mb> megabytes\n");
    printf ("   -n, --noauto            don't start run automatically\n");
    //#endif
    printf("\n");
}

void MainWindow::status_mbox_accept()
{
    error_msg_shown = "";
    runControl.changeStateForced(ST_INIT);
}

void MainWindow::status_mbox_reject()
{
    error_msg_shown = "";
}

void MainWindow::powerOff()
{
    if (QMessageBox::warning (this, "Shutdown",
                              "System will be shut down.",
                              QMessageBox::Ok, QMessageBox::Cancel) != QMessageBox::Ok)
        return;
    int ret=system("poweroff");
    Q_UNUSED(ret);
}

void MainWindow::transitionTask(daq_state_t toState)
{
//    qDebug() << QString("MainWindow::transitionTask: starting task for state %1").arg(daq_state_str(toState));

    localWaitCursor lwc(this);
    qApp->processEvents();

    //  usleep(200000);
    //  qApp->processEvents();
    //  emit transitionTaskComplete(toState);
    //  return;


    taskret_t taskret = TASK_ERROR;
    switch (toState) {
    case ST_OFF:
        taskret = task_off ();
        break;
    case ST_INIT:
        taskret = task_init ();
        break;
    case ST_RESET:
        taskret = task_reset ();
        break;
    case ST_RUN:
        if(checkStopCondition()){
            toState = ST_PAUSE;
            qDebug()<<"Can't start readout due to stop conditions.";
        } else {
            taskret = task_run ();
            break;
        }
    case ST_PAUSE:
        taskret = task_pause ();
        break;
    case ST_ERROR:
        taskret = task_error ();
        break;
    case ST_FAILURE:
        taskret = task_failure ();
        break;
    default:
        qCritical() << (QString("No task for state %1").arg(daq_state_str(toState)));
        taskret = TASK_OK;
    }
    if (taskret == TASK_OK)
        runControl.transitionTaskComplete(toState);
    else
        runControl.transitionTaskFailed(toState);
}

void MainWindow::adcMatchWindowChanged(const AdcConfig &adcConfig)
{
    const double step = (adcConfig.size() > 0) ? adcConfig.step[0] : 0; // FIXME
    const double winL = -1*adcConfig.prewin * step;
    const double winR = winL + adcConfig.matchwin * step;
    double waveformScaler = 1e-3;
    waveformViewAdcm->setWin(winL*waveformScaler, winR*waveformScaler);
}

/** modifies d->adcConfMem */
void MainWindow::setAdcConfMem(const conf_mem_t& conf_mem)
{
    Q_D(MainWindow);
    d->adcConfMem = conf_mem;
    AdcConfig newConfig(d->cfg.adc);
    newConfig.read_adc_setup_memory(conf_mem);
    emit adcConfigModified(newConfig);
//    setAdcConfig(newConfig);

    disable_cfg_slots++;
    init_table_st(d->adcConfMem);
    setupTrigger_setupAction->setVisible(d->adcConfMem.hasTriggerControl());
    updateWaveformViewFilters(d->adcConfMem, d->cfg.dec);
    disable_cfg_slots--;
}

/** modifies d->cfg.adc */
void MainWindow::setAdcConfig(const AdcConfig &adcConfig)
{
    Q_D(MainWindow);
//    const AdcConfig origAdcConfig(d->cfg.adc);
    AdcConfig newAdcConfig(adcConfig);
    newAdcConfig.read_adc_setup_memory(d->adcConfMem);

//    if (newAdcConfig == origAdcConfig) return;
    d->cfg.adc = newAdcConfig;
    CfgApply();

    disable_cfg_slots++;
    adcMatchWindowChanged(newAdcConfig);
    display_update_channel_controls_visibility(newAdcConfig);
    display_refresh_trig_control(newAdcConfig);
    display_refresh_channel_controls();
    ThrRestore();
    GainRestore();
    DecPermChange();
    TrigRestore ();
    disable_cfg_slots--;
}

/** modifies d->cfg.dec */
void MainWindow::setDecoderConfig(const DecoderConfig &decoderConfig)
{
    Q_D(MainWindow);
//    const DecoderConfig origDecoderConfig(d->cfg.dec);
//    if (decoderConfig == origDecoderConfig) return;
    d->cfg.dec = decoderConfig;

    disable_cfg_slots++;
    updateWaveformViewFilters(d->adcConfMem, d->cfg.dec);
    DecRestore(decoderConfig);
    DecPermChange();
    TrigRestore ();
    disable_cfg_slots--;
}

void MainWindow::updateEventWriterTimer(const StorageConfig &stor)
{
    if(!stor.enable || stor.mode)
        timerEventWriter->stop();
    else
        timerEventWriter->start(stor.period*1e3, false);
}

void MainWindow::updateRawWriterTimer(const StorageConfig &stor)
{
    if(!stor.enable || stor.mode)
        timerRawWriter->stop();
    else
        timerRawWriter->start(stor.period*1e3, false);
}


/** modifies d->cfg.dec.calzero */
void MainWindow::setCalZeroConfig(const CalZeroConfig &newConfig)
{
    Q_D(MainWindow);
    d->cfg.dec.calzero = newConfig;
    emit calZeroConfigModified(newConfig);
}

/** modifies d->cfg.dec.calavg */
void MainWindow::setCalAvgConfig(const CalAvgConfig &newConfig)
{
    Q_D(MainWindow);
    d->cfg.dec.calavg = newConfig;
    emit calAvgConfigModified(newConfig);
}

/** modifies d->cfg all at once */
void MainWindow::setAllConfig(const AllConfig &newConfig)
{
    Q_D(MainWindow);
    qDebug() << "MainWindow::setAllConfig";
    d->cfg = newConfig;
    d->cfg.adc.read_adc_setup_memory(d->adcConfMem);
    emit adcConfigModified(newConfig.adc);
    emit calZeroConfigModified(newConfig.dec.calzero);
    emit calAvgConfigModified(newConfig.dec.calavg);
    emit decoderConfigModified(d->cfg.dec);
    emit rawStorageConfigModified(d->cfg.stor_raw);
    emit evStorageConfigModified(d->cfg.stor_dev);
//    emit histConfigModified(d->cfg.hist); // Unused (there is no slots for this signal)
//    emit runConfigModified(d->cfg.run); // Unused (there is no slots for this signal)
    updateWindowTitle();
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    writeGuiSettings();
    QMainWindow::closeEvent(event);
}

// Autoconnected Menu Actions
void MainWindow::on_fileInitializeAction_activated()
{
    //  error_msg_shown = "";
    runControl.changeStateForced(ST_INIT);
}

void MainWindow::on_fileNewAction_activated ()
{
    if (QMessageBox::warning (this, "Reset All Settings...",
                              "Do you want to reset all the settings to default values and remove calibration data?",
                              QMessageBox::Yes, QMessageBox::No|QMessageBox::Escape|QMessageBox::Default
                              ) != QMessageBox::Yes) return;

    lastSaveFileName.clear();
    *savedcfg = AllConfig();
    setAllConfig(AllConfig());

    runControl.readSettings();
    runControl.enable();
    runControl.changeState(ST_INIT);
}

void MainWindow::on_fileOpenAction_activated ()
{
    if (lastFileOpenFileName.isEmpty())
        lastFileOpenFileName = lastSaveFileName;

    QString newFileName = QFileDialog::getOpenFileName (
                this, "Open configuration",
                lastFileOpenFileName.isEmpty() ? defaultConfigDir : lastFileOpenFileName,
                "INI files (*.ini);;All files (*)");

    if (newFileName.isEmpty())
        return;

    lastFileOpenFileName = newFileName;

    QSettings s(newFileName, QSettings::IniFormat);
    AllConfig tmpConfig;
    tmpConfig.readFromIni(&s);
    if (s.status() != QSettings::NoError) {
        QMessageBox::warning (
                    this, "Open...",
                    QString("Can't read file '%1'").arg(newFileName),
                            QMessageBox::Cancel, Qt::NoButton);
        return;
    }
    if (!tmpConfig.isValid()) {
        QMessageBox::warning (
                    this, "Open...",
                    QString("Invalid configuration file '%1'").arg(newFileName),
                            QMessageBox::Cancel, Qt::NoButton);
        return;
    }

    lastSaveFileName = newFileName;

    setAllConfig(tmpConfig);
//    status ("Loaded setup from %s", workfilename.utf8 ());
}

void MainWindow::on_fileSaveAction_activated ()
{
    if (lastSaveFileName.isEmpty())
        on_fileSaveAsAction_activated();
    else
        doSaveWork();
}

void MainWindow::on_fileSaveAsAction_activated ()
{
    QString s = getSaveFilename ();
    if (s.isEmpty())
        return;
    lastSaveFileName = s;
    doSaveWork ();
}

void MainWindow::on_fileExitAction_activated ()
{
    //  AutoSave();
    close ();
}

void MainWindow::on_setupFitter_setupAction_activated ()
{
    Q_D(const MainWindow);
    FitterSettingsDialog fitterSettingsDialog(this, d->adcConfMem, d->cfg.adc, d->cfg.dec);
    connect(&fitterSettingsDialog, SIGNAL(decoderConfigModified(DecoderConfig)), SIGNAL(decoderConfigModified(DecoderConfig)));
    if (fitterSettingsDialog.exec () == QDialog::Accepted) {
        emit decoderConfigModified(fitterSettingsDialog.getdata());
    }
}

void MainWindow::on_setupDecoder_setupAction_triggered()
{
    Q_D(const MainWindow);
    if(d->cfg.adc.size() == 0){
        qDebug() << "Can't display decoder setup. AdcConfig is not be initialized.";
        return;
    }
    dcconf dcconfDialog(this, d->adcConfMem, d->cfg.adc, d->cfg.dec);
    connect(&dcconfDialog, SIGNAL(decoderConfigModified(DecoderConfig)), SIGNAL(decoderConfigModified(DecoderConfig)));
    dcconfDialog.exec();
}

void MainWindow::on_fileRawFileOutput_activated()
{
    set_data_file_generic(SET_DATAFILE_RAW);
}

void MainWindow::on_fileDecEvFileOutput_activated()
{
    set_data_file_generic(SET_DATAFILE_DEV);
}

void MainWindow::on_setupTrigger_setupAction_activated()
{
    Q_D(const MainWindow);
    AdcConfig adcConfig(d->cfg.adc);
    TrigSetupDialog trigSetupDialog;
    trigSetupDialog.setData(adcConfig.trigbits);
    if (trigSetupDialog.exec() == QDialog::Accepted) {
        adcConfig.setTrigBits(trigSetupDialog.getData());
        emit adcConfigModified(adcConfig);
    }
//    display_refresh_channel_controls();
//    display_refresh_trig_control();
//    CfgApply();
}

void MainWindow::on_statisticsBaselineOffsetAction_activated()
{
    show_baseline_cal_dialog();
}

void MainWindow::on_statisticsPulseShapeAction_activated()
{
    show_shape_cal_dialog();
}

void MainWindow::on_optionsOptionsAction_activated()
{
    Q_D(const MainWindow);
    SettingsDialog settingsDialog(d->adcConfMem, this);
    SettingsData data;

    data.copyFromCfg(d->cfg);
    settingsDialog.setData(data);
    runControl.writeSettings();
    if (settingsDialog.exec() == QDialog::Accepted) {
        AllConfig newConfig(d->cfg);
        settingsDialog.getData().copyToCfg(&newConfig);
        runControl.readSettings();
        if(d->cfg != newConfig)
            setAllConfig(newConfig);
    }
//    CfgApply();
//    RestoreAll();
}

void MainWindow::on_helpAboutAction_activated ()
{
    QString astr = QString("<p>") + OEMStrings::name() + QString("<br>")+
            OEMStrings::version() +", "+ OEMStrings::revision() + "<br><br>" +
            OEMStrings::copyright() +"<br><br>" +
            OEMStrings::address() +"<br><br>" +
            "<a href=\""+OEMStrings::www()+"\">"+OEMStrings::www()+"</a>"+
            "</p>";
    QMessageBox::about (this, "About", astr);

}

void MainWindow::on_helpAboutQtAction_activated ()
{
    qApp->aboutQt();
}

// Autoconnected slots
void MainWindow::on_table_cc_valueChanged( int r, int c)
{
    Q_D(const MainWindow);
    if (disable_cfg_slots)
        return;

    AdcConfig newAdcConfig(d->cfg.adc);

    {
        bool ok = false;
        double value = table_cc->text(r, table_cc_col_thr).toDouble(&ok);

        if (ok) {
            newAdcConfig.setThr(r, value);
        } else {
            qWarning("Input threshold value isn't valid, it set to default.");
            newAdcConfig.setThr(r, 20);
            table_cc->setText(r, table_cc_col_thr, "20");
        }
    }
    {
        bool ok = false;
        double value = table_cc->text(r, table_cc_col_gain).toDouble(&ok);
        if (ok)
            newAdcConfig.setGain(r, value);
    }

    if (r < newAdcConfig.size()) {

        QCheckBox *cb = qobject_cast<QCheckBox*> (table_cc->cellWidget (r, c));
        if (cb) {
            switch (c) {
            case table_cc_col_En:
                set_bit_value (newAdcConfig.trig[r], ADCM_CCTRL_BIT_CMPEN|ADCM_CCTRL_BIT_TEST, cb->isChecked ());
                break;
            case table_cc_col_Zero:
                set_bit_value (newAdcConfig.trig[r], ADCM_CCTRL_BIT_ZERO, cb->isChecked ());
                break;
            case table_cc_col_Inv:
                set_bit_value (newAdcConfig.trig[r], ADCM_CCTRL_BIT_INV, cb->isChecked ());
                break;
            case table_cc_col_M:
                set_bit_value (newAdcConfig.trig[r], ADCM_CCTRL_BIT_M, cb->isChecked ());
                break;
            case table_cc_col_S:
                set_bit_value (newAdcConfig.trig[r], ADCM_CCTRL_BIT_S, cb->isChecked ());
                break;
            }
        }
    }

//    newAdcConfig.updateTriggerBits();
    emit adcConfigModified(newAdcConfig);
}

void MainWindow::on_rbDecRec_clicked()
{
    setDecoderMode(DM_REC);
}

void MainWindow::on_rbDecAvg_clicked()
{
    setDecoderMode(DM_AVG);
}

void MainWindow::on_rbDecOffset_clicked()
{
    setDecoderMode(DM_ZERO);
}

void MainWindow::on_rbDecOff_clicked()
{
    setDecoderMode(DM_OFF);
}

void MainWindow::on_checkBoxHistogramFilling_toggled(bool checked)
{
    QSettings s;
    s.setValue("histogramFilling", checked);
}

void MainWindow::on_checkBoxWaveformEnableAverage_toggled( bool checked )
{
    /*
  if (display_average_state == state) return;
  display_average_state = state;
  if (display_average_state)
    timer8->start ((int)(0.2*update_scope_time*1e3), FALSE);
  else
    timer8->start ((int)(update_scope_time*1e3), FALSE);
  display_refresh_scope_contents ();
*/
}

void MainWindow::on_groupBox_dt_cut_toggled( bool checked )
{
    Q_D(const MainWindow);
    DecoderConfig newConfig(d->cfg.dec);
    newConfig.dt_cut_en = checked;
    emit decoderConfigModified(newConfig);
}

void MainWindow::on_awl_valueChanged( int value )
{
    Q_D(const MainWindow);
    DecoderConfig newConfig(d->cfg.dec);
    int step = d->cfg.adc.size() ? d->cfg.adc.step[0] : 10;
    newConfig.dt_cut_l = (value / step) * step;
    emit decoderConfigModified(newConfig);
}

void MainWindow::on_awr_valueChanged( int value )
{
    Q_D(const MainWindow);
    DecoderConfig newConfig(d->cfg.dec);
    int step = d->cfg.adc.size() ? d->cfg.adc.step[0] : 10;
    newConfig.dt_cut_r = (value / step) * step;
    emit decoderConfigModified(newConfig);
}

void MainWindow::on_tabWidget2_selected( const QString &)
{
    status("");
    //  std::cout << title <<std::endl;
    if (histograms_tab_shown())
        update_mainhist_view();
}

///////////////////////// End
// slot
void MainWindow::updateWaveformViewFilters(const conf_mem_t &conf_mem, const DecoderConfig &dec)
{
    for(int c=0; c<dec.fir.size(); c++)
        waveformViewAdcm->setFilterData(c, dec.fir[c]);
    int bits = conf_mem.getMasterAdcBits();
    for (int c=0; c<conf_mem.nch; c++)
        waveformViewAdcm->setFullScale(c, 1<<(bits-1));
}

void MainWindow::CfgApply()
{
    Q_D(const MainWindow);
    QMetaObject::invokeMethod(adcInstance, "setup", Q_ARG(AllConfig, d->cfg));
}

//--------------------------------------------------------------------------
//
//  Functions for setting GUI boxes from config parameters
//
void MainWindow::RestoreAll ()
{
    disable_cfg_slots++;        // disable CfgApply()
    // FIXME FIXME FIXME    QToolTip::setGloballyEnabled(d->cfg.flags & cfg_flag_tooltips);
    {
        runControl.readSettings();
        QSettings settings;
        checkBoxWaveformEnableAverage->setChecked (settings.value("waveform_averaging").toBool());
    }

    StorRestore ();
//    display_refresh_channel_controls ();
//    display_update_channel_controls_visibility();
    display_refresh_live_statistics ();
    histExplorerView->updateView();
    update_info();
    disable_cfg_slots--;
}

/** depends on d->cfg.dec, d->cfg.adc */
void MainWindow::DecPermChange()
{
    Q_D(const MainWindow);
    int avgok = 1, zerok = 1, firok = 1;
    int zerst = 0, avgst = 0, recst = 0;

    if (MAINDEBUG >= 2) {
        d->cfg.dec.calzero.print();
        d->cfg.dec.calavg.print();
    }

    const int n = d->cfg.adc.size();
    if (d->cfg.dec.calzero.nent.size() < n) zerok = false;
    if (d->cfg.dec.calavg.nent.size() < n) avgok = false;
    if (d->cfg.dec.fir.size() < n) firok = false;

    for (int c = 0; c < n; c++) {
        if (!(d->cfg.adc.trig[c] & ADCM_CCTRL_BIT_CMPEN)) continue;
        if (!(d->cfg.adc.trig[c] & (ADCM_CCTRL_BIT_M|ADCM_CCTRL_BIT_S))) continue;

        if (zerok && d->cfg.dec.calzero.nent[c] < 100)
            zerok = 0;
        if (avgok && d->cfg.dec.calavg.nent[c] < 100)
            avgok = 0;
        if (firok && (!fir_valid (&d->cfg.dec.fir[c]) || (d->cfg.dec.fir[c].klen <= 0)))
            firok = 0;
        //	qDebug() << (1, "ch %2d, zer: %d, avg: %d, fir: %d", c, avgok, zerok, firok);
    }

    if (zerok && avgok) {
        if (firok)
            recst = 2;
        else
            recst = 1;
    } else
        recst = 1;

    if (zerok) {
        if (avgok)
            avgst = 0;
        else
            avgst = 2;
    } else
        avgst = 1;
    if (!zerok)
        zerst = 2;

    //  qDebug() << (0, "zer: %d, avg: %d, fir: %d\n", avgok, zerok, firok);

    if (zerst == 2)
        rbDecOffset->setPaletteForegroundColor (Qt::blue);
    else
        rbDecOffset->unsetPalette ();

    if (avgst == 2)
        rbDecAvg->setPaletteForegroundColor (Qt::blue);
    else if (avgst == 1)
        rbDecAvg->setPaletteForegroundColor (Qt::red);
    else
        rbDecAvg->unsetPalette ();

    if (recst == 2)
        rbDecRec->setPaletteForegroundColor (Qt::blue);
    else if (recst == 1)
        rbDecRec->setPaletteForegroundColor (Qt::red);
    else
        rbDecRec->unsetPalette ();
}

/** depends on d->cfg.adc */
void MainWindow::GainRestore ()
{
    Q_D(const MainWindow);
    int n = d->cfg.adc.size();
//    Q_ASSERT(table_cc->numRows() == n);
    n = std::min(table_cc->numRows(), n);
    for (int c = 0; c < n; c++) {
        QSpinBox *sb = static_cast<QSpinBox *>(table_cc->cellWidget(c, table_cc_col_gain));
        if (sb) {
            sb->setValue((int)d->cfg.adc.gain[c]);
        }
    }
}

/** depends on d->cfg.adc */
void MainWindow::ThrRestore ()
{
    Q_D(const MainWindow);
    int n = d->cfg.adc.size();
//    Q_ASSERT(table_cc->numRows() == n);
    n = std::min(table_cc->numRows(), n);
    for (int c = 0; c < n; c++) {
        Q3TableItem *item = table_cc->item (c, table_cc_col_thr);
        if (!item)
            continue;
        item->setText (QString("%1").arg(d->cfg.adc.thr[c], 0, 'f', 0));
        //    else
        //      qDebug() <<  (2, QString("item(%1, %2) = 0!").arg(c).arg(table_cc_col_thr));
    }
}

/** depends on d->cfg.adc, d->cfg.dec */
void MainWindow::TrigRestore ()
{
    Q_D(const MainWindow);
    //  sbRW = d->cfg.adc.recwin[0];
    // calculate bounds
    const int maxrecwin = d->cfg.adc.adc_cfg_get_recwin_max();
    const int lat_max = std::min(maxrecwin, d->adcConfMem.adc_get_latency_max());
    const int mw_max = std::min(maxrecwin, d->adcConfMem.adc_get_matchwin_max());
    const double step = (d->cfg.adc.size() > 0) ? d->cfg.adc.step[0] : 0; // FIXME
    const double winL = -1*d->cfg.adc.prewin * step;
    const double winR = winL + d->cfg.adc.matchwin * step;

    sbPW->setMinimum (- d->adcConfMem.adc_get_prewin_max() * step);
    sbPW->setMaximum (- d->adcConfMem.adc_get_prewin_min() * step);
    sbPW->setValue (winL);
    sbPW->setSingleStep (step);

    sbMW->setMinimum (d->adcConfMem.adc_get_matchwin_min() * step);
    sbMW->setMaximum (mw_max * step);
    sbMW->setValue (winR-winL);
    sbMW->setSingleStep (step);

    sbLat->setMinimum (d->adcConfMem.adc_get_latency_min() * step);
    sbLat->setMaximum (lat_max * step);
    sbLat->setValue (d->cfg.adc.latency * step);
    sbLat->setSingleStep (step);

    groupBox_dt_cut->setChecked(d->cfg.dec.dt_cut_en);
    awl->setValue((int)d->cfg.dec.dt_cut_l);
    awl->setSingleStep (step);
    awr->setValue((int)d->cfg.dec.dt_cut_r);
    awr->setSingleStep (step);
    adcMatchWindowChanged(d->cfg.adc);
//    display_refresh_trig_control();
}

void MainWindow::StorRestore ()
{
}

void MainWindow::DecRestore(const DecoderConfig &dec)
{
    switch(dec.mode) {
    case DM_OFF :
        rbDecOff->setChecked(true);
        break;
    case DM_ZERO :
        rbDecOffset->setChecked(true);
        break;
    case DM_AVG :
        rbDecAvg->setChecked(true);
        break;
    case DM_REC :
        rbDecRec->setChecked(true);
        break;
    }
}

//--------------------------------------------------------------------------
//
//  Slots activated by configuration parameters change
//


/*
void MainWindow::gain_changed_n (int ch)
{

  if (disable_cfg_slots)
    return;

  QSpinBox *sb = get_gain_sb (ch);
  if (!sb)
    return;
  d->cfg.adc.gain[ch] = sb->value ();
  display_refresh_channel_controls ();
  CfgApply();
}

void MainWindow::thr_changed_n (int ch)
{
  if (disable_cfg_slots)
    return;

  QSpinBox *sb = get_thr_sb (ch);
  if (!sb)
    return;

  if (thr_mv)
    d->cfg.adc.thr[ch] = thr_mv_to_value (cfg, ch, sb->value ());
  else
    d->cfg.adc.thr[ch] = sb->value ();
  display_refresh_channel_controls ();
  CfgApply();
}

void MainWindow::trig_changed_n (int type, int ch, bool st)
{
  if (disable_cfg_slots)
    return;

  if (st)
    d->cfg.adc.trig[ch] |= type;
  else
    d->cfg.adc.trig[ch] &= ~type;
  display_refresh_channel_controls ();
  display_refresh_scope_contents ();
  update_display ();
  CfgApply();
  qDebug() <<  (0, "trig[%d]=%08X\n", ch, d->cfg.adc.trig[ch]);
}
*/

void MainWindow::on_sbPW_valueChanged (double value)
{
    Q_D(const MainWindow);
    if (disable_cfg_slots)
        return;
    AdcConfig adcConfig(d->cfg.adc);
    double step = adcConfig.size() ? adcConfig.step[0] : 10;
    adcConfig.prewin = -value/step;
    emit adcConfigModified(adcConfig);
}

void MainWindow::on_sbMW_valueChanged (double value)
{
    Q_D(const MainWindow);
    if (disable_cfg_slots)
        return;
    AdcConfig adcConfig(d->cfg.adc);
    double step = adcConfig.size() ? adcConfig.step[0] : 10;
    adcConfig.matchwin = value/step;
    emit adcConfigModified(adcConfig);
}

void MainWindow::on_sbLat_valueChanged (int value)
{
    Q_D(const MainWindow);
    if (disable_cfg_slots)
        return;
    AdcConfig adcConfig(d->cfg.adc);
    double step = adcConfig.size() ? adcConfig.step[0] : 10;
    adcConfig.latency = value / step;
    emit adcConfigModified(adcConfig);
}

void MainWindow::on_spinBoxSamplingLength_valueChanged(int)
{
    // TODO
//    Q_D(const MainWindow);
//    if (disable_cfg_slots)
//        return;
//    AdcConfig adcConfig(d->cfg.adc);
//    adcConfig;
//    setAdcConfig(adcConfig);
}

//--------------------------------------------------------------------------

void MainWindow::stopall ()
{
    QMetaObject::invokeMethod(adcInstance, "stopReadout");

//    append_decoder_statistics();
//    display_refresh_run_statistics ();
    display_refresh_live_statistics();
}

void MainWindow::reset_threads()
{
    QMetaObject::invokeMethod(adcInstance, "reset");
    QMetaObject::invokeMethod(filler, "reset");
    QMetaObject::invokeMethod(writeInstance, "reset");
}

taskret_t MainWindow::task_off ()
{
    return TASK_OK;
}

taskret_t MainWindow::task_init ()
{
    stopall ();
    clearStats();

    if (adcThread->isRunning()) {
        QMetaObject::invokeMethod(adcInstance, "connectToDevice", Q_ARG(QUrl, QUrl(devname)));
    } else {
        qWarning() << "task_init: adcThread not running";
//        return TASK_ERROR;
    }
    return TASK_OK;
}

taskret_t MainWindow::task_reset ()
{
    stopall ();
    clearStats();
    updateBaseStats();
    reset_threads();
//    CfgApply();
//    RestoreAll();
    return TASK_OK;
}

taskret_t MainWindow::task_run ()
{
    startReadout();
    return TASK_OK;
}

taskret_t MainWindow::task_pause ()
{
    stopall ();
    return TASK_OK;
}

taskret_t MainWindow::task_error ()
{
    qDebug() << "--- ERROR ---";
    stopall ();
    return TASK_OK;
}

taskret_t MainWindow::task_failure ()
{
    qDebug() << "--- FAILURE ---";
    stopall ();
    return TASK_OK;
}

void MainWindow::onDeviceConnected()
{
    runControl.enable();
//    init_table_st();
//    recreateHistograms();
//    clearstats();
//    reset_threads();
//    CfgApply();
//    RestoreAll();
    idLabel->setText(adcInstance->getDeviceIdStrList().join(" | "));
    runControl.changeState(ST_RESET);
}

void MainWindow::onDeviceDisconnected()
{
    runControl.disable();
}

void MainWindow::startReadout ()
{
//    emit configModified(*cfg);
    QMetaObject::invokeMethod(adcInstance, "startReadout");
}

/*
void MainWindow::update_wst ()
{
    char str[20];
    double mb;

    worker_state wst = WRK_OFF;
    if (writethread) {
        wst = writethread->getstate ();

        mb = (double) (writethread->getnwritten ()) / 1048576;
        if (mb < 1e1)
            sprintf (str, "%.3f", mb);
        else if (mb < 1e2)
            sprintf (str, "%.2f", mb);
        else if (mb < 1e3)
            sprintf (str, "%.1f", mb);
        else
            sprintf (str, "%d", (int) mb);
        // TODO
    }
}
*/

void MainWindow::decoderProgress_changed(double progress)
{
    //    qDebug() << "decoderProgress_changed" << progress;
    pbDECW->setValue ((int)(100. * progress));
}

void MainWindow::eventsProceed_changed(quint64 evNumbers)
{
    eventsProceed = evNumbers;
    if (runControl.state() == ST_RUN && checkStopCondition()) {
        runControl.changeState (ST_PAUSE);
    }
}

void MainWindow::updateRuntime(double seconds)
{
    runSeconds = seconds;
    // check if run has to be stopped
    if (runControl.state() == ST_RUN && checkStopCondition()) {
        runControl.changeState (ST_PAUSE);
    }
}

void MainWindow::updateRunBytes(quint64 bytes)
{
    runBytes = bytes;
    // check if run has to be stopped
    if (runControl.state() == ST_RUN && checkStopCondition()) {
        runControl.changeState (ST_PAUSE);
    }
}

void MainWindow::updateBaseStats ()
{
    char str[20];
    double mb;
    double rt, sec;
    int hour, min;

    //  qDebug() <<  (3, "update_dmast()");
    mb = runBytes / 1024. / 1024.;

    if (mb < 10240)
        sprintf (str, "MB:%9.3f", mb);
    else
        sprintf (str, "GB:%9.1f", mb / 1024.);
    tlrunsize->setText (str);

    rt = runSeconds;
    hour = (int) (rt / 3600);
    min = (int) ((rt - hour * 3600) / 60);
    sec = (rt - hour * 3600 - min * 60);

    sprintf (str, "T:%3d:%02d:%04.1f", hour, min, sec);
    tlruntime->setText (str);

    QTime tv = QTime::currentTime();
    double dmb = mb - prev_mb;
    double dt = 1e-3*prev_tv.msecsTo(tv);
    prev_tv = tv;
    prev_mb = mb;
    double rate;
    if (dt > 0)
        rate = dmb / dt;
    else
        rate = 0;
    sprintf (str, "MB/s: %6.3f", rate);
    tlrunrate->setText (str);
}

/*
 *
 * Scope display options
 *
 */

void MainWindow::clear_wf_view()
{
    waveformViewAdcm->clear();
}

void MainWindow::display_refresh_trig_control(const AdcConfig &adc)
{
    bool state = !adc.is_testmode() && adc.is_co_trigger();
    //  groupBox_trig_windows->setEnabled(state);
    sbPW->setEnabled(state);
    sbMW->setEnabled(state);
    if (state) groupBox_dt_cut->show();
    else groupBox_dt_cut->hide();
    //  groupBox_dt_cut->setEnabled(state);
}

//static double scope_update_timestamp = 0;

//void MainWindow::display_refresh_scope_contents ()
//{
//    if (!oscilloscope_tab_shown()) return;
//    //  waveformView->setAveraging(display_average_state);
//    //  waveformView->setWaveformData(waveformModel_ADC.get());
//}

void MainWindow::simulate_event()
{
#ifdef DEBUG
    // simulate when no hardware present
    if (runControl.state() != ST_OFF) return;
    WaveformDataSimulator sim;
    WaveformDataAll wd(sim.run());
    waveformModel_ADC->set(wd);
    waveformViewAdcm->update();
#endif /* DEBUG */
}
/*
void MainWindow::update_scope_event ()
{
    update_scope();
    int t = (int)(update_scope_time*1e3);
    if (checkBoxWaveformEnableAverage->isChecked()) t = 5;
    timerUpdateScope->start(t);
}
*/

void MainWindow::displayDecodedEvent(const AdcmEvent &ev)
{
    Q_D(const MainWindow);
    if (ev.getFlags() & ADCM_EV_INVAL) {
        qWarning() << "displayDecodedEvent(): invalid event";
        return;
    }
    //  qDebug() <<  (4, "update_scope(): got event #%d, ts:%08X", ev->ec, ev->ts);
    //    int ec = ev.getEventNumber();
    //    if (ec == save_ec) return;
    //    save_ec = ec;

    waveformModel_ADC->addEvent(ev, d->adcConfMem, d->cfg);
}

void MainWindow::update_display ()
{
    Q_D(const MainWindow);

    int cpu_user = sysmon.get("cpu_user").toInt();
    int cpu_nice = sysmon.get("cpu_nice").toInt();
    int cpu_sys = sysmon.get("cpu_sys").toInt();
    int cpu_idle = sysmon.get("cpu_idle").toInt();
    //  int cpu_iowait = sysmon.get("cpu_iowait").toInt();
    //  int cpu_irq = sysmon.get("cpu_irq").toInt();
    //  int cpu_softirq = sysmon.get("cpu_softirq").toInt();

    int busy = cpu_nice+cpu_user+cpu_sys;
    int all = busy+cpu_idle;
    double value = 0;
    int total = 100;
    if (all>0) {
        value = total*(double)busy/all;
    }
    progressBar_cpu_load->setMaximum(total);
    progressBar_cpu_load->setValue(value);

    double bytes_total=1, bytes_avail=0;
    QString fn;
    fn = d->cfg.stor_dev.bn + QDir::separator() + d->cfg.stor_dev.fn;
    get_disk_space(fn, &bytes_total, &bytes_avail);
    if (bytes_total>0)
        value = total * (bytes_total-bytes_avail)/bytes_total;
    else
        value=0;

    bool enable;
    enable = d->cfg.stor_dev.enable;
    textLabel_disk_ev->setEnabled(enable);
    progressBar_disk_ev->setEnabled(enable);
    progressBar_disk_ev->setMaximum(total);
    progressBar_disk_ev->setValue((int)value);

    fn = d->cfg.stor_raw.bn + QDir::separator() + d->cfg.stor_raw.fn;
    get_disk_space(fn, &bytes_total, &bytes_avail);
    if (bytes_total>0)
        value = total * (bytes_total-bytes_avail)/bytes_total;
    else
        value=0;
    enable = d->cfg.stor_raw.enable;
    textLabel_disk_raw->setEnabled(enable);
    progressBar_disk_raw->setEnabled(enable);
    progressBar_disk_raw->setMaximum(total);
    progressBar_disk_raw->setValue((int)value);

    //    update_dst ();
    //    update_wst ();
    //  timerUpdateDisplay->singleShot((int)(update_display_time*1e3));
}

void MainWindow::update_table_cc_size()
{
    //    table_cc->adjustSize();
    //  return;
    const Q3Table *t = table_cc;
    int w = t->columnPos(t->numCols()-1) - t->columnPos(0) + t->columnWidth(t->numCols()-1);
    w += t->verticalHeader()->width();
    w += 10;
    //  t->resize(QSize(w, t->height()));
    QList<int> sizes;
    sizes += w;
    //    sizes += table_cc->width();
    //    splitter->setSizes(sizes);
}

void MainWindow::init_table_st (const conf_mem_t &adcConfMem)
{
    int nch = adcConfMem.nch;
    if (Q3Table * t = table_cc) {
        int nrow = nch;
        int ncol = table_cc_col_number;
        //    std::cout << "Rows: " << nrow << std::endl;
        t->setNumRows (nrow);
        t->setNumCols (ncol);
        Q3Header *vh = t->verticalHeader ();
        Q3Header *hh = t->horizontalHeader ();

        QString s;
        for (int i = 0; i < nrow; i++)
            vh->setLabel (i, s.sprintf ("%d", i));

        hh->setLabel (table_cc_col_En, "E");
        hh->setLabel (table_cc_col_Inv, "Inv");
        hh->setLabel (table_cc_col_Zero, "Z/S");
        hh->setLabel (table_cc_col_M, ustr_ch_label_M);
        hh->setLabel (table_cc_col_S, ustr_ch_label_S);
        hh->setLabel (table_cc_col_thr, "thr");
        hh->setLabel (table_cc_col_gain, "Gain");
        hh->setLabel (table_cc_col_title, "title");
        hh->setLabel (table_cc_col_rate, "Freq, kHz");

        for (int i = 0; i < nrow; i++) {
            t->setItem (i, table_cc_col_En, new MyCheckTableItem (t));
            t->setItem (i, table_cc_col_Inv, new MyCheckTableItem (t));
            t->setItem (i, table_cc_col_Zero, new MyCheckTableItem (t));
            t->setItem (i, table_cc_col_M, new MyCheckTableItem (t));
            t->setItem (i, table_cc_col_S, new MyCheckTableItem (t));
            t->setText (i, table_cc_col_thr, "");
            //      t->setText (i, table_cc_col_gain, "");
            QSpinBox *sb = new QSpinBox(t, "");
            int min_gain = (int)adcConfMem.getMinGainDb();
            int max_gain = (int)adcConfMem.getMaxGainDb();
            int gain_step = (int)adcConfMem.getGainStepDb();
            sb->setMinValue(min_gain);
            sb->setMaxValue(max_gain);
            sb->setLineStep(gain_step);
            sb->setEnabled(min_gain<max_gain && gain_step > 0);
            sb->setSuffix(" dB");
            sb->setButtonSymbols(QSpinBox::PlusMinus);
            t->setCellWidget (i, table_cc_col_gain, sb);
            connect(sb, SIGNAL (valueChanged(int)), this, SLOT(gain_changed(int)) );

            t->setText (i, table_cc_col_title, "");
            t->setText (i, table_cc_col_rate, "");
        }
        t->setColumnReadOnly(table_cc_col_rate, true);
        t->setColumnReadOnly(table_cc_col_title, true);
        t->setColumnStretchable(table_cc_col_title, true);

        for (int i = 0; i < ncol; i++) {
            t->adjustColumn (i);
        }
        if (adcConfMem.hasGainControl())
            t->showColumn(table_cc_col_gain);
        else
            t->hideColumn(table_cc_col_gain);

        t->hideColumn(table_cc_col_title); // hide by default - it is empty
        update_table_cc_size();

        display_refresh_channel_controls();
        ThrRestore();
        GainRestore();
    }
}

void MainWindow::display_refresh_channel_controls ()
{
    Q_D(const MainWindow);
    disable_cfg_slots++;
    Q3Table *t = table_cc;
    Q_ASSERT(t);
    if (!t)
        return;

    for (int ch = 0; ch < std::min(d->cfg.adc.size(), d->adcConfMem.nch); ch++) {
        const int r = ch;
        Q3CheckTableItem *cb=0;

        cb = (Q3CheckTableItem *) t->item (r, table_cc_col_En);
        if (cb)
            cb->setChecked (d->cfg.adc.trig[ch] & ADCM_CCTRL_BIT_CMPEN);

        cb = (Q3CheckTableItem *) t->item (r, table_cc_col_Inv);
        if (cb)
            cb->setChecked (d->cfg.adc.trig[ch] & ADCM_CCTRL_BIT_INV);

        cb = (Q3CheckTableItem *) t->item (r, table_cc_col_Zero);
        if (cb)
            cb->setChecked (d->cfg.adc.trig[ch] & ADCM_CCTRL_BIT_ZERO);

        cb = (Q3CheckTableItem *) t->item (r, table_cc_col_M);
        if (cb)
            cb->setChecked (d->cfg.adc.trig[ch] & ADCM_CCTRL_BIT_M);

        cb = (Q3CheckTableItem *) t->item (r, table_cc_col_S);
        if (cb)
            cb->setChecked (d->cfg.adc.trig[ch] & ADCM_CCTRL_BIT_S);
    }
    disable_cfg_slots--;
}

void MainWindow::display_update_channel_controls_visibility(const AdcConfig &adc)
{
    Q3Table *t = table_cc;
    Q_ASSERT(t);
    if (!t)
        return;
    bool testmode = adc.is_testmode();
    bool hide_MS = testmode || !adc.is_co_trigger(); // Test mode or not internal trigger

    t->setColumnReadOnly(table_cc_col_M, hide_MS);
    t->setColumnReadOnly(table_cc_col_S, hide_MS);

    if (hide_MS) {
        t->hideColumn(table_cc_col_M);
        t->hideColumn(table_cc_col_S);
        //    t->hideColumn(table_cc_col_Zero);
    } else {
        t->showColumn(table_cc_col_M);
        t->showColumn(table_cc_col_S);
        //    t->showColumn(table_cc_col_Zero);
    }
    if (adc.is_ext_trigger())
        t->hideColumn(table_cc_col_Zero);
    else
        t->showColumn(table_cc_col_Zero);
    update_table_cc_size();
}

void MainWindow::onHitCountsChanged(const adcm_counters_t &adcm_counters)
{
    Q3Table *t = table_cc;
    unsigned int nr = std::min(t->numRows(), (int)adcm_counters.n);
    for(unsigned int i=0; i<std::min(nr, adcm_counters.n); i++)  {
        double rate = (qint32)adcm_counters.rawhits[i];
        if (adcm_counters.time > 0)
            rate /= adcm_counters.time;
        else
            rate = 0;
        t->setText(i, table_cc_col_rate, QString("%1").arg(rate/1e3, 0, 'f', 3));
    }
}

void MainWindow::display_refresh_live_statistics ()
{
    // TODO
    /*
    //    TimerStarterOnDestruct(timerUpdateLiveStatistics, (int)(update_livestat_time*1e3), QTimerSingleShot);

    if (!live_statistics_tab_shown()) return;

    Q3Table *t = table_cc;
    Q_ASSERT(t);

    int nr = d->cfg.nch;
    if (nr > t->numRows()) {
        qDebug() << (2, QString("d->cfg.nch = %1, t->numRows() = %2").arg(d->cfg.nch).arg(t->numRows()));
        nr = t->numRows();
    }

    for (int i = 0; i < nr; i++) {
        QString s;
        double time = allstats->adcm_counters.time;
        if (time < 0.1)
            time = DMA_CYCLE_TIME;
        double hits = allstats->adcm_counters.rawhits[i];
        if (hits > 0)
            s = s.sprintf ("%.3f", (double) hits / time * 1.e-3);
        else
            s = "0";
        t->setText (i, table_cc_col_rate, s);
    }
    if (nr > 0)
        t->adjustColumn(table_cc_col_rate);
    //  table_cc->updateGeometry();
    update_table_cc_size();
    */
}

void MainWindow::onDecoderStatisticsUpdated(const adcm_decstat_t &st)
{
//    if (save_decst_nw == st.nw) return;
    sumdecst.add(st);
//    save_decst_nw = st.nw;
    QString decoderstats = make_decst_report(st, sumdecst, runSeconds);
    teinfo->setText(decoderstats);
    QString perfText(st.prof.toString());
    QString perfTextHtml = "<pre>"+perfText+"<pre>";
    if (!perfText.isEmpty())
        emit performanceInfoModified(perfTextHtml);
}

//void MainWindow::append_decoder_statistics()
//{
//    const adcm_decstat_t &st = decodethread->get_decstat ();
//    if (save_decst_nw == st.nw) return;
//    sumdecst.add(st);
//    save_decst_nw = st.nw;
//}

// Decoder info page
//void MainWindow::display_refresh_run_statistics ()
//{
//    QString decoderstats;
//    if (decodethread) {
//        const adcm_decstat_t &ds = decodethread->get_decstat();
//        decoderstats = make_decst_report(ds, sumdecst);
//    }
//    teinfo->setText (decoderstats);
//}

void MainWindow::onHistListModified(const HistList &hl)
{
    histExplorerView->recreateHistograms(hl);
}

void MainWindow::clearStats ()
{
    eventsProceed = 0;
    runSeconds = 0;
    runBytes = 0;
    prev_mb = 0;
    runstats_clear (&rst);

    QMetaObject::invokeMethod(filler, "clearStats");

    update_mainhist_view();

    // clear oscilloscope data
    clear_wf_view();

    // clear main statistics
//    if (allstats)
//        memset (allstats, 0, sizeof (struct allstats_t));
    sumdecst.clear();
}

//double MainWindow::bursttime ()
//{
//  struct timeval tv;
//  if (dma_running) {
//    gettimeofday (&tv, NULL);
//    return (tv.tv_sec - tv_rundma.tv_sec + 1e-6 * (tv.tv_usec - tv_rundma.tv_usec));
//  } else {
//    return (tv_stopdma.tv_sec - tv_rundma.tv_sec + 1e-6 * (tv_stopdma.tv_usec - tv_rundma.tv_usec));
//  }
//}

void MainWindow::update_mainhist_view()
{
    histExplorerView->updateView();
    //  TimerStarterOnDestruct(timerUpdateHistograms, (int)(update_histograms_time*1e3), QTimerSingleShot);
}


void MainWindow::parse_cmdline(int argc, char **argv)
{
    //#ifdef DEBUG
    int dl = 0;
    //#endif
    int go_exit = 0, show_help=0, show_ver=0;
    char str[20];
    long d;
    char *endptr;
    //  struct option opt;
    int option_index = 0;

    static struct option long_options[] = {
        //#ifdef DEBUG
        {"debug", optional_argument, 0, 'd'},
        //#endif
        {"version", no_argument, 0, 'v'},
        {"help", no_argument, 0, 'h'},
        {"noauto", no_argument, 0, 'n'},
        {"config", required_argument, 0, 'c'},
//        {"devname", required_argument, 0, 'm'}, // TODO
        {"bufsize", required_argument, NULL, 'b'},
        {"enable-test", no_argument, NULL, 0},
        {NULL, 0, NULL, 0}
    };
    //#ifdef DEBUG
    strcpy(str, "d::");
    //#endif
    strcat(str, "vhnc:m:b:");
    while (1) {
        int c = getopt_long (argc, argv, str, long_options, &option_index);
        if (c == -1) break;
        /*
// print all options
   if ((c != ':')&&(c != '?')) {
      printf ("option %s", long_options[option_index].name);
      if (optarg)
        printf (" with arg %s", optarg);
      printf ("\n");
  }
*/
        switch (c) {
        //#ifdef DEBUG
        case 'd':
            if (optarg) {
                d = strtol(optarg, &endptr, 10);
                if ((*endptr == 0)&& (d>=0)) {
                    dl = d;
                } else {
                    fprintf(stderr, "bad option 'd' value: %s\n", optarg);
                }
            } else {
                dl++;
            }
            break;
            //#endif
        case 'v':
            show_ver=1;
            go_exit=1;
            break;

        case 'h':
            show_help=1;
            go_exit=1;
            break;

        case 'n':
            runControl.setAutoRun(false);
            break;

        case 'c':
            if (optarg) {
                initialConfigFileName = optarg;
            }
            break;

        case 'm':
            if (optarg) {
                devname = optarg;
            }
            break;
        case 'b':
            if (optarg) {
                d = strtol(optarg, &endptr, 10);
                if ((*endptr == 0) && (d>=0) /*&& (d < 1024*64)*/) // 64GB limit
                {
                    force_bufsize = (unsigned long)d*1024*1024;
                } else {
                    fprintf(stderr, "bad option 'd' value: %s\n", optarg);
                }
            }
            break;

        case '?':
            show_help=1;
            go_exit=2;
            break;
        case 0:
        {
            const char *name = long_options[option_index].name;
            if (0==strcmp(name, "enable-test")) {
                // TODO
            } else {
                fprintf(stderr, "unhandled long option %s\n", name);
                show_help=1;
                go_exit=2;
            }
            break;
        }
        default:
            printf ("?? getopt returned character %c (code 0%o), optarg='%s', ind=%d ??\n", c, c, optarg?optarg:"", option_index);
            show_help=1;
            go_exit=2;
            break;
        }
    }

    if (optind < argc) {
        printf ("non-option ARGV-elements: ");
        while (optind < argc)
            printf ("%s ", argv[optind++]);
        printf ("\n");
        show_help=1;
        go_exit=2;
    }

    if (show_ver) {
        QTextStream cout(stdout, QIODevice::WriteOnly);
        cout << OEMStrings::name() << " " << OEMStrings::version() << " " << OEMStrings::revision() << "\n";
        if (!go_exit) go_exit=1;
        exit(go_exit-1);
    }

    if (show_help) {
        usage(argv[0]);
        if (!go_exit) go_exit=1;
        exit(go_exit-1);
    }

    if (go_exit) exit(go_exit-1);

    //#ifdef DEBUG
    if (dl) {
        //    printf ("qDebug() <<  level: %d\n", dl);
        //    logger.setDebugLevel(dl);
    }
    //#endif

    if (go_exit) exit(go_exit-1);
}

QString MainWindow::test_report_str()
{
    QString msg;
    if (decodethread) {
        const quint64 n_frame_err = decstat_n_frame_err(sumdecst);
        const quint64 n_data_err = decstat_n_data_err(sumdecst);
        const quint64 n_dsp_err = decstat_n_dsp_err(sumdecst);
        const quint64 n_err = n_frame_err+n_data_err+n_dsp_err;
        msg += "<p>";
        if (n_err) {
            msg += "<font color=\"red\">";
            if (n_frame_err) msg += tr("Frame errors: %1 <br>").arg(n_frame_err);
            if (n_data_err) msg += tr("Data errors: %1 <br>").arg(n_data_err);
            if (n_dsp_err) msg += tr("DSP errors: %1 <br>").arg(n_dsp_err);
            msg += "</font>";
        } else {
            msg += "Ok";
        }
    }
    return msg;
}

bool MainWindow::changeStateSynchronous(daq_state_t toState)
{
    runControl.changeState (toState);
    // state change is syncronous within single thread - signals are executed sequentially.
    return true;
}


void MainWindow::set_data_file_generic(int storageMode)
{
    Q_D(MainWindow);
    Q_ASSERT(storageMode==SET_DATAFILE_RAW || storageMode==SET_DATAFILE_DEV);
    StorageConfig newset;
    if(storageMode == SET_DATAFILE_RAW)
        newset = d->cfg.stor_raw;
    else
        newset = d->cfg.stor_dev;

    SetupDataOutputDialog setupDataOutputDialog(newset, storageMode, runControl.state()==ST_RUN, this);
    if (setupDataOutputDialog.exec() == QDialog::Accepted) {
        if(setupDataOutputDialog.updateStorageConfig(newset)) {
            if (storageMode == SET_DATAFILE_RAW) {
                d->cfg.stor_raw = newset;
                emit rawStorageConfigModified(newset);
            } else {
                d->cfg.stor_dev = newset;
                emit evStorageConfigModified(newset);
            }
            reset_threads();
        }
    }
}

bool MainWindow::histograms_tab_shown() const
{
    return tabWidget2->currentPage() == tabHistograms;
}

bool MainWindow::oscilloscope_tab_shown() const
{
    return tabWidget2->currentPage() == tabSettings;
}

bool MainWindow::live_statistics_tab_shown() const
{
    return oscilloscope_tab_shown();
}
void MainWindow::update_info()
{
    sysinfo si;
    textLabel_info->setText(si.text());
    textLabel_info->adjustSize();
}

bool MainWindow::checkStopCondition()
{
    Q_D(const MainWindow);
    if (!d->cfg.run.stopCondition) return false;
    if (d->cfg.run.stopByTimer && runSeconds >= d->cfg.run.stopTimerValue) return true;
    if (d->cfg.run.stopByEventCount && eventsProceed >= d->cfg.run.stopEventCountValue) return true;
    return false;
}

void MainWindow::readGuiSettings(const QString &file)
{
    QSettings* settings;
    if(!file.isEmpty() && QFile(file).exists()){
        settings = new QSettings(file, QSettings::IniFormat);
    } else {
        settings = new QSettings();
    }
    settings->beginGroup("MainWindow");
    restoreGeometry(settings->value("geometry").toByteArray());
    restoreState(settings->value("windowState").toByteArray());
    splitter->restoreGeometry(settings->value("splitter.geometry").toByteArray());
    tabWidget2->setCurrentIndex(settings->value("tabWidget2.currentIndex").toInt());
    //TODO    histExplorerView->setCurrentHist(settings.value("histExplorerView.currentHist").toString());
    settings->endGroup();
    delete settings;
}

void MainWindow::writeGuiSettings(const QString &file)
{
    QSettings *settings;
    if(!file.isEmpty() && QFile(file).exists()){
        settings = new QSettings(file, QSettings::IniFormat);
    } else {
        settings = new QSettings();
    }
    settings->beginGroup("MainWindow");
    settings->setValue("geometry", saveGeometry());
    settings->setValue("windowState", saveState());
    settings->setValue("splitter.geometry", splitter->saveGeometry());
    settings->setValue("tabWidget2.currentIndex", tabWidget2->currentIndex());
    settings->setValue("histExplorerView.currentHist", histExplorerView->currentHistName());
    settings->endGroup();
    settings->sync();
    delete settings;
}

void MainWindow::updateTemperature(QVector<double> v)
{
    QStringList sl;
    Q_FOREACH(double t, v){
        sl.push_back(QString("%1 %2C").arg(t, 0, 'f', 1).arg(QChar(0x02da)));
        if(t>80) {
            temperatureOverflow++;
            qCritical(QString("Board temperature is critical %1.").arg(sl.last()));
            if(temperatureOverflow>2){
                qCritical("Exiting.");
                close();
            }
        } else if(t>75) {
            qWarning() << "Board temperature is rather high " << sl.last();
        } else {
            temperatureOverflow=0;
        }
        adcInstance;
    }
    temperatureLabel->setText(sl.join(", "));
}

void MainWindow::setWorkFileName(const QString &newFileName)
{
    lastSaveFileName = newFileName;
    updateWindowTitle();
}

void MainWindow::updateWindowTitle()
{
    QString s = tr("ADCM") + " - ";
    if (lastSaveFileName.isEmpty()) {
        s += "unnamed";
    } else {
        QDir dir;
        if (lastSaveFileName.startsWith(dir.canonicalPath()))
            s += dir.relativeFilePath(lastSaveFileName);
        else
            s += dir.cleanPath(lastSaveFileName);
    }
    s += " [*]";
    setWindowTitle(s);
    setWindowModified(isModified());
}

void MainWindow::setDecoderMode (int mode)
{
    Q_D(const MainWindow);
    if ((mode < DM_OFF) || (mode > DM_REC))
        return;                     // other buttons pressed

    if (mode == DM_ZERO) {
        show_baseline_cal_dialog();
    }

    if (mode == DM_AVG) {
        show_shape_cal_dialog();
    }

    if ((mode == DM_ZERO) && d->cfg.enabled_ch_has_calzero ()) {

        switch (QMessageBox::warning (this, "Baseline Calibration",
                                      "Do you want to reset before new baseline calibration?"
                                      "Only enabled channels are affected.", "&Yes", "&No", "Cancel", 0, 2)) {
        case 0:
        {
            CalZeroConfig newConfig(d->cfg.dec.calzero);
            newConfig.clear_enabled(d->cfg.adc);
            setCalZeroConfig(newConfig);
        }
            break;
        case 1:
            break;
        default:
            return;
        }
    }

    if ((mode == DM_AVG) && d->cfg.enabled_ch_has_calavg ()) {

        switch (QMessageBox::warning (this, "Average Shape Calibration",
                                      "Do you want to reset before new average waveform shape calibration? "
                                      "Only enabled channels are affected.", "&Yes", "&No", "Cancel", 0, 2)) {
        case 0:
        {
            CalAvgConfig newConfig(d->cfg.dec.calavg);
            newConfig.clear_enabled(d->cfg.adc);
            setCalAvgConfig(newConfig);
        }
            break;
        case 1:
            break;
        default:
            return;
        }
    }

    DecoderConfig newDecoderConfig(d->cfg.dec);
    newDecoderConfig.mode = mode;
//    setDecoderConfig(newDecoderConfig);
    emit decoderConfigModified(newDecoderConfig);

    // set run mode
    adcm_runmode_t newRunMode = (d->cfg.dec.mode == DM_ZERO) ? ADCM_RUNMODE_BASECAL : ADCM_RUNMODE_RUN;
    AdcConfig newAdcConfig(d->cfg.adc);
    newAdcConfig.runmode = newRunMode;
    emit adcConfigModified(newAdcConfig);
}
