//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <vector>

#include <QObject>
#include <QMutex>
#include <QDateTime>

#include "ui_mainwindowbase.h"
#include "sysmon.h"
#include "runcontroladcm.h"
//#include "waveformmodeladc.h"
#include "runstats.h"
#include "adcm_dws.h"
#include "adcm_mem.h"
#include "adcm_df.h"

class QLabel;
class QSpinBox;
class QTimer;
class CalAvgDialog;
class CalZeroDialog;
class StatusMessageBox;
class RawDataWriter;
class DecodeManager;
class EventListFiller;
class AdcInstance;
class AdcmDevice;
class AllConfig;
class WaveformModel_ADC;

class MainWindowPrivate;

class MainWindow : public QMainWindow, public Ui::MainWindowBase
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    void connectSignals();
    void status (const QString &str);
    void initialize(int argc, char *argv[]);
    void initialize_gui(int argc, char *argv[]);

private:
    bool isModified() const;
    QString getSaveFilename();
    bool saveConfig(const QString &fileName);
    void doSaveWork();
    void loadAutoSave();
    void CfgApply();
    void RestoreAll();
    void DecPermChange();
    void GainRestore();
    void ThrRestore();
    void TrigRestore();
    void StorRestore();
    void DecRestore(const DecoderConfig &dec);
    void stopall();
    void startReadout();
    void display_refresh_trig_control(const AdcConfig &adc);
    void init_table_st(const conf_mem_t &adcConfMem);
    void display_refresh_channel_controls();
    void display_update_channel_controls_visibility(const AdcConfig &adc);
//    void append_decoder_statistics();
//    void display_refresh_run_statistics();
    void clearStats();
    //    double bursttime();
    QString test_report_str(); //Unused
    bool changeStateSynchronous(daq_state_t); //Unused
    bool checkStopCondition();
    void parse_cmdline( int argc, char * argv[]);

    void reset_threads();
    taskret_t task_off();
    taskret_t task_init();
    taskret_t task_reset();
    taskret_t task_run();
    taskret_t task_pause();
    taskret_t task_error();
    taskret_t task_failure();
    void clear_wf_view();
    void update_table_cc_size();
    void set_data_file_generic(int which);
    bool histograms_tab_shown() const;
    bool oscilloscope_tab_shown() const;
    bool live_statistics_tab_shown() const;
    void update_info();
    void updateWaveformViewFilters(const conf_mem_t &conf_mem, const DecoderConfig &dec);
    void readGuiSettings(const QString &file = QString());
    void writeGuiSettings(const QString &file = QString());
    void setWorkFileName(const QString &newFileName);
    void updateWindowTitle();
    void setDecoderMode( int mode );

    QLabel * const idLabel;
    QLabel * const temperatureLabel;
    int temperatureOverflow;
    QString lastSaveFileName; // Path to last file saved via File->Save/Save as...
    QString lastFileOpenFileName; // Path to last file opened via File->Open; if valid, writes to lastSaveFileName
//    QMutex hardwareMutex;
    SystemMonitor sysmon;
    const QString defaultConfigFileName;
    const QString defaultConfigDir;
    QString initialConfigFileName; // Uses only on start-up stage
    QString devname;
    RunControlAdcm runControl;
    WaveformModel_ADC * const waveformModel_ADC;
    runstats_t rst;
    adcm_decstat_t sumdecst;
    QString error_msg_shown;
    QTimer * const timerAutoSave;
    QTimer * const timerBaseStatistics;
    QTimer * const timerRawWriter;
    QTimer * const timerEventWriter;
    QTimer * const timer_sim;
    QTimer * const timerUpdateDisplay;
    QTimer * const timerUpdateHistograms; //TODO remove it
    QTimer * const timerUpdateLiveStatistics;
    AllConfig * const savedcfg;
    AllConfig * const lastAutoSaveConfig;
    CalAvgDialog * const calAvgDialog;
    CalZeroDialog * const calZeroDialog;
//    struct allstats_t * const allstats;
    StatusMessageBox * const status_mbox;
    AdcInstance * const adcInstance;
    DecodeManager * const decodeManager;
    EventListFiller * const filler;
    RawDataWriter * const writeInstance;
    QThread * const adcThread;
    QThread * const decodethread;
    QThread * const fillerThread;
    QThread * const writethread;
    quint64 eventsProceed;
    double runSeconds;
    quint64 runBytes;
    double prev_mb;
    QTime prev_tv;
    bool autoSaveLoaded;

Q_SIGNALS:
//    void configModified(const AllConfig &);
    void adcConfigModified(const AdcConfig &);
    void calZeroConfigModified(const CalZeroConfig &);
    void calAvgConfigModified(const CalAvgConfig &);
    void decoderConfigModified(const DecoderConfig &);
    void histConfigModified(const HistConfig &); // Unused (there is no slots for this signal)
    void runConfigModified(const RunConfig &); // Unused (there is no slots for this signal)
    void rawStorageConfigModified(const StorageConfig &); // sends to TWrite
    void evStorageConfigModified(const StorageConfig &); // sends to DecodeManager

    void performanceInfoModified(QString);
///////////////////////// Start
public Q_SLOTS:
    void startThreads();
    void stopThreads();
    void gain_changed(int value); // disable_cfg_slots
    void show_shape_cal_dialog();
    void show_baseline_cal_dialog();
    void usage( const char * name );
    void status_mbox_accept();
    void status_mbox_reject();
    void powerOff();
    void transitionTask( daq_state_t );
    void adcMatchWindowChanged(const AdcConfig &adcConfig);
    void setAdcConfMem(const conf_mem_t &);
    void setAdcConfig(const AdcConfig &adcConfig);
    void setDecoderConfig(const DecoderConfig &decoderConfig);
    void updateEventWriterTimer(const StorageConfig &stor);
    void updateRawWriterTimer(const StorageConfig &stor);
    void setCalZeroConfig(const CalZeroConfig &newConfig);
    void setCalAvgConfig(const CalAvgConfig &newConfig);
    void setAllConfig(const AllConfig &newConfig);

protected:
    void changeEvent(QEvent *e);
    void closeEvent(QCloseEvent *event);

private Q_SLOTS:
    // Autoconnected Menu Actions
    void on_fileInitializeAction_activated();
    void on_fileNewAction_activated();
    void on_fileOpenAction_activated();
    void on_fileSaveAction_activated();
    void on_fileSaveAsAction_activated();
    void on_filePrintAction_activated() {} //no invocation
    void on_fileExitAction_activated();

    void on_setupFitter_setupAction_activated();
    void on_setupDecoder_setupAction_triggered();
    void on_fileRawFileOutput_activated();
    void on_fileDecEvFileOutput_activated();
    void on_setupTrigger_setupAction_activated(); //no invocation

    void on_statisticsBaselineOffsetAction_activated();
    void on_statisticsPulseShapeAction_activated();

    void on_optionsOptionsAction_activated();

    void on_helpIndexAction_activated() {}  //no invocation
    void on_helpContentsAction_activated() {}  //no invocation
    void on_helpAboutAction_activated();
    void on_helpAboutQtAction_activated();

    // Autoconnected slots
    void on_table_cc_valueChanged( int r, int c ); //disable_cfg_slots
    void on_rbDecRec_clicked(); //Full
    void on_rbDecAvg_clicked(); //Shape
    void on_rbDecOffset_clicked(); //Offset
    void on_rbDecOff_clicked(); //Off
    void on_checkBoxHistogramFilling_toggled(bool checked);
    void on_checkBoxWaveformEnableAverage_toggled(bool checked);
    void on_groupBox_dt_cut_toggled(bool checked);
    void on_awl_valueChanged( int value ); //dT Cut left
    void on_awr_valueChanged( int value ); //dT Cut right
    void on_sbPW_valueChanged(double value); //pre-threshold //disable_cfg_slots
    void on_sbMW_valueChanged(double value); //Match Window //disable_cfg_slots
    void on_sbLat_valueChanged(int value); // Data latency //disable_cfg_slots
    void on_spinBoxSamplingLength_valueChanged(int value); // TODO
    void on_tabWidget2_selected( const QString &);
///////////////////////// End
    // Internal Timer slots
    void autoSave();
    void updateBaseStats();
    void update_display();
//    void update_scope_event();
    void simulate_event();
    void display_refresh_live_statistics();
    void update_mainhist_view();

    // Other Internal slots
    void addCalZero(const CalZeroConfig &);
    void addCalAvg(const CalAvgConfig &);
    void displayDecodedEvent(const AdcmEvent &ev);
    void decoderProgress_changed(double percent);
    void eventsProceed_changed(quint64 evNumbers);
    void updateRuntime(double);
    void updateRunBytes(quint64);
    void updateTemperature(QVector<double> v);
    void onDeviceConnected();
    void onDeviceDisconnected();
    void onHitCountsChanged(const adcm_counters_t &);

    // decoder
    void onDecoderStatisticsUpdated(const adcm_decstat_t &);
    void onHistListModified(const HistList &hl);

protected:
    MainWindowPrivate * const d_ptr;
private:
    Q_DECLARE_PRIVATE(MainWindow)
};

#endif // MAINWINDOW_H
