#ifndef RUNCONTROL_H
#define RUNCONTROL_H

#include "daqtypes.h"

#include <QColor>
#include <QMap>
#include <QObject>
#include <QList>
class QTimer;

class RunControl : public QObject {
    Q_OBJECT
public:
    RunControl();
    virtual ~RunControl() {}
    daq_state_t state() const { return rcState; }
    static bool isTransitionAllowed(daq_state_t fromState, daq_state_t toState);
    static daq_state_t nextState(daq_state_t fromState);
    static daq_state_t highlightState(daq_state_t fromState);
    static QList<daq_state_t> getAllStates();

    void enable();
    void disable();
    bool isEnabled() const { return enabled; }
    //  void tryRecover();
    void setAutoRecover(bool);
    void setAutoRun(bool);
    bool getAutoRecover() { return autorecover; }
    bool getAutoRun() { return autorun; }
    void readSettings();
    void writeSettings();
    bool autoStateChangeEnabled();

public slots:
    void changeState (daq_state_t);
    void changeStateForced (daq_state_t); // clears all errors
    void periodicTask ();
    void transitionTaskComplete(daq_state_t toState);
    void transitionTaskFailed(daq_state_t toState);

signals:
    void stateChanged(daq_state_t);
    void transitionTask(daq_state_t);
    void stateChangeEnabled(bool);

protected:
    void transitionPreTask(daq_state_t);
    void autoRun();
    void clearAllErrors();
    bool isStateChangeEnabled();
    void updateStateChangeEnabled();
    bool isAutorunBlockedByError();
    static QList<daq_state_t> statePath(daq_state_t fromState, daq_state_t toState);

    QColor statusPaletteBackgroundColor() const;
    QMap<daq_state_t, QColor> statusBgColor;
    //  QMap<daq_state_t, QString> statusText;
    daq_state_t rcState;
    QTimer * const periodicTaskTimer;

    bool recovery_requested;
    bool autorecover;
    bool autorun;
    bool enabled;
    bool taskRunning;
    unsigned int error_state_count;
};

#endif /* RUNCONTROL_H */
