
#include <cstring>
#include <iostream>

#include "settings_data.h"
#include "adcm16.h"

SettingsDataStorage::SettingsDataStorage()
{
    timestamp_is_utc = 1;
    trig_min = 0;
    trig_max = 0;
    samples_base = 0;
    flen_max = 0;
    fcnt_max = 0;
    ch_dpm_ks = 0;
}

SettingsData::SettingsData()
{
}

SettingsData::SettingsData(const SettingsData &p)
{
    copyData(p);
}

SettingsData &SettingsData::operator=(const SettingsData &p)
{
    if (this != &p) copyData(p);
    return *this;
}

void SettingsData::copyData(const SettingsData &p)
{
    sds = p.sds;
}

void SettingsData::copyFromCfg(const AllConfig &cfg)
{
    sds.cfg = cfg;
    sds.timestamp_is_utc = cfg.adc.timestamp_is_utc;
    sds.trig_min = cfg.adc.mult_min;
    sds.trig_max = cfg.adc.mult_max;
    sds.flen_max = cfg.adc.fragm_len_max;
    sds.fcnt_max = cfg.adc.fragm_count_max;
    sds.ch_dpm_ks = cfg.adc.ch_dpm_ks;

    unsigned int smin=ADCM_MAX_N_SAMP;
//    unsigned int smax=2;
    for(int c=0; c<cfg.adc.size(); c++) {
        const unsigned int s = cfg.adc.recwin[c];
//        if (s > smax)
//            smax = s;
        if (s < smin)
            smin = s;
    }
//    if (smax > ADCM_MAX_N_SAMP) smax = ADCM_MAX_N_SAMP;
    smin &= ~1; // make multiple of 2
    if (smin < 2) smin = 2;
    sds.samples_base = smin;
    //    std::cout << "smin="<<smin<<", smax="<<smax<<std::endl;
    for(int c=0; c < std::min(cfg.adc.size(), sds.samples_mult.size()); c++) {
        unsigned int s = cfg.adc.recwin[c];
        s >>= 1; s <<= 1; if (s<2) s=2; if(s>ADCM_MAX_N_SAMP) s=ADCM_MAX_N_SAMP;
        int mult = (int)(1.*s/sds.samples_base);
        //  	  std::cout << "recwin["<<c<<"]="<<s<<", mult="<<mult<<std::endl;
        sds.samples_mult[c] = mult;
    }
}

void SettingsData::copyToCfg(AllConfig *cfg) const
{
    if (!cfg) return;
    cfg->adc.timestamp_is_utc = sds.timestamp_is_utc;
    unsigned long mmin = sds.trig_min;
    unsigned long mmax = sds.trig_max;
//    if (cfg->nch > 0) {
//        if (mmax > cfg->nch)
//            mmax = cfg->nch;
//        if (mmin > mmax) {
//            unsigned int tmp = mmin;
//            mmin = mmax;
//            mmax = tmp;
//        }
//    }
    cfg->adc.mult_min = mmin;
    cfg->adc.mult_max = mmax;
    //    int b = sds.samples_base;
    for(int c=0; c < cfg->adc.size(); c++) {
        cfg->adc.recwin[c] = sds.samples_base;
        //  	  int m = sds.samples_mult[c];
        //  	  int s = m*b;
        //	  cfg->adc.recwin[c] = s;
        //	  std::cout << "recwin["<<c<<"]="<<s<<" (base="<<b<<", mult="<<m<<")"<<std::endl;
    }
    cfg->run = sds.cfg.run;
}

void SettingsDataStorage::print() const
{
    std::cout << "mult_min="<<std::dec<<trig_min<<std::endl;
    std::cout << "mult_max="<<std::dec<<trig_max<<std::endl;
}
