#ifndef _TH1_H
#define _TH1_H 1

#include <QString>
#include <QSharedData>
#include <vector>
class QMutex;
class QWidget;

class th1 : public QSharedData {

public:

    enum {FIT=0x0001, AUTORANGE=0x0002, NO_AUTORANGE=0x0004, FIT_RESOL=0x0008};

    th1 (const QString &name, int fnbins, double fmin, double fmax, int foptions = 0);
    virtual ~ th1 ();
    void add(const th1 *add);
    void lock();
    void unlock();
    void copyEntry(const th1 *copy);

    // options
    int get_options() const {return options;}
    bool autorange_enabled() const;
    bool fit_enabled() const;
    void set_autorange_enable(bool state) const;
    void set_fit_enable(bool state) const;

    unsigned int get_nbins () const;
    std::vector<double> get_all_bins () const;
    double get_bin (unsigned int index) const;
    void fill (double x);
    void filln (double x, double count);
    void clear ();
    bool save_to_file (QWidget *parent, const QString &dir = QString()) const;
    void setrange (double inmin, double inmax);
    void getrange (double *outmin, double *outmax) const;
    int fitgaus (double *value, double *mean, double *sigma, double *fnorm=0) const;
    QString getname () const;
    void update_autorange() const;

    // statistical values
    unsigned int get_nent () const;
    unsigned int get_over () const;
    unsigned int get_under () const;
    double get_sum () const;
    int get_modseq () const;
    double get_xmin () const;
    double get_xmax () const;
    double bincenter (int n) const;

    int findbin (double x) const;

private:

    int mod; // Number of modifications
    std::vector<double> py;
    unsigned int under, over, nent;
    double xmin, xmax;
    const QString fn;
    const unsigned int nbins;
    // autorange data
    mutable double ar_min, ar_max;
    mutable unsigned int options;
    QMutex * const explorerManagerMutex;

};

#endif /* _TH1_H */
