
#include <cassert>

#include "adcm16.h"
#include "adcm_ev.h"
#include "AdcmChbuf.h"
#include "waveformmodeladc.h"
#include "util.h"
#include "config/allconfig.h"
#include "adcmraw/adcm_mem.h"

//! return time scale point for channel x, step i
double waveform_x(double step_ns, int latency, int i)
{
    return ((double)i-latency) * 0.001 * step_ns;
}

void WaveformModel_ADC::addEvent (const AdcmEvent &ev, const conf_mem_t &conf_mem, const AllConfig &cfg)
{
//    // channel offsets
//    std::vector<double> offs;
//    for (int c = 0; c < cfg.dec.calzero.nent.size(); c++) {
//        offs.push_back(cfg.dec.calzero.get(c));
//    }
    //  bool use_offset = isDisplayBipolar();
    //  bool use_offset = true;

    //  WaveformDataAll &wd(cache);
    WaveformDataAll wd;
    //  wd.resize(ev->ncb);
    //  adcm_print_event(ev.get());
    for (uint n = 0; n < ev.numChBuf; n++) {
        const struct adcm_chbuf_t &cb = ev.chBuf.at(n);
        //    Q_CHECK_PTR(cb);
        unsigned int ch = cb.ch;
        const unsigned int len = cb.len();
        wd[n].c = ch;

        wd[n].t.resize(0); // resize(0) seems faster than clear()
        double adc_step = conf_mem.getMasterAdcStepNs();
        for (uint t=0;t<1;t++) {
            double ptx = cb.pt * adc_step * 0.001; // in us
            //  		wd[n].t.push_back(waveform_x(cfg, ch, cb.pp));
            wd[n].t.push_back(ptx);
            //  		qDebug("ch %d, pt=%e", ch, t);
        }
        wd[n].p.resize(len);
        wd[n].baseline = cfg.dec.calzero.get(ch);
        for(unsigned int i=0;i<len;i++) {
            wd[n].p[i].x = waveform_x(adc_step, cfg.adc.latency, i); // us
            wd[n].p[i].y = cb.w[i];
            //  	  if (i<10)
            //  	    qDebug("x[%d]=%e", i, wd[n].p[i].x);
        }
    }
    /*
  // TODO: move to WaveformModelAdcm
// Add match window marks
  WaveformData marks;
  marks.c = -1; // negative id will be painted with black
  const double step = cfg->adc.step[0] * 0.001; // ns->us
  const bool is_co_trigger = cfg->adc.trigbits & ADCM_TRIGBIT_INT;
  if (is_co_trigger) {
        const double w1 = (- cfg->adc.prewin)*step;
        const double w2 = (- cfg->adc.prewin + cfg->adc.matchwin) * step;
        marks.t.push_back(w1 - 0.5*step);
    marks.t.push_back(w2 + 0.5*step);
  }
  wd[marks.c] = marks;
*/
    set(wd);
}
