
#include <cstdio> // FIXME: printf
#include <cstring>
#include <cassert>

#include <QDebug>
#include <QString>

#include "adcm_dws.h"
#include "adcm_ev.h"
#include "AdcmChbuf.h"
//#include "timer.h"

void adcm_decstat_t::clear()
{
    assert((void*)this == &nw); // check that we will not erase vtbl
    memset(this, 0, sizeof(*this)); // FIXME: dangerous memset
    decst_prof_clear(&prof);
}


const char *decst_prof_label(decst_items_t n)
{
    switch(n) {
    case decst_n_dummy:
        return ("dummy");
    case decst_n_vld:
        return ("vld");
#ifdef FIT_SINE
    case decst_n_sine:
        return ("sine");
#endif
    case decst_n_thr:
        return ("thr");
    case decst_n_rms:
        return ("rms");
    case decst_n_bl:
        return ("bl");
    case decst_n_int:
        return ("int");
    case decst_n_clip:
        return ("clip");
    case decst_n_mwb:
        return ("mwb");
    case decst_n_mwbso:
        return ("mwbso");
    case decst_n_conv:
        return ("conv");
    case decst_n_pksrch:
        return ("pksrch");
    case decst_n_pu:
        return ("pu_rej");
    case decst_n_tfast:
        return ("tfast");
    case decst_n_dtcut:
        return ("dtcut");
    case decst_n_fit:
        return ("fit");
    case decst_n_rawdec:
        return ("rawdec");
    case decst_n_hist:
        return ("hist");
    case decst_n_all:
        return ("all");
    case decst_n_items:
        return ("-");
    }
    return ("unknown");
}

quint64 decstat_n_frame_err(const adcm_decstat_t &decst)
{
    return decst.n_resync + decst.nf_crc + decst.nf_seq;
}
quint64 decstat_n_data_err(const adcm_decstat_t &decst)
{
    return decst.nblk_bad + decst.nblk_unkn + decst.nblk_adc_len + decst.nblk_adc_roll +
            decst.nblk_adc_evnum + decst.nblk_adc_dup;
}
quint64 decstat_n_dsp_err(const adcm_decstat_t &decst)
{
    return decst.dsp_bad_len + decst.dsp_thr_nf + decst.dsp_bl_range + decst.dsp_int_range + decst.dsp_clip_pos +
            decst.dsp_clip_neg + decst.dsp_pk_range + decst.dsp_fit_fail + decst.dsp_fit_bad;
}

//    double t_thr, t_rms, t_bl, t_int, t_clip, t_mwb, t_conv, t_pu, t_tfast, t_fit, t_all;
//    double tm_thr, tm_rms, tm_bl, tm_int, tm_clip, tm_mwb, tm_conv, tm_pu, tm_tfast, tm_fit, tm_all;

void decst_print_prof (const struct decst_prof_t &decstProf)
{
    qDebug() << decstProf.toString();
}

QString decst_prof_t::toString() const
{
    QString ts;
    int i;
    QVector<double> da(decst_n_items);
    QVector<double> dm(decst_n_items);
    const int w = 20; // bar graph width
    double vmin=1e10, vmax=-1e10;

    if (n == 0) {
        return QString();
    }

    for(i=0;i<decst_n_items;i++) {
        da[i] = ta[i]*1e6 / n;
        dm[i] = tm[i]*1e6;
    }

    for(i=0;i<decst_n_items-1;i++) {
        double v = da[i];
        if (v>vmax) vmax=v;
        if (v<vmin) vmin=v;
    }

    ts += "---- Decode time (us) -----\n";
    ts += "   item     avg       min\n";
    ts += "---------------------------\n";
    double sumavg=0, summin=0;
    for(i=0;i<decst_n_items;i++) {
        double tavg = da[i];
        double tmin = dm[i];// - 1e6*timer_read_cost_min;
        sumavg += tavg;
        summin += tmin;
        char s1[w+1];
        {
            double v = tavg;
            int ss;
            for (ss = 0; ss < w; ss++) {
                double val = (v-vmin)*(w-1)/(vmax-vmin);
                if (val >= ss)
                    s1[ss] = '*';
                else
                    s1[ss] = ' ';
            }
            s1[w] = 0;
        }
//        printf("%-7s %8.2f  %8.2f  %s\n",
//               decst_prof_label((decst_items_t)i),
//               tavg,
//               tmin,
//               ((i+1)==decst_n_items)?"":s1);
        ts += QString("%1 %2  %3  %4\n")
             .arg(decst_prof_label((decst_items_t)i), 7)
             .arg(tavg, 8, 'f', 2)
             .arg(tmin, 8, 'f', 2)
             .arg(((i+1)==decst_n_items)?"":s1);
        if (i+2 == decst_n_items) {
            ts += "---------------------------\n";
//            printf("%-7s %8.2f  %8.2f\n", "sum", sumavg, summin);
            ts += QString("%1 %2  %3\n")
                 .arg("sum", 7)
                 .arg(sumavg, 8, 'f', 2)
                 .arg(summin, 8, 'f', 2);
        }
        if (i+1 == decst_n_items)
            ts += "===========================\n";
    }
    return ts;
}

void decst_prof_clear(struct decst_prof_t * p)
{
    assert(p);
    memset (p, 0, sizeof (struct decst_prof_t));
    for(int i=0;i<decst_n_items;i++)
        p->tm[i] = 999999;
}

void decst_add_prof (adcm_decstat_t * decst, const struct decst_prof_t &p)
{
    assert(decst);
    for(int i=0; i<decst_n_items; i++) {
        decst->prof.ta[i] += p.ta[i];
        decst->prof.tl[i] = p.ta[i];
    }
    decst->prof.n += p.n;
    if (decst->prof.n > 0 && p.n > 0) {
        for(int i=0; i<decst_n_items; i++) {
            if (decst->prof.tm[i] > p.tm[i] || decst->prof.tm[i] == 0)
                decst->prof.tm[i] = p.tm[i];
            //  	  printf("%2d %e, %e\n", i, decst->prof.tm[i], p->tm[i]);
        }
    }
    //  printf("decst_add_prof: added %u entries\n", p->n);
}

void decst_add_chstat (adcm_decstat_t * decst, const struct adcm_chstat_t &p)
{
    assert(decst);
    for(unsigned int c=0;c<ADCM_N_CH;c++) {
        decst->cs.all[c] += p.all[c];
        decst->cs.blnd[c] += p.blnd[c];
        decst->cs.intnd[c] += p.intnd[c];
        decst->cs.blout[c] += p.blout[c];
        decst->cs.clip[c] += p.clip[c];
        decst->cs.pu[c] += p.pu[c];
    }
}

void adcm_decstat_t::add(const adcm_decstat_t &incr)
{
    nw += incr.nw;
    overruns += incr.overruns;
    lastpos = incr.lastpos; // non-additive value
    n_resync += incr.n_resync;
    nf_crc += incr.nf_crc;
    nf_seq += incr.nf_seq;
    nf += incr.nf;
    nf_echo += incr.nf_echo;
    nf_dadc += incr.nf_dadc;
    nf_creq += incr.nf_creq;
    nf_cack += incr.nf_cack;
    nf_stat += incr.nf_stat;
    nf_unkn += incr.nf_unkn;
    praw_err = incr.praw_err; // non-additive value
    pf_err = incr.pf_err; // non-additive value

    nblk_all += incr.nblk_all;
    nblk_adc_0 += incr.nblk_adc_0;
    nblk_adc_1 += incr.nblk_adc_1;
    nblk_adc_2 += incr.nblk_adc_2;
    //  nblk_stat += incr.nblk_stat;
    nblk_bad += incr.nblk_bad;
    nblk_unkn += incr.nblk_unkn;
    nblk_crc += incr.nblk_crc;
    pblk_err = incr.pblk_err; // non-additive value

    nblk_adc_len += incr.nblk_adc_len;
    nblk_adc_roll += incr.nblk_adc_roll;
    nblk_adc_dup += incr.nblk_adc_dup;
    nblk_adc_evnum += incr.nblk_adc_evnum;
    nblk_adc_ok += incr.nblk_adc_ok;

    dsp_good_waveform += incr.dsp_good_waveform;
    dsp_total += incr.dsp_total;
    dsp_bad_len += incr.dsp_bad_len;
    dsp_thr_nf += incr.dsp_thr_nf;
    dsp_bl_range += incr.dsp_bl_range;
    dsp_int_range += incr.dsp_int_range;
    dsp_clip_pos += incr.dsp_clip_pos;
    dsp_clip_neg += incr.dsp_clip_neg;
    dsp_pk_range += incr.dsp_pk_range;
    dsp_fit_fail += incr.dsp_fit_fail;
    dsp_fit_bad += incr.dsp_fit_bad;
    dsp_fit_ok += incr.dsp_fit_ok;
    dsp_sine_ok += incr.dsp_sine_ok;

    ev_total += incr.ev_total;
    ev_dt_cut += incr.ev_dt_cut;

    frame_errors_suppressed += incr.frame_errors_suppressed;
    data_errors_suppressed += incr.data_errors_suppressed;
    dsp_errors_suppressed += incr.dsp_errors_suppressed;

    decst_add_prof(this, incr.prof);
    decst_add_chstat(this, incr.cs);
}

void decst_add_ev_chstat (adcm_decstat_t * decst, const AdcmEvent &ev)
{
    assert(decst);
    struct adcm_chstat_t *cs = &decst->cs;
    for(unsigned int c=0;c<ADCM_N_CH;c++) {
        const adcm_chbuf_t *cb = ev.get_ev_chbuf_const(c);
        if (!cb) continue;
        int f = cb->flags;
        if (! (f & ADCM_CH_HAS_DATA)) continue;
        cs->all[c]++;
        if (f & ADCM_CH_BL_NODATA) cs->blnd[c]++;
        if (f & ADCM_CH_INT_NODATA) cs->intnd[c]++;
        if (f & ADCM_CH_BL_OUT) cs->blout[c]++;
        if (f & ADCM_CH_HAS_CLIP) cs->clip[c]++;
        if (f & ADCM_CH_HAS_PU) cs->pu[c]++;
    }
}
