#ifndef _ADCM_DWS_H
#define _ADCM_DWS_H 1

#include <QtCore/QtGlobal>
#include <QMetaType>

#include "adcm16.h"
#include "decoder/decst_t.h"

struct decst_prof_t {
    QString toString() const;
    decst_t_t ta; // average
    decst_t_t tm; // minimum
    decst_t_t tl; // last event
    unsigned int n;
};

typedef quint64 stcnt_t;

// accumulated channel statistics
struct adcm_chstat_t {
    quint64 all[ADCM_N_CH]; /* total pulses */
    quint64 blnd[ADCM_N_CH]; /* ADCM_CH_BL_NODATA */
    quint64 intnd[ADCM_N_CH]; /* ADCM_CH_INT_NODATA */
    quint64 blout[ADCM_N_CH]; /* ADCM_CH_BL_OUT baseline out of limits */
    quint64 clip[ADCM_N_CH]; /* ADCM_CH_EVAL_CLIP clipping detected */
    quint64 pu[ADCM_N_CH]; /* ADCM_CH_EVAL_PU pile-up detected */
};

class adcm_decstat_t
{
public:
    adcm_decstat_t() {
        clear();
    }
    void clear();
    void add(const adcm_decstat_t &incr);


    // find_frame_forw():
    quint64 nw;            // number of words processed
    quint64 overruns;
    unsigned int lastpos;       // last processed word index (not byte!)
    quint64 n_resync;      // number of stream resyncronisations (bad data skipped)
    quint64 nf_crc;        // frames with crc error
    quint64 nf_seq;        // frames with sequence error (lost frames?)
    quint64 nf;            // total number of frames found
    quint64 nf_echo;       // echo frames (type 0)
    quint64 nf_dadc;       // data frames (type 0x100)
    quint64 nf_creq;       // control request frames (type 0x101)
    quint64 nf_cack;       // control acknowledge frames (type 0x102)
    quint64 nf_stat;       // status frames (type 0x103)
    quint64 nf_unkn;       // unknown frame type
    unsigned int praw_err;      // error position, rel. 0
    unsigned int pf_err;        // error position, rel. to frame (data)

    // adcm_decode_data_blocks():
    quint64 nblk_all;      // total data blocks
    quint64 nblk_adc_0;      // total adc data blocks (type 0)
    quint64 nblk_adc_1;      // total adc data blocks (type 1)
    quint64 nblk_adc_2;      // total adc data blocks (type 2)
    //    quint64 nblk_stat;     // total statistical data blocks
    quint64 nblk_bad;      // bad data blocks (corrupted data)
    quint64 nblk_unkn;     // data blocks with unknown data type id
    quint64 nblk_crc;      // data blocks with bad CRC
    unsigned int pblk_err;      // error position, rel. to data block

    quint64 nblk_adc_len;  // adc data blocks with length error
    quint64 nblk_adc_roll; // adc data blocks with roll error
    quint64 nblk_adc_dup;  // adc data blocks with duplicate channel data
    quint64 nblk_adc_evnum;// adc data blocks with bad event number
    quint64 nblk_adc_ok;   // good adc data blocks

    // Signal processing
    quint64 dsp_total;     // total pulses processed (0 if not decoding pulses)
    quint64 dsp_good_waveform;     // total waveforms processed (always nonzero)
    quint64 dsp_bad_len;   // waveform length error
    quint64 dsp_thr_nf;    // threshold position expected but not found
    quint64 dsp_bl_range;  // baseline insuff. data
    quint64 dsp_int_range; // integral insuff. data
    quint64 dsp_clip_pos;  // positive clipping
    quint64 dsp_clip_neg;  // negative clipping
    quint64 dsp_pk_range;  // peak search range bad
    quint64 dsp_fit_fail;  // fit failed
    quint64 dsp_fit_bad;   // fit bad
    quint64 dsp_fit_ok;    // fit ok
    quint64 dsp_sine_ok;   // sine fit ok

    // Event manager
    quint64 ev_total;    // total events processed
    quint64 ev_dt_cut;    // event dropped by dt cut

    // messages
    quint64 frame_errors_suppressed;
    quint64 data_errors_suppressed;
    quint64 dsp_errors_suppressed;

    // profiling timers
    struct decst_prof_t prof;

    // channel statistics
    struct adcm_chstat_t cs;
};

Q_DECLARE_METATYPE(adcm_decstat_t)

//  void print_decoder_frame_err (adcm_decstat_t * decst, char *fmt, ...);
//  void print_decoder_data_err (adcm_decstat_t * decst, char *fmt, ...);
//  void print_decoder_dsp_err (adcm_decstat_t * decst, char *fmt, ...);

extern quint64 decstat_n_frame_err(const adcm_decstat_t &decst);
extern quint64 decstat_n_data_err(const adcm_decstat_t &decst);
extern quint64 decstat_n_dsp_err(const adcm_decstat_t &decst);

const char *decst_prof_label(decst_items_t n);
void decst_print_prof (const struct decst_prof_t &p);
void decst_prof_clear(struct decst_prof_t * p);
void decst_add_prof (adcm_decstat_t * decst, const struct decst_prof_t &p);
void decst_add_chstat (adcm_decstat_t * decst, const struct adcm_chstat_t &p);

class AdcmEvent;

void decst_add_ev_chstat (adcm_decstat_t * decst, const AdcmEvent &ev);

#endif /* _ADCM_DWS_H */
