
#include <cstdio> // FIXME

#include <QDebug>
#include <QString>

#include "adcm_ev.h"
#include "adcm_dws.h"
#include "AdcmChbuf.h"
#include "adcm16.h"

#define EV_PRINT_SHORT_WAVEFORM 0

enum {ADCM_EV_MAX_CH = ADCM_N_CH};

AdcmEvent::AdcmEvent()
    : numChBuf(0), chIndexes(ADCM_N_CH), chBuf(ADCM_EV_MAX_CH),
      burstNum(0), evNum(0), flags(0), ts(0)
{
    for(unsigned int i=0; i<ADCM_N_CH; i++)
        chIndexes[i] = -1;
    //    for(int i=0; i<ADCM_EV_MAX_CH; i++)
    //        cb[i] = adcm_chbuf_t();
}

void AdcmEvent::clear()
{
    numChBuf = 0;
    burstNum = 0;
    evNum = 0;
    flags = 0;
    ts = 0;
    for(unsigned int i=0; i<ADCM_N_CH; i++)
        chIndexes[i] = -1;
}

/*
struct adcm_chstat_t *adcm_alloc_chstat()
{
  struct adcm_chstat_t *p = malloc(sizeof(struct adcm_chstat_t));
  adcm_clear_chstat(p);
  return p;
}

void adcm_free_chstat(struct adcm_chstat_t *cs)
{
  if (cs) free(cs);
}

void adcm_clear_chstat(struct adcm_chstat_t *cs)
{
  memset(cs, 0, sizeof(struct adcm_chstat_t));
}

void adcm_add_chstat(struct adcm_chstat_t *cs, const struct adcm_ev_t *ev)
{
  int c;
  if (!cs || !ev) return;

  for(c=0;c<ADCM_N_CH;c++) {
        const adcm_chbuf_t *evc = &ev->ch[c];
        int f = evc->flags;
        if (! f & ADCM_CH_HAS_DATA) continue;
        cs->all[c]++;
        if (f & ADCM_CH_BL_NODATA) cs->blnd[c]++;
        if (f & ADCM_CH_INT_NODATA) cs->intnd[c]++;
        if (f & ADCM_CH_BL_OUT) cs->blout[c]++;
        if (f & ADCM_CH_EVAL_CLIP) cs->clip[c]++;
        if (f & ADCM_CH_EVAL_PU) cs->pu[c]++;
  }
}

void adcm_print_chstat(const struct adcm_chstat_t *cs)
{
  if (!cs) {printf("adcm_print_chstat: (null)"); return;}
}
*/

//! create new chbuf in event and return its pointer
adcm_chbuf_t *AdcmEvent::new_ev_chbuf(unsigned int c)
{
    Q_ASSERT(c < ADCM_N_CH);
    if (c >= ADCM_N_CH) return 0;

    const short cbufIndex = chIndexes[c]; // chbuf index for channel c: 0..(ev->ncb-1)
    if ((cbufIndex < numChBuf) && (cbufIndex >= 0) && (cbufIndex < ADCM_EV_MAX_CH)) {
        //    printf("ev %d, ch %d: returned existing chbuf at %d\n", ev->ec, c, bn);
        return &chBuf[cbufIndex]; // already exists
    }

    if (numChBuf >= ADCM_EV_MAX_CH) {
        qWarning(QString("ev %1, ch %2: no more chbuf available").arg(getEventNumber()).arg(c));
        return 0; // overflow
    }
    chIndexes[c] = numChBuf;
    const short newCbufIndex = chIndexes[c];
    adcm_chbuf_t *cbuf = &chBuf[newCbufIndex];
    cbuf->clear();
    cbuf->ch = c;
    //  ev->cb[bn].flags=0;
    //  ev->cb[bn].len=0;
    //  printf("ev %d, ch %d: allocated new chbuf at %d\n", ev->ec, c, bn);
    numChBuf++;
    return cbuf; // already exists
    /*
  if (ev && c>=0 && c<ADCM_N_CH)
    return &ev->ch[c];
  else
    return 0;
*/
}

adcm_chbuf_t *AdcmEvent::get_ev_chbuf(unsigned int c)
{
    Q_ASSERT(numChBuf <= ADCM_EV_MAX_CH);
    Q_ASSERT(c < ADCM_N_CH);

    const short cbufIndex = chIndexes[c];
    if (cbufIndex == -1) // not allocated
        return 0;
	//TODO
    if(cbufIndex >= numChBuf){
        qDebug()<<QString("Bad channel: cbufIndex=%1; numChBuf=%2; evNum=%3").arg(cbufIndex).arg(numChBuf).arg(evNum&0xffffffff);
    } else
    Q_ASSERT(cbufIndex < numChBuf); // range check
    if ((cbufIndex < numChBuf) && (cbufIndex >= 0 && cbufIndex < ADCM_EV_MAX_CH))
        return &chBuf[cbufIndex]; // already exists
    else
        return 0;
}

const adcm_chbuf_t *AdcmEvent::get_ev_chbuf_const(unsigned int c) const
{
    Q_ASSERT(numChBuf <= ADCM_EV_MAX_CH);
    Q_ASSERT(c < ADCM_N_CH);
    if (c >= ADCM_N_CH) return 0;

    const short chBufIndex = chIndexes[c];
    if ((chBufIndex < numChBuf) && (chBufIndex >= 0) && (chBufIndex < ADCM_EV_MAX_CH))
        return &chBuf[chBufIndex]; // already exists
    else
        return 0;
    //  return &ev->ch[c];
}

int adcm_get_chbuf_flags(const adcm_chbuf_t *cb)
{
    if (cb)
        return cb->flags;
    else
        return 0;
}

double adcm_get_chbuf_pt(const adcm_chbuf_t *cb)
{
    if (cb)
        return cb->pt;
    else
        return 0;
}

/*
int adcm_write_event (const struct adcm_ev_t * ev)
{

  if (!ev)
    return (-EINVAL);

  FILE *fp = fopen ("test.txt", "a+");
  if (!fp)
    return (-errno);

  fprintf (fp, "#EV: burst %04X, event %04X, ts %16llX\n", ev->bc, ev->ec, ev->ts);
  for (int c = 0; c < ADCM_N_CH; c++) {
        const adcm_chbuf_t *cb = adcm_get_ev_chbuf_const(ev, c);
        if (!cb) continue;
    unsigned int len = cb->len;
    quint64 ts = cb->ts;

    if (len <= 0)
      continue;
    fprintf (fp, "# CH %2d, timestamp: %16llX\n", c, ts);
    for (size_t s = 0; s < cb->len; s++)
      fprintf (fp, "   %3zu\t%d\n", s, cb->w[s] - (1<<(cb->w_bits-1)));
  }
  fclose (fp);
  return (0);
}
*/

void AdcmEvent::print_flags () const
{
    const unsigned int ef = getFlags();
    //  if (!(ef & ADCM_EV_INIT)) {
    //    printf ("event %u not initialized\n", ev->ec);
    //    return;
    //  }
    printf ("event %u, err(%s%s%s%s%s%s%s%s%s%s%s)\n", getEventNumber(),
            (ef & ADCM_EV_INVAL) ? "INVAL " : "",
            (ef & ADCM_EV_ECH) ? "CH" : "",
            (ef & ADCM_EV_EDLEN) ? "LEN" : "",
            (ef & ADCM_EV_EDUPCH) ? "DUP" : "",
            (ef & ADCM_EV_EID) ? "ID" : "",
            (ef & ADCM_EV_ENS) ? "NS" : "",
            (ef & ADCM_EV_EROLL) ? "ROLL" : "",
            (ef & ADCM_EV_ESPW) ? "SPW" : "",
            (ef & ADCM_EV_EBIT) ? "BIT" : "", (ef & ADCM_EV_EUNEXP) ? "UNEXP" : "", (ef & ADCM_EV_BADBLK) ? "BADBLK" : "");

    for (unsigned int c = 0; c < ADC_NCH; c++) {
        const adcm_chbuf_t *cb = get_ev_chbuf_const(c);
        if (!cb) continue;
        const unsigned int f = cb->flags;

        if (!(f & ADCM_CH_HAS_DATA))
            continue;
        printf ("ch %2u: %s, eval(%s%s%s%s%s%s%s%s), has(%s%s%s), err(%s%s%s%s%s%s%s)\n", c,
                (f & ADCM_CH_FINISHED) ? "finished" : "incomplete",
                (f & ADCM_CH_EVAL_THR) ? "THR " : "",
                (f & ADCM_CH_EVAL_BL) ? "BL " : "",
                (f & ADCM_CH_EVAL_INT) ? "INT " : "",
                (f & ADCM_CH_EVAL_PU) ? "PU " : "",
                (f & ADCM_CH_EVAL_CLIP) ? "CLIP " : "",
                (f & ADCM_CH_EVAL_TFINE) ? "TFINE " : "",
                (f & ADCM_CH_EVAL_TFAST) ? "TFAST " : "",
                (f & ADCM_CH_EVAL_RMS) ? "RMS " : "",
                (f & ADCM_CH_HAS_THR) ? "THR " : "",
                (f & ADCM_CH_HAS_CLIP) ? "CLIP " : "",
                (f & ADCM_CH_HAS_PU) ? "PU " : "",
                (f & ADCM_CH_BL_NODATA) ? "BL_NODATA " : "",
                (f & ADCM_CH_INT_NODATA) ? "INT_NODATA " : "",
                (f & ADCM_CH_BL_OUT) ? "BL_OUT " : "",
                (f & ADCM_CH_BAD_LEN) ? "BAD_LEN " : "",
                (f & ADCM_CH_PKS_NODATA) ? "PKS_NODATA " : "",
                (f & ADCM_CH_TFIT_ERR) ? "TFIT_ERR " : "", (f & ADCM_CH_TFIT_OFF) ? "TFIT_OFF " : "");
    }
}

int AdcmEvent::print_event () const
{
//    const unsigned int print_max_samples = 10;

    int nc=0;
    for (unsigned int c = 0; c < ADCM_N_CH; c++) {
        const adcm_chbuf_t *cb = get_ev_chbuf_const(c);
        if (!cb) continue;
        if ((cb->flags & ADCM_CH_HAS_DATA)) nc++;
    }

    printf ("EV #%u, burst %04X, ts:%16llX, nc:%2d\n", getEventNumber(), getBurstNumber(), getEventTimestamp(), nc);
    QString s = "chbuf contents: ";
    for (unsigned int c = 0; c < ADCM_N_CH; c++) {
        const adcm_chbuf_t *cb = get_ev_chbuf_const(c);
        if (!cb) s += '.';
        else
            if (cb->flags & ADCM_CH_HAS_DATA)
                s += "D";
        else s += "e";
    }
    qDebug() << s;
    for (unsigned int c = 0; c < ADCM_N_CH; c++) {
        const adcm_chbuf_t *cb = get_ev_chbuf_const(c);
        if (!cb) continue;
        unsigned int cf = cb->flags;

        if ((cf & ADCM_CH_HAS_DATA)) {

            const unsigned int len = cb->len();
            quint64 ts = cb->ts;
            unsigned int bl = (1<<(cb->w_bits-1));

            printf (" ch:%2u, ts:%16llX, s[%u]: ", cb->ch, ts, len);
#if EV_PRINT_SHORT_WAVEFORM
            if (len >= print_max_samples) {
                for (int s = 0; s < print_max_samples / 2; s++)
                    printf ("%d ", cb->w[s] - bl);
                printf ("... ");
                for (int s = len - print_max_samples / 2; s < len; s++)
                    printf ("%d ", cb->w[s] - bl);
            } else {
                for (int s = 0; s < len; s++)
                    printf ("%d ", cb->w[s] - bl);
            }
#else
            for (unsigned int s = 0; s < len; s++)
                printf ("[%3u]: %5d %s\n", s, cb->w[s] - bl, ((cf & ADCM_CH_HAS_THR) && (s==cb->tp))?"<":"");
#endif
            printf ("f=%08X\n", cf);

            // printf("", ev->ch[c].flags);
            if (cf & ADCM_CH_HAS_THR)
                printf ("ch %2u: thr pos %u\n", c, cb->tp);
            if (cf & ADCM_CH_EVAL_BL)
                printf ("ch %2u: Baseline: %f\n", c, cb->b);
            if (cf & ADCM_CH_EVAL_INT)
                printf ("ch %2u: Integral: %f\n", c, cb->i);
            if (cf & ADCM_CH_EVAL_RMS)
                printf ("ch %2u: DC:%f, RMS:%f \n", c, cb->dc, cb->rms);
            if (cf & ADCM_CH_HAS_PU)
                printf ("ch %2u: pile-up detected\n", c);
            if (cf & ADCM_CH_HAS_CLIP)
                printf ("ch %2u: clipping detected\n", c);
            if ((cf & ADCM_CH_EVAL_TFAST) || (cf & ADCM_CH_EVAL_TFINE))
                printf ("ch %2u: peak time: %f, value:%f, width:%f, chi2:%f\n", c, cb->pt, cb->pa, cb->pw, cb->pchi2);
        }

    }

    return (0);
}

void adcm_ev_ch_flag_set(AdcmEvent *ev, unsigned int ch, unsigned int mask, unsigned int value)
{
    adcm_chbuf_t *cb = ev->get_ev_chbuf(ch);
    if (!cb) return;
    cb->flags = (cb->flags & ~mask) | value;
}

int adcm_ev_ch_flag_cmp(const AdcmEvent *ev, unsigned int ch, unsigned int mask, unsigned int value)
{
    const adcm_chbuf_t *cb = ev->get_ev_chbuf_const(ch);
    if (!cb) return 0;
    return (cb->flags & mask) == value;
}

int adcm_ev_ch_flag_getbit(const AdcmEvent *ev, unsigned int ch, unsigned int mask)
{
    const adcm_chbuf_t *cb = ev->get_ev_chbuf_const(ch);
    if (!cb) return 0;
    return (cb->flags & mask) != 0;
}
void adcm_ev_ch_flag_setbit(AdcmEvent *ev, unsigned int ch, unsigned int mask)
{
    adcm_chbuf_t *cb = ev->get_ev_chbuf(ch);
    if (!cb) return;
    cb->flags |= mask;
}
void adcm_ev_ch_flag_clrbit(AdcmEvent *ev, unsigned int ch, unsigned int mask)
{
    adcm_chbuf_t *cb = ev->get_ev_chbuf(ch);
    if (!cb) return;
    cb->flags &= ~mask;
}
