
#include <cstdio>
#include <cstring>

#include "adcm_util.h"
#include "adcm_ev.h"
#include "adcm_el.h"
#include "adcmraw/adcm_raw.h"
#include "adcm_dec.h"
#include "decoder/waveformdecoder.h"
#include "adcm_dws.h"

#include "mlink/mlink.h"
#include "mlink/mlink_frame_header.h"
#include "mlink/frame_types.h"
#include "regio/frame_util.h"
#include "mlink/frame_hw.h" /* FIXME: ml_get_dma_buf() */
#include "colorlog/log.h"

//#define PRINT_EVENTS 1

int AdcmUtil::handle_decoded_event (AdcmEvent * ev)
{

#ifdef PRINT_EVENTS
    adcm_print_event (ev);
#else
    Q_UNUSED(ev);
#endif
    //  calzero_add(&calzero, ev);
    dec_nev_ok++;
    return (0);
}

void AdcmUtil::print_baseline_calibration (CalZeroConfig * cz)
{
    if (!cz)
        return;
    for (unsigned int i = 0; i < ADC_NCH; i++)
        if (cz->nent[i] > 0)
            printf ("ch %2u:  %.2f   %.2f\n", i, cz->base[i] / cz->nent[i], cz->rms[i] / cz->nent[i]);
}

void AdcmUtil::print_raw_decoded (const quint32 *buf, size_t start, size_t end)
{
/*
    //  int count = 0;

    if (!buf)
        return;

    size_t pos = start;

    while (pos < end) {
        int found;
        size_t newpos, fstart, fend;

        found = frameFinder->find_frame_forw (buf, pos, end, &newpos, &fstart, &fend, &decst);
        //    printf("searching: %d-%d: %d\n", pos,end, found);
        if (found <= 0)
            break;
//    if (found==0 && pos != end) {
//          printf("unknown data\n");
//      adcm_print_raw_frame (&buf[pos], end-pos);
//    }
        pos = newpos;
        if (found > 0) {
            const quint32 *frame;
            unsigned int flen;
            frame_hdr_t *fh = (frame_hdr_t *) & buf[fstart];

            frame = &buf[fstart];
            flen = fend - fstart;
            if (fh->type == FRAME_TYPE_ADC_DATA) {
                int ret, dlen;
                const quint32 *dbuf = reinterpret_cast<const quint32 *>(&frame[ML_FRAME_HEADER_WORDS]);
                //              adcm_clear_event(&adcm_ev);
                dlen = flen - (ML_FRAME_HEADER_WORDS + ML_FRAME_TRAILER_WORDS);
                if (dlen < 0)
                    ret = -1;
                else
                    ret = adcmDataBlockDecoder->adcm_decode_data_blocks (dbuf, dlen, el, &decst);
                //                ret=adcm_decode_data_blocks(dbuf, dlen, &adcm_ev, &decst);
                if (ret < 0) {
                    printf ("decode error (%d)\n", ret);
                    //          adcm_frame_print (frame, flen);
                    dec_frame_err++;
                } else {
                    //                adcm_print_event(&adcm_ev);
                    //                printf("Frame decode: %d blocks\n", ret);
                    dec_blk_ok += ret;
                    dec_frame_ok++;
                }
            } else {
                //        adcm_frame_print (frame, flen);
            }

            if (el) {
                int ret;
                unsigned int ind;

                ret = el->get_last_complete_ev_index (&ind);

                if ((ret == 0) && (ind != ev_ind_prev)) {  // ok and updated
                    AdcmEvent *ev;
                    //        printf ("*************************************************************************\n");
                    //          printf ("ind:%d, prev:%d\n", ind, ev_ind_prev);
                    if (ind > ev_ind_prev) {
                        for (unsigned int i = ev_ind_prev + 1; i <= ind; i++) {
                            ev = el->get_at(i);
                            handle_decoded_event (ev);
                        }
                    } else {              // ind < ev_ind_prev
                        for (unsigned int i = ev_ind_prev + 1; i < el->numEvents(); i++) {
                            ev = el->get_at(i);
                            handle_decoded_event (ev);
                        }
                        for (unsigned int i = 0; i <= ind; i++) {
                            ev = el->get_at(i);
                            handle_decoded_event (ev);
                        }
                    }
                    //          printf ("\n");
                    ev_ind_prev = ind;
                }
            }                         // end if (el)
            //#ifndef PRINT_EVENTS
            {
                int ferr = ml_frame_validate (frame, flen);
                if (ferr || debug_level_ge(4))
                    adcm_print_raw_frame(frame, flen);
            }
            //#endif

            //        if (dec_frame_err > 5) break;
            //      count++;
        }
    }
//  {
//  int i;
//  for (i = start; i <= end; i++) {
////    printnorm ("%s", i == mark ? "> " : "  ");
//    adcm_print_dbuf (buf, i);
////    printnorm ("\n");
//  }
//  }
    printf ("\nDecode stats: raw bytes: %zd, frames: %d ok, %d error; blocks: %d ok, events: %d\n\n",
            (end-start)*4, dec_frame_ok, dec_frame_err, dec_blk_ok, dec_nev_ok);
*/
}

int AdcmUtil::decode_and_print(const char *buf, size_t data_size)
{
    //  int ret=0;

    if (data_size < 4) return(0);

    if (1) {
        print_raw_decoded(reinterpret_cast<const quint32 *>(buf), 0, data_size / 4);
    }
    //  adcm_el_free(el);
    if (dec_nev_ok>0) {
        WaveformDecoder::adcm_decoder_print_stat (&decst);
        print_baseline_calibration (&calzero);
        //  print_calzero(&calzero);
    }
    return 0;
    //err:
    //  return(ret);
}

AdcmUtil::AdcmUtil()
    : frameFinder(new FrameFinder),
      adcmDataBlockDecoder(new AdcmDataBlockDecoder)
//    ,el(new adcm_el_t)
{
    ev_ind_prev = 0;
    dec_frame_err = 0;
    dec_frame_ok = 0;
    dec_blk_ok = 0;
    dec_nev_ok = 0;

    //  adcm_dec_init ();
    WaveformDecoder::adcm_decoder_clear_stat (&decst);
    calzero.clear();

    //  el = adcm_el_alloc(10);
//    ev_ind_prev = el->numEvents() - 1;
}

AdcmUtil::~AdcmUtil()
{
//    delete el;
    delete adcmDataBlockDecoder;
    delete frameFinder;
}

