
#include <cstddef>
#include <ctime>

#include <sys/time.h>

#include <cstdio>
#include "timer.h"
#define _POSIX_CPUTIME
//#ifdef _POSIX_CPUTIME // BUG: CPU-time clocks support needs to be checked at runtime.
//static clockid_t clock_id = 0;
//#endif

//#ifdef DEBUG
#ifdef _POSIX_CPUTIME
#define HP_TIMING_NOW(Var)  do { struct timespec tp; clock_gettime(CLOCK_MONOTONIC, &tp); (Var) = tp.tv_sec+1e-9*tp.tv_nsec;} while (0)
#else
#define HP_TIMING_NOW(Var)  do { struct timeval tv; gettimeofday(&tv, 0); (Var) = tv.tv_sec+1e-6*tv.tv_usec;} while (0)
#endif
//#else
//#define HP_TIMING_NOW(Var)  do {} while (0)
//#endif

double timer_read_cost_min = 0;
double timer_read_cost_avg = 0;

double get_cpu_clk ()
{
//#ifdef _POSIX_CPUTIME
//    pid_t pid = getpid();
//    if (clock_getcpuclockid(pid, &clock_id))
//        perror("clock_getcpuclockid");
//    struct timespec res;
//    if (clock_getres(clock_id, &res))
//        perror("clock_getres");
//    printf("hi-res clock: %e\n", res.tv_sec+1e-9*res.tv_nsec);
//#else
//#endif
    /*
  struct timespec tp1, tp2;
  struct timeval tv1, tv2;
  gettimeofday(&tv1, NULL);
  if (clock_gettime(clock_id, &tp1))
        perror("clock_gettime");
//  printf("now time: %e\n", tp1.tv_sec+1e-9*tp1.tv_nsec);
  double c=1;
  usleep(1000);
//  for(int i=0;i<1000000;i++)
//	c += sin(0.01*i)+cos(0.05*i)+exp(-0.001*i*i);

  gettimeofday(&tv2, NULL);
  if (clock_gettime(clock_id, &tp2))
        perror("clock_gettime");
//  printf("now time: %e\n", tp2.tv_sec+1e-9*tp2.tv_nsec);
  double tp_el = (tp2.tv_sec+1e-9*tp2.tv_nsec) - (tp1.tv_sec+1e-9*tp1.tv_nsec);
  double tv_el = (tv2.tv_sec+1e-6*tv2.tv_usec) - (tv1.tv_sec+1e-6*tv1.tv_usec);

  printf("Elapsed time: tp %e, tv %e, difference %e\n", tp_el, tv_el, (tv_el-tp_el)/tv_el);
  printf("c=%e\n", c);
  */
    return 1;
}

/*
void timer_reset(ftimer_t *t) {
  if (!t) return;
  t->running = 0;
  t->start = 0;
  t->stop = 0;
  t->saved = 0;
}
*/
void timer_start (ftimer_t * t)
{
    if (!t)
        return;
    //  if (t->running) return;
    t->n_reads = 0;
    HP_TIMING_NOW (t->start);// t->n_reads++;
    t->stop = 0;
    t->saved = 0;
    t->running = 1;
}

void timer_resume (ftimer_t * t)
{
    if (!t)
        return;
    if (t->running)
        return;
    HP_TIMING_NOW (t->start); t->n_reads++;
    t->running = 1;
}

void timer_stop (ftimer_t * t)
{
    double elaps;
    if (!t)
        return;
    if (!t->running)
        return;
    HP_TIMING_NOW (t->stop); t->n_reads++;
    elaps = t->stop - t->start;
    t->saved += elaps;
    //  printf("start: %llu, stop:%llu, elaps:%llu, saved:%f\n", t->start, t->stop, elaps, t->saved);
    //  printf("start: %e, stop:%e, elaps:%e, saved:%e\n", (double)t->start, (double)t->stop, (double)elaps, t->saved);
    t->running = 0;
}

double timer_read (ftimer_t * t)
{
    double elaps = 0;

    if (!t)
        return (0);
    if (t->running) {
        HP_TIMING_NOW (elaps); t->n_reads++;
        elaps -= t->start;
    }
    return (t->saved + elaps - t->n_reads * timer_read_cost_min);
}

double timer_overhead_min (ftimer_t * t)
{
    if (!t)
        return (0);
    return t->n_reads * timer_read_cost_min;
}
double timer_overhead_avg (ftimer_t * t)
{
    if (!t)
        return (0);
    return t->n_reads * timer_read_cost_avg;
}

double timer_read_clear (ftimer_t * t)
{
    double t_now = 0, elaps = 0;
    double ret;

    if (!t)
        return (0);
    HP_TIMING_NOW (t_now); t->n_reads++;
    if (t->running) {
        elaps = t_now - t->start;
    }
    ret = t->saved + elaps - t->n_reads * timer_read_cost_min;
    t->saved = 0;
    t->start = t_now;
    t->stop = 0;
    t->n_reads=0;
    return (ret);
}

void init_tsc_timer ()
{
    //  pid_t pid = getpid();
    //  if (clock_getcpuclockid(pid, &clock_id))
    //	perror("clock_getcpuclockid");

    double min=1, max=0, avg=0, navg=0;
    ftimer_t t;
    timer_start(&t);
    double vp=0;
    for(int i=0;i<5000;i++) {
        double v1=0; HP_TIMING_NOW(v1);
        double v = v1-vp; vp=v1;
        if (i>100) {
            avg += v;
            navg++;
            if (v>max) max=v;
            if (v< min) min=v;
        }
    }

    timer_read_cost_min = min;
    timer_read_cost_avg = avg/navg;

    fprintf(stderr, "Timer routine overhead: avg %.3f, min %.3f, max %.3f  us\n", avg/navg*1e6, min*1e6, max*1e6);
    return;
    //  get_cpu_clk ();
    //  printf("TSC timer initialized, clock %f MHz\n", cpu_clk*1e-6);
    //  printf("ULL_MAX:%e, ULL_HALF:%e\n", (double)ULL_MAX, (double)ULL_HALF);
}
