/*
 *  adchardw - ADC low-level interface library
 *	(C) 2003-2006 Ilja Slepnev <Ilja.Slepnev@jinr.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _ADCHARDW_H
#define _ADCHARDW_H

#include <QObject>
#include <QVector>

#include "adcmdevice.h"
#include "adcmregs.h"
#include "adcm_mem.h"
#include "adchwcounters.h"
#include "mlink/mlink.h"
#include "regio/regiopacket.h"
#include "config/adcconfig.h"

typedef enum {
    ADCM_STATUS_UNKNOWN,
    ADCM_STATUS_DRIVER_OK,
    ADCM_STATUS_NO_DMA_MEM,
    ADCM_STATUS_NO_DRIVER,
    ADCM_STATUS_DRIVER_ERROR,
    ADCM_STATUS_HOST_ERROR,
    ADCM_STATUS_DEVICE_ERROR,
    ADCM_STATUS_UNKNOWN_DEVICE,
    ADCM_STATUS_OK
} adcm_status_t;

typedef union {
    unsigned int all;
    struct {
        unsigned int intloop: 1;
        unsigned int extloop: 1;
        unsigned int adcregtest_m: 1;
        unsigned int adcregtest_s1: 1;
        unsigned int adcregtest_s2: 1;
        unsigned int adcregtest_s3: 1;
        unsigned int unused: 29;
        } __attribute__ ((packed)) bit;
} adcm_testpass_t;

//extern unsigned long DEFAULT_DMA_BUFFER_SIZE;

struct adcm_counters_t;
class AllConfig;

class conf_runtime_t
{
public:
    explicit conf_runtime_t(size_t numberOfChannels);
    void clear();
    adcm_runmode_t runmode; // AdcConfig.runmode
    bool run_enable; // just here
    unsigned int trigbits; // AdcConfig.trigbits
    unsigned int timestamp_is_utc; // AdcConfig.timestamp_is_utc
    QVector<adcm_chctrl_reg_t> ch_ctrl;
    adcm_testpass_t testpass;
    adcm_testpass_t testfail;
};

class AllConfig;

class AdcmDevice: protected CustomMlinkDevice, protected AdcmRegs
{
public:
    static bool decomp_recwin(unsigned int recwin, unsigned int *flen, unsigned int *fcnt, unsigned int flen_max, unsigned int fcnt_max);

    explicit AdcmDevice(mlink::RegIOBase &adcmLink);
    QString getDeviceIdStr() const;
    // General Control and Status
    void adc_reset();
    bool adc_start();
    bool adc_stop();
    QString readFirmwareIds();
    void setClock();
    int adc_get_testpass(adcm_testpass_t *fpass, adcm_testpass_t *ffail) const;
    void setBufferSize();
    void poll();
    mlink::RawPacketList popDataPackets();
//    qint64 read(char * data, qint64 maxBytes);
    unsigned int adc_dma_cycle () const;
    int adc_nrun () const;              // cycle count

    void setup(const AllConfig &cfg);

    AdcHwCounters adc_get_counters();
    QVector<double> readTemperatures();

    int nch() const;

    static float adc_gain_val_to_db(int value);
    static int adc_gain_val_to_raw(float db_value);

    conf_mem_t getConfMem() const { return conf_mem; }

protected:
    conf_mem_t discover();
    void writeClock(quint64 value);
    quint64 readClock();
    double ADC_TS_FREQ() const;

    typedef enum { ADC_OFF = 0, ADC_READY = 2, ADC_RUN = 3, ADC_ERROR = 4 } adc_state_t;

    static const char *adc_state_str (int st);

    bool isRunning() const;
    int state_online() const;
    void adc_setreg_inrun(int reg, int data);
    void adc_write_ctrl_reg();
    bool test_regs();
    void clearcounts ();

    // per-channel settings
    void adc_setgain (int ch, float gain); // dB
    void adc_setthr (int ch, int thr);
    void adc_sethyst (int ch, int hyst);
    void adc_settrig (int ch, int trig);

    // General settings
//    void adc_setrunmode (adcm_runmode_t runmode);
    void adc_settrigbits (unsigned int trigbits);
    void adc_setwindow (int searchback, int searchwin, int latency);
    void set_timestamp_is_utc(bool value);
    void adc_set_trig_mult (unsigned int mult_min, unsigned int mult_max);
    void adc_set_fragm_len (unsigned int fragm_len);
    void adc_set_fragm_count (int ch, unsigned int fragm_count);
    int adc_get_fragm_len_max (unsigned int *fragm_len_max) const;
    int adc_get_fragm_count_max (unsigned int *fragm_count_max) const;

    void read_adc_setup_memory(AllConfig *cfg);

    adc_state_t ast;
//    const unsigned int *dma_buf;   // address
//    unsigned long dma_size;     // size, bytes
//    unsigned long dma_used;     // bytes, dma buffer usage
//    unsigned long prev_dma_used;
    unsigned long nrun;         // run number
    conf_runtime_t conf_runtime;
    qint64 ts_write_ts;
    conf_mem_t conf_mem;
};

#endif                          /* _ADCHARDW_H */
