//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "adcm_ids.h"

#include <QString>

QString adc_board_name_str(const int id)
{
    if (id == ADCM_BOARD_ID) return "ADCM16";
    if (id == ADCMLTC_BOARD_ID) return "ADC16-LTC";
    if (id == ADCM3_BOARD_ID) return "ADCM16-3";
    if (id == ADCMG_BOARD_ID) return "ADCM8";
    if (id == ADC64_BOARD_ID) return "ADC64";
    if (id == ADC64S2_BOARD_ID) return "ADC64S2";
    if (id == BPM4100_BOARD_ID) return "BPM4100";
    return "unknown";
}

int maxSlaveCount(quint16 id)
{
    switch (id) {
    case ADCM_BOARD_ID: return 1;
    case ADCMG_BOARD_ID: return 0;
    case ADCMLTC_BOARD_ID: return 1;
    case ADCM3_BOARD_ID: return 0;
    case BPM4100_BOARD_ID: return 0;
    case ADC64_BOARD_ID: return 3;
    case ADC64S2_BOARD_ID: return 3;
    default: return 0;
    }
}

bool isDeviceTypeSupported(quint16 id)
{
    switch (id) {
    case ADCM_BOARD_ID: return true;
    case ADCMG_BOARD_ID: return true;
    case ADCMLTC_BOARD_ID: return true;
    case ADCM3_BOARD_ID: return true;
    case BPM4100_BOARD_ID: return true;
    case ADC64_BOARD_ID: return true;
    case ADC64S2_BOARD_ID: return true;
    default: return false;
    }
}

int boardChannelCount(quint16 id)
{
    switch (id) {
    case ADCM_BOARD_ID: return 16;
    case ADCMG_BOARD_ID: return 8;
    case ADCMLTC_BOARD_ID: return 16;
    case ADCM3_BOARD_ID: return 16;
    case BPM4100_BOARD_ID: return 4;
    case ADC64_BOARD_ID: return 16;
    case ADC64S2_BOARD_ID: return 16;
    default: return 16;
    }
}

int boardAdcBits(quint16 id)
{
    switch (id) {
    case ADCM_BOARD_ID: return 10;
    case ADCMG_BOARD_ID: return 10;
    case ADCMLTC_BOARD_ID: return 14;
    case ADCM3_BOARD_ID: return 14;
    case BPM4100_BOARD_ID: return 14;
    case ADC64_BOARD_ID: return 12;
    case ADC64S2_BOARD_ID: return 12;
    default: return 10;
    }
}

double boardAdcStepNs(quint16 id)
{
    switch (id) {
    case ADCM_BOARD_ID: return 10.;
    case ADCMG_BOARD_ID: return 10.;
    case ADCMLTC_BOARD_ID: return 10.;
    case ADCM3_BOARD_ID: return 20.;
    case BPM4100_BOARD_ID: return 10.;
    case ADC64_BOARD_ID: return 20.;
    case ADC64S2_BOARD_ID: return 16.;
    default: return 10.;
    }
}
