#ifndef ADCM_MEM_H
#define ADCM_MEM_H

#include "adcmregs.h"

#include <QMetaType>
#include <QVector>
#include <QStringList>

// Hardware configuration, read-only register data
class AdcmHardwareConfig {
public:
    AdcmHardwareConfig();
    bool operator==(const AdcmHardwareConfig &rhs) const;
    bool operator!=(const AdcmHardwareConfig &rhs) const { return !operator==(rhs); }
    bool validateAny(int i);
    bool validateSlave(int i, const AdcmHardwareConfig &master);
    quint16 fw_ver;
    quint16 fw_rev;
    quint32 serial_id;
    quint16 ch_dpm_ks;
    quint16 flen_max;
    quint16 fcnt_max;
    adcm_prop_reg_t prop_reg;
};

class conf_mem_board_t {
public:
    conf_mem_board_t();
    conf_mem_board_t(quint16 typeId, const AdcmHardwareConfig &adcmHardwareConfig);
    bool operator==(const conf_mem_board_t &rhs) const;
    bool operator!=(const conf_mem_board_t &rhs) const { return !operator==(rhs); }
    QString descriptionStr() const;
    QString firmwareStr() const;
    QString fwPropStr() const;
    int getAdcHalfRange() const;
    int getAdcMaxValue() const;
    int getAdcMidValue() const;
    int getAdcMinValue() const;

    quint16 board_type;
    int nch;
    unsigned int adcbits;
    double adcstep_ns;
    AdcmHardwareConfig hw;

//    friend class conf_mem_t;
};

class conf_mem_t {
public:
    explicit conf_mem_t(const QVector<conf_mem_board_t> &boards);
    conf_mem_t(const conf_mem_t &ref);
    conf_mem_t();
    conf_mem_t &operator=(const conf_mem_t &rhs);
    bool operator==(const conf_mem_t &rhs) const;
    bool operator!=(const conf_mem_t &rhs) const { return !operator==(rhs); }
    unsigned int master_board_id() const;
    adcm_prop_reg_t master_firmware_prop() const;
    int getAdcHalfRange(int ch) const;
    int getAdcMaxValue(int ch) const;
    int getAdcMidValue(int ch) const;
    int getAdcMinValue(int ch) const;
    bool hasHyst(int ch) const;
    bool hasExtMatchWin() const;
    bool hasMultiFragment() const;
    bool hasTrigMultiplicity() const;

    void clear();
    bool hasGainControl() const;
    bool hasTriggerControl() const;
    int adc_get_prewin_min() const;
    int adc_get_prewin_max() const;
    int adc_get_matchwin_min() const;
    int adc_get_matchwin_max() const;
    int adc_get_latency_min() const;
    int adc_get_latency_max() const;
    int adc_halfrange() const;

    double getMinGainDb() const;
    double getMaxGainDb() const;
    double getGainStepDb() const;

    // FIXME: assumed slave has same hardware limitations (see AdcmHardwareConfig::validateSlave)
    int getMasterAdcBits() const;
    double getMasterAdcStepNs() const;
    int getMasterMaxFlen() const; //!< maximum fragment length, samples, payload only
    int getMasterMaxFcnt() const; //!< maximum number of fragments
    int getMasterChannelMemorySizeKs() const; //!< channel memory size in samples (header, payload, trailer)

    QStringList getDeviceIdStrList() const;
    const QVector<conf_mem_board_t> &boards() const { return b; }
    void append(const conf_mem_board_t &board) {b.push_back(board); nch = getNch();}

//    QVector<double> temperature;

private:
    QVector<conf_mem_board_t> b;
    int getNch() const;
    bool map_ch(int ch, int *bn, int *bc) const;

public:
    int nch; // cached for speed
};

Q_DECLARE_METATYPE(conf_mem_t)

#endif /* ADCM_MEM_H */
