#ifndef _ADCM_RAW_H
#define _ADCM_RAW_H 1

#include <QObject>
#include <QList>
#include <QByteArray>
#include <QMetaType>
#include <QDateTime>

#include "regio/regiopacket.h"

typedef unsigned int frame_search_t;
enum {
    FRAME_SEARCH_BIT_FOUND  = 1,
    FRAME_SEARCH_BIT_RESYNC = 2
};

const unsigned int FRAMELIST_CAP = 16;
struct framelist_t
{
public:
    framelist_t();
    size_t count;
    size_t start[FRAMELIST_CAP];
    size_t end[FRAMELIST_CAP];
};

class FrameList: public mlink::RawPacketList//QList<QByteArray>
{
public:
    FrameList() {seq = 0;}
    explicit FrameList(const mlink::RawPacketList &pl)
    {
        Q_FOREACH(QByteArray buf, pl) {
            push_back(buf);
        }
    }

    bool shouldDrop() const {
        int diff = created.msecsTo(QTime::currentTime());
        if(diff<0) diff += 86400000;
        return  diff > 200;
    }
    QTime created;
    quint64 seq;
};
Q_DECLARE_METATYPE(FrameList)

class adcm_decstat_t;
class adcm_el_t;

void adcm_print_raw_frame (const unsigned int *f, size_t len);

//
// Decode frame data. Input is frame data block without header and trailer.
//
class AdcmDataBlockDecoder
{
public:
    AdcmDataBlockDecoder(int index=0) : decoderIndex(index), prev_ah_ec(decoderIndex), prev_ec(decoderIndex), ec_offset(0) {}
    int adcm_decode_data_blocks (const unsigned int *f, size_t len, struct adcm_el_t * el, adcm_decstat_t * st);
    void reset();
    void setTimeStampBitsFlag(bool val) { useLowerTimeStampBits = val; }

private:
    const int decoderIndex;
    unsigned int prev_ah_ec;
    unsigned long prev_ec;
    unsigned int ec_offset;
    bool useLowerTimeStampBits;
};


class FrameFinder: public QObject
{
public:
    explicit FrameFinder(QObject *parent = 0) : QObject(parent), prev_seq(0), seq_initialized(false) {}
    void reset();
    frame_search_t find_frame_forw (const unsigned int *buf, size_t pos, size_t end, size_t * fnewpos,
                                    size_t * ffstart, size_t * ffend, adcm_decstat_t * decst);
private:
    quint16 prev_seq;
    bool seq_initialized;
};

#endif /* _ADCM_RAW_H */
