    //
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "adcconfig.h"

#include <cstdio>
#include <QDebug>

#include "adcm16.h"
#include "adcmraw/adcm_mem.h"

AdcConfig::AdcConfig()
{
    //    trig[0] = 1;
    //    trig[ADC_NCH/2] = 1;
    prewin = 20;
    latency = 30;
    matchwin = 40;
    runmode = ADCM_RUNMODE_RUN;
    trigbits = ADCM_TRIGBIT_INT;
    mult_min = 1;
    mult_max = ADC_NCH;
    fragm_len_max = 120;
    fragm_count_max = 1;
    ch_dpm_ks = 8;

//    resize(ADC_NCH);
    timestamp_is_utc = 1;
    nch = 0;
}

void AdcConfig::resize(int newNCh)
{
    if(bits.size() < newNCh){
        std::vector<int> v = bits.toStdVector();
        v.resize(newNCh, 2);//ADCM_BITS
        bits = bits.fromStdVector(v);
    }
    if(step.size() < newNCh){
        std::vector<double> v = step.toStdVector();
        v.resize(newNCh, 1);//1e9 / ADC_TS_FREQ
        step = step.fromStdVector(v);
    }
    if(gain.size() < newNCh){
        std::vector<double> v = gain.toStdVector();
        v.resize(newNCh, 0/*ADC_DEFAULT_GAIN*/);
        gain = gain.fromStdVector(v);
    }
    if(thr.size() < newNCh){
        std::vector<double> v = thr.toStdVector();
        v.resize(newNCh, 20);
        thr = thr.fromStdVector(v);
    }
    if(trig.size() < newNCh){
        std::vector<int> v = trig.toStdVector();
        v.resize(newNCh, ADCM_CCTRL_BIT_CMPEN|ADCM_CCTRL_BIT_TEST|ADCM_CCTRL_BIT_ZERO);// enable comparator
        trig = trig.fromStdVector(v);
    }
    if(recwin.size() < newNCh){
        std::vector<unsigned int> v = recwin.toStdVector();
        v.resize(newNCh, 120);//ADCM_BITS;
        recwin = recwin.fromStdVector(v);
    }
    nch = newNCh;
}

bool AdcConfig::operator==(const AdcConfig &other) const
{
    if(DEBUG_PRINT_CONFIG_DIFFERENCE){
        if(bits != other.bits)
            qDebug() << "bits differs";
        if(step != other.step)
            qDebug() << "step differs";
        if(gain != other.gain)
            qDebug() << "gain differs";
        if(thr != other.thr)
            qDebug() << "thr differs";
        if(trig != other.trig)
            qDebug() << "trig differs";
        if(recwin != other.recwin)
            qDebug() << "recwin differs";
        if(prewin != other.prewin)
            qDebug() << "prewin differs";
        if(matchwin != other.matchwin)
            qDebug() << "matchwin differs";
        if(latency != other.latency)
            qDebug() << "latency differs";
        if(runmode != other.runmode)
            qDebug() << "runmode differs";
        if(trigbits != other.trigbits)
            qDebug() << "trigbits differs";
        if(mult_min != other.mult_min)
            qDebug() << "mult_min differs";
        if(mult_max != other.mult_max)
            qDebug() << "mult_max differs";
        if(fragm_len_max != other.fragm_len_max)
            qDebug() << "fragm_len_max differs";
        if(fragm_count_max != other.fragm_count_max)
            qDebug() << "fragm_count_max differs";
        if(ch_dpm_ks != other.ch_dpm_ks)
            qDebug() << "ch_dpm_ks differs";
    }
    return true
            && bits == other.bits
            && step == other.step
            && gain == other.gain
            && thr == other.thr
            && trig == other.trig
            && recwin == other.recwin
            && prewin == other.prewin
            && matchwin == other.matchwin
            && latency == other.latency
            && runmode == other.runmode
            && trigbits == other.trigbits
            && mult_min == other.mult_min
            && mult_max == other.mult_max
            && fragm_len_max == other.fragm_len_max
            && fragm_count_max == other.fragm_count_max
            && ch_dpm_ks == other.ch_dpm_ks
            && timestamp_is_utc == other.timestamp_is_utc
            ;
}

bool AdcConfig::isBaselineCalibrationMode() const
{
    return runmode == ADCM_RUNMODE_BASECAL;
}

void AdcConfig::adjust_adc_recwin()
{
    const unsigned int recwin_max = adc_cfg_get_recwin_max();
    for (int ch = 0; ch < size(); ch++) {
        if (recwin[ch] > recwin_max)
            recwin[ch] = recwin_max;
    }
}

void AdcConfig::print() const
{
    fprintf (stderr, "adc_bits: ");
    for (int ch = 0; ch < size(); ch++)
        fprintf (stderr, "%d ", bits[ch]);
    fprintf (stderr, "\n");

    fprintf (stderr, "adc_gain: ");
    for (int ch = 0; ch < size(); ch++)
        fprintf (stderr, "%f ", gain[ch]);
    fprintf (stderr, "\n");

    fprintf (stderr, "adc_thr: ");
    for (int ch = 0; ch < size(); ch++)
        fprintf (stderr, "%f ", thr[ch]);
    fprintf (stderr, "\n");

    fprintf (stderr, "adc_trig: ");
    for (int ch = 0; ch < size(); ch++)
        fprintf (stderr, "%d ", trig[ch]);
    fprintf (stderr, "\n");

    //  fprintf (stderr, "adc_offs: ");
    //  for (c = 0; c < adc_offs.size(); c++)
    //    fprintf (stderr, "%d ", adc_offs[c]);
    //  fprintf (stderr, "\n");

    fprintf (stderr, "adc_recwin: ");
    for (int ch = 0; ch < size(); ch++)
        fprintf (stderr, "%d ", recwin[ch]);
    fprintf (stderr, "\n");
    fprintf (stderr, "adc_prewin: %d\n", prewin);
    fprintf (stderr, "adc_matchwin: %d\n", matchwin);
    fprintf (stderr, "adc_latency: %d\n", latency);
    fprintf (stderr, "adc_runmode: %08X\n", runmode);
    fprintf (stderr, "adc_mult_min: %08X\n", mult_min);
    fprintf (stderr, "adc_mult_max: %08X\n", mult_max);
    fprintf (stderr, "adc_fragm_len_max: %u\n", fragm_len_max);
    fprintf (stderr, "adc_fragm_count_max: %u\n", fragm_count_max);
    fprintf (stderr, "adc_ch_dpm_ks: %u\n", ch_dpm_ks);
    fprintf (stderr, "adc_cfg->timestamp_is_utc: %u\n", timestamp_is_utc);
}

// Threshold conversion: mV (-0.5..0.5) and value (0..1023)

const double thr_min_mv = -512;
const double thr_max_mv = 511;

//double AdcConfig::mv_to_value_scaler ()
//{
//    /*
//  double range_mv = thr_max_mv - thr_min_mv;
//  double range_val = ADC_MAX_THR - ADC_MIN_THR;
//  if (range_mv == 0) return(1);
//  return(range_val/range_mv);
//*/
//    return (1);
//}

int AdcConfig::adc_rawdata_maxval(int ch) const
{
    if (ch<0 || ch>=size()) return 1;
    int n = (1<<(bits[ch]-1))-1;
    return n;
}

int AdcConfig::adc_rawdata_minval(int ch) const
{
    if (ch<=0 || ch>=size()) return (-1);
    int n = -(1<<(bits[ch]-1));
    return n;
}

int AdcConfig::adc_rawdata_midval(int ch) const
{
    if (ch<=0 || ch>=size()) return 0;
    return 0;
}

double AdcConfig::thr_mv_min () const
{
    return thr_min_mv;
}

double AdcConfig::thr_mv_max () const
{
    return thr_max_mv;
}

void AdcConfig::setThr(int ch, double value)
{
    if (value < adc_rawdata_minval(ch))
        value = adc_rawdata_minval(ch);
    if (value > adc_rawdata_maxval(ch))
        value = adc_rawdata_maxval(ch);
    if (ch >= 0 && ch < size())
        thr[ch] = value;
}

void AdcConfig::setGain(int ch, double value)
{
    // FIXME: check gain range
    if (value >= 0 && value <= ADC_MAX_GAIN && ch >= 0 && ch < size())
        gain[ch] = value;
}

unsigned int AdcConfig::adc_cfg_get_recwin_max() const
{
    int num_enabled = 0; // number of enabled channels
    for (int ch = 0; ch < size(); ch++) {
        int enabled = (trig[ch] & ADCM_CCTRL_BIT_TEST) ||
                      ( (trig[ch] & ADCM_CCTRL_BIT_CMPEN) && (trig[ch] & (ADCM_CCTRL_BIT_M|ADCM_CCTRL_BIT_S)) );
        if (enabled)
            num_enabled++;
    }

    unsigned int max_samples_in_event = fragm_len_max * fragm_count_max; // event fifo size
    unsigned int mult = mult_max;
    //  if (mult > num_enabled) mult = num_enabled;
    if (isBaselineCalibrationMode()) { // baseline calibration mode
        mult = num_enabled;
    }
    if (mult<1) mult=1;
    unsigned int maxsize = max_samples_in_event / mult;
    if (mult > 1) { // DPM works pass-through when onle 1 channel enabled
        unsigned int max_total_size = ch_dpm_ks<<10;
        unsigned int max_ch_size = max_total_size/mult;
        if (maxsize > max_ch_size)
            maxsize = max_ch_size;
    }

    if (maxsize < ADCM_MIN_N_SAMP)
        maxsize = ADCM_MIN_N_SAMP;

    // limit set by decoder software
    if (maxsize > ADCM_MAX_N_SAMP)
        maxsize = ADCM_MAX_N_SAMP;
    maxsize &= ~1;

    //  printf("num_enabled=%u, max_samples_in_event=%u, mult=%u, maxsize=%u\n", num_enabled, max_samples_in_event, mult, maxsize);

    return maxsize;
}

bool AdcConfig::is_testmode() const
{
    return isBaselineCalibrationMode();
}

bool AdcConfig::is_co_trigger() const
{
    return (trigbits & ADCM_TRIGBIT_INT);
}

bool AdcConfig::is_ext_trigger() const
{
    return (trigbits & ADCM_TRIGBIT_EXT);
}

bool AdcConfig::is_simple_trigger() const
{
    return (trigbits & ADCM_TRIGBIT_INT_SIMPLE);
}

void AdcConfig::setTrigBits(int value)
{
    trigbits = value;
//    updateTriggerBits();
}

//void AdcConfig::updateTriggerBits()
//{
//    for (int c = 0; c < trig.size(); c++) {
//        if (is_testmode()) { //baseline calibration
//            //            trig |= ADCM_CCTRL_BIT_M;
//            //            trig |= ADCM_CCTRL_BIT_S;
//            trig[c] |= ADCM_CCTRL_BIT_TEST;
//        } else {
//            // normal run, clear TEST bit
//            trig[c] &= ~ADCM_CCTRL_BIT_TEST;
//            if (is_simple_trigger()) {
//                // sync by any channel independently
//                trig[c] |= ADCM_CCTRL_BIT_M;
//                trig[c] |= ADCM_CCTRL_BIT_S;
//            } else if (is_ext_trigger()) {
//                // external trigger mode, master is special channel N+1
//                trig[c] &= ~ADCM_CCTRL_BIT_M;
//                //      trig &= ~ADCM_CCTRL_BIT_S;
//                trig[c] |= ADCM_CCTRL_BIT_S;
//            } else {
//                // in coincidence mode, leave settings as it is
//            }
//        }
//    }
//}


bool AdcConfig::ch_enabled_offset_calibration (int ch) const
{
    if (ch >= size())
        return false;
//  return (trig[ch] & ADCM_CCTRL_BIT_TEST);
    return (trig[ch] & ADCM_CCTRL_BIT_CMPEN);
}

bool AdcConfig::ch_enabled_shape_calibration (int ch) const
{
    unsigned int require_bits = (ADCM_CCTRL_BIT_CMPEN | ADCM_CCTRL_BIT_M | ADCM_CCTRL_BIT_S);

    if (ch >= size())
        return false;
    return ((trig[ch] & require_bits) == require_bits);
}

bool AdcConfig::ch_enabled_reconstruction (int ch) const
{
    if (ch >= size())
        return false;
    if (!(trig[ch] & ADCM_CCTRL_BIT_CMPEN))
        return false;
    return (trig[ch] & (ADCM_CCTRL_BIT_M | ADCM_CCTRL_BIT_S));
}

void AdcConfig::read_adc_setup_memory(const conf_mem_t &conf_mem)
{
    // enlarge only
    if (size() < conf_mem.nch)
        resize(conf_mem.nch);

    fragm_len_max = conf_mem.getMasterMaxFlen();
    fragm_count_max = conf_mem.getMasterMaxFcnt();
    ch_dpm_ks = conf_mem.getMasterChannelMemorySizeKs();

    //  unsigned int maxrecwin = conf_mem.fragm_len_max * conf_mem.fragm_count_max; // FIXME

    for(int ch=0; ch<conf_mem.nch; ++ch){
        bits[ch] = conf_mem.getMasterAdcBits();
        step[ch] = conf_mem.getMasterAdcStepNs();
    }
//    bits = QVector<int>(conf_mem.nch, conf_mem.getMasterAdcBits());
//    step = QVector<double>(conf_mem.nch, conf_mem.getMasterAdcStepNs());


    if ((int)mult_max > conf_mem.nch && conf_mem.nch) {
        qWarning() << QString("Configuration error: mult_max is too large (%1), limited to %2")
                      .arg(mult_max).arg(conf_mem.nch);
        mult_max = conf_mem.nch;
    }

    //Make adjusting just if any board recognized
    if(conf_mem.boards().size())
        adjust_adc_recwin();

    if (!conf_mem.hasTriggerControl()) {
        trigbits = ADCM_TRIGBIT_INT;
    }
}
