//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ADCCONFIG_H
#define ADCCONFIG_H

#include "inibased.h"
#include <QVector>

typedef enum {
    ADCM_RUNMODE_RUN  = 0x8000u,
    ADCM_RUNMODE_BASECAL = 0x4000u
} adcm_runmode_t;

class conf_mem_t;
class AdcConfig : public IniBased
{
public:
    AdcConfig();
    virtual ~AdcConfig() {}
    void print() const;
    bool operator==(const AdcConfig &other) const;
    bool operator!=(const AdcConfig &other) const {return !operator==(other);}
    bool isBaselineCalibrationMode() const;
//    static double mv_to_value_scaler ();
    double thr_mv_min () const;
    double thr_mv_max () const;
    void setThr(int ch, double value);
    void setGain(int ch, double value);
    void setTrigBits(int value);
    int adc_rawdata_maxval(int ch) const;
    int adc_rawdata_midval(int ch) const;
    int adc_rawdata_minval(int ch) const;
    bool is_testmode() const;
    bool is_co_trigger() const;
    bool is_ext_trigger() const;
    bool is_simple_trigger() const;
    bool ch_enabled_offset_calibration (int ch) const;
    bool ch_enabled_shape_calibration (int ch) const;
    bool ch_enabled_reconstruction (int ch) const;
    unsigned int adc_cfg_get_recwin_max() const;
    void adjust_adc_recwin();
    void read_adc_setup_memory(const conf_mem_t &conf_mem);
    int size() const { return nch; }

    virtual void writeToIni(QSettings *settings, const QString &section = QString()) const;
    virtual void readFromIni(QSettings *settings, const QString &section = QString());
    virtual QString sectionName() const { return "adc"; }

    QVector<int> bits;            //!< ADC resolution, bits
    QVector<double> step;         //!< ADC time step, ns
    QVector<double> gain;         //!< preamp gain, dB
    QVector<double> thr;          //!< digital threshold, mV
    QVector<int> trig;            //!< trigger on this channel when not zero
    QVector<unsigned int> recwin; //!< full recording window (incl. prewin), step-ns clocks
    int prewin;                   //!< pre-threshold recording window, step-ns clocks
    int matchwin;                 //!< coincidence match window, step-ns clocks
    int latency;                  //!< data latency, step-ns clocks
    adcm_runmode_t runmode;       //!< run mode bit field
//    bool run_enable;              //
    int trigbits;                 //!< trigger bits
    unsigned int mult_min;        //!< minimal multiplicity
    unsigned int mult_max;        //!< maximal multiplicity
    unsigned int fragm_len_max;
    unsigned int fragm_count_max;
    unsigned int ch_dpm_ks;       //!< channel ring memory size in kilosamples
    unsigned int timestamp_is_utc;         //!< no timestamp reset on cycle start, use UTC time

protected:
    void resize(int n);
    int nch;
//    void updateTriggerBits();
};

Q_DECLARE_METATYPE(AdcConfig)

#endif // ADCCONFIG_H
