//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "calavgconfig.h"

#include <cstdio>
#include <cmath>

#include "adcm16.h"
#include "dsplib/dsplib.h"
#include "adcconfig.h"

CalAvgConfig::CalAvgConfig()
{
//    for (int c = 0; c < step.size(); c++) {
//        step[c] = 1.;
//    }
//    for (int c = 0; c < (int)ADC_NCH; c++) {
//        //data[c].resize(ADCM_MAX_N_SAMP);
//        const int CALAVG_DEFAULT_SIZE = 100;
//        data[c].resize(CALAVG_DEFAULT_SIZE);
//        for(int i=0; i<data[c].size(); i++) {
//            double rt = 2.5;
//            double ft = 500;
//            double t0 = CALAVG_DEFAULT_SIZE/4;
//            data[c][i] = detpulse(1, i-t0, rt, ft);
//        }
//        nent[c] = 1;
//    }
//    clear();
}

bool CalAvgConfig::operator==(const CalAvgConfig &other) const
{
    return true
            && nent == other.nent
            && nf == other.nf
            && data == other.data
            && step == other.step
            ;
}

void CalAvgConfig::print() const
{
    for (int c = 0; c < nent.size(); c++) {
        if (nent[c] <= 0)
            continue;
        fprintf (stderr, "calavg[%u]: nent=%d, step=%g, nf=%g, data={", c, nent[c], step[c], nf[c]);
        for (int i = 0; i < data[c].size(); i++)
            fprintf (stderr, "%d ", data[c][i]);
        fprintf (stderr, "}\n");
    }
}

void CalAvgConfig::resize(int n)
{
    if(nent.size() >= n) return;
    nent.resize(n);
    nf.resize(n);
    step.resize(n);
}

CalAvgConfig &CalAvgConfig::operator+=(const CalAvgConfig &rhs)
{
    if (nent.size() < rhs.nent.size())
        resize(rhs.nent.size());

    for(int c=0; c<rhs.nent.size(); c++) {
        if(rhs.nent[c] == 0) continue;
        nent[c] += rhs.nent[c];
        if(data[c].size() < rhs.data[c].size())
            data[c].resize(rhs.data[c].size());
        for(int i=0; i<rhs.data[c].size(); i++)
            data[c][i] += rhs.data[c][i]; // FIXME: optimize
    }
    return *this;
}

const CalAvgConfig &CalAvgConfig::operator+(const CalAvgConfig &other) const
{
    return CalAvgConfig(*this) += other;
}

void CalAvgConfig::clear_calavg_ch (int ch)
{
    if (ch > nent.size())
        return;
    nent[ch] = 0;
//    nf[ch] =
//            /*DEFAULT_CALAVG_NF */ fft_noisefloor (1 / sqrt (12.) /*ADC_RMSNOISE[c] */ , ADCM_MAX_N_SAMP);
//    //      printf("nf: %e\n", buf->nf[c]);
    data[ch].clear();
}

void CalAvgConfig::clear()
{
    nent.clear();
    nf.clear();
    data.clear();
    step.clear();
}

void CalAvgConfig::clear_enabled(const AdcConfig &adc)
{
    for (int c = 0; c < nent.size(); c++) {
        if (!adc.ch_enabled_shape_calibration (c))
            continue;
        clear_calavg_ch(c);
    }
}

bool CalAvgConfig::isValid()
{
    for(int ch = 0; ch < nent.size(); ++ch){
        if(nent[ch] && !data.contains(ch) && data[ch].size() < 4)
            return false;
    }
    if(step.size() != nent.size() || nf.size() != nent.size())
        return false;
    return true;
}
