//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CALAVGCONFIG_H
#define CALAVGCONFIG_H

#include <QMap>

#include "inibased.h"

class AdcConfig;

/// average shape calibration structure
class CalAvgConfig : public IniBased
{
public:
    CalAvgConfig();
    virtual ~CalAvgConfig() {}
    void print() const;
    void resize(int n);
    void clear();
    void clear_calavg_ch (int ch);
    void clear_enabled(const AdcConfig &adc);
    CalAvgConfig &operator+=(const CalAvgConfig &rhs);
    const CalAvgConfig &operator+(const CalAvgConfig &rhs) const;
    bool operator==(const CalAvgConfig &other) const;
    bool operator!=(const CalAvgConfig &other) const {return !operator==(other);}
    virtual void writeToIni(QSettings *settings, const QString &section = QString()) const;
    virtual void readFromIni(QSettings *settings, const QString &section = QString());
    virtual QString sectionName() const { return "calavg"; }
    bool isValid();

    QVector<int> nent;           //!< number of averaged waveforms, must be < 2M (overflow warning)
    QVector<double> nf;          //!< FFT noise floor, dB
    QMap<int, QVector<int> > data; //!< unnormalized average waveform (divide by nent when use it)
    QVector<double> step; // unused yet
};

Q_DECLARE_METATYPE(CalAvgConfig)


#endif // CALAVGCONFIG_H
