//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CALZEROCONFIG_H
#define CALZEROCONFIG_H

#include <QVector>

#include "inibased.h"

class AdcConfig;

// zero calibration structure
class CalZeroConfig : public IniBased
{
public:
    explicit CalZeroConfig(int numChannels = 0);
    void resize(int n);
    int nch() const;
    void print() const;
    void clear();
    void clear_calzero_ch(int ch);
    void clear_enabled(const AdcConfig &adc);
    double get(int ch) const;
    double getcalnoiserms (int ch) const;
    CalZeroConfig &operator+=(const CalZeroConfig &rhs);
    const CalZeroConfig &operator+(const CalZeroConfig &rhs) const;
    bool operator==(const CalZeroConfig &other) const;
    bool operator!=(const CalZeroConfig &other) const {return !operator==(other);}
    virtual void writeToIni(QSettings *settings, const QString &section = QString()) const;
    virtual void readFromIni(QSettings *settings, const QString &section = QString());
    virtual QString sectionName() const { return "calzero"; }

    QVector<int> nent;          // number of measured waveforms, must be < 2M (overflow warning)
    QVector<double> base;
    QVector<double> rms;
    QVector<double> asym;        // noise asymmetry
    int initialized_nch;
};

Q_DECLARE_METATYPE(CalZeroConfig)

#endif // CALZEROCONFIG_H
