//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DECODERCONFIG_H
#define DECODERCONFIG_H

#include "inibased.h"
#include "calzeroconfig.h"
#include "calavgconfig.h"
#include "dsplib/fir_types.h"

enum {
    FIT_PROFILE_FAST      = 1,
    FIT_PROFILE_OPTIMAL   = 2,
    FIT_PROFILE_PRECISION = 3,
    FIT_PROFILE_CUSTOM    = 4,
    FIT_PROFILE_SUPERFAST = 5,
    FIT_PROFILE_DEFAULT   = FIT_PROFILE_OPTIMAL
};

typedef int prof_id_t;

//class intfir_t;

class DecoderConfig : public IniBased
{
public:
    DecoderConfig();
    virtual ~DecoderConfig() {}
    void print() const;
    bool operator==(const DecoderConfig &other) const;
    bool operator!=(const DecoderConfig &other) const {return !operator==(other);}

    int nch() const;
    int get_dec_fititer() const;
    int get_dec_fitalgo() const;
    int get_dec_fitfixpar() const;
    virtual void writeToIni(QSettings *settings, const QString &section = QString()) const;
    virtual void readFromIni(QSettings *settings, const QString &section = QString());
    virtual QString sectionName() const { return "dec"; }

    int mode;                   //!< decoder mode
//    unsigned int stages;
    CalZeroConfig calzero;
    CalAvgConfig calavg;
    //	QVector<prof_id_t> prof_id; //!< channel profile ID
    QVector<fwinf_t> fwinf;         //!< Filter window function
    QVector<double> ppos;       //!< ref pulse position, ns
    QVector<double> pwid;       //!< ref pulse width (2.35*sigma), ns
    QVector<double> pnf;        //!< ref pulse threshold
    QVector<double> tadj;       //!< fine time adjust, ns, (-200..200)
    QVector<intfir_t> fir;      //!< FIR digital filters
    //  QVector<double>  pwmin;       //!< fitted pulse width - minimum
    //  QVector<double>  pwmax;       //!< fitted pulse width - maximum
    QVector<int> blr1;          //!< BLR window start
    QVector<int> blr2;          //!< BLR window end
    QVector<int> int1;          //!< integral window start (rel. to threshold position)
    QVector<int> int2;          //!< integral window end
    QVector<int> expfall;       //!< exponential falloff correction for shape reconstruction
    QVector<int> puthr;         //!< Pile-Up threshold, percent (>=100 == disabled)
    QVector<int> pumindist;     //!< Pile-Up minimum distance, ns
    int fprof;                  //!< Fitter profile select (superset of fit- iter, side, fixpar, algo)
    int fititer;                //!< Fitter iterations limit
    int fitside;                //!< Fitter number of points (2*N+1 points to fit), 0 - auto
    int fitfixpar;              //!< bit field, select which parameter to fix
    int fitalgo;                //!< Fitter algorithm select
    //    QVector<int> search_start;
    //    QVector<int> search_end;
    QVector<double> baserc;      //!< baseline validation range, rel. to rms noise
    double zero_thr;             //!< (mV) Minimal signal threshold for run-time baseline calibration
    int dt_cut_en;
    double dt_cut_l; //!< ns
    double dt_cut_r; //!< ns
    int initialized_nch;
};

Q_DECLARE_METATYPE(DecoderConfig)

#endif // DECODERCONFIG_H
