//
//    Copyright 2013 Ivan Filippov
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef INIBASED_H
#define INIBASED_H

#include <QString>
#include <QVector>
#include <QVariant>
#include <QSettings>

#include "dsplib/WindowFunction.h"

class IniBased
{
public:
    virtual ~IniBased() {}

protected:
    template<typename T>
    void setValueIfPresent(QSettings *settings, QString key, T &value){
        if(settings->allKeys().contains(key)) setValue(value, settings->value(key));
    }

    template<typename T>
    void setValue(T &value, QVariant var);
    void setValue(bool &value, QVariant var) { value = var.toBool(); }
    void setValue(int &value, QVariant var) { value = var.toInt(); }
    void setValue(unsigned int  &value, QVariant var) { value = var.toUInt(); }
    void setValue(quint64 &value, QVariant var) { value = var.toULongLong(); }
    void setValue(double &value, QVariant var) { value = var.toDouble(); }
    void setValue(QString &value, QVariant var) { value = var.toString(); }

private:
    virtual void writeToIni(QSettings *settings, const QString &section = QString()) const = 0;
    virtual void readFromIni(QSettings *settings, const QString &section = QString()) = 0;
    virtual QString sectionName() const = 0;

};

#endif // INIBASED_H
