
#include "log.h"

#include <cstdarg>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <ctime>
#include <string>
#include <cerrno>
#include <cstdio>
#include <cstring>

#include "util/gettimeofday.h"
#include "ansi_col.h"

#define USER_FOREGR
//#define SHORT_TIME

namespace colorlog {

    const size_t LOGBUFFER_SIZE = 1024;

    int debug_level = 0;

    // colors for cmsg()
    enum { C_NORM = 0, C_RED = 1, C_GREEN = 2, C_BLUE = 3, C_YELLOW = 4, C_MAGENTA = 5 };

    struct shm_usrlog_t *shm_usrlog = 0;
    bool shm_usrlog_readonly = true;

    void get_debug_level (int *level)
    {
        if (level)
            *level = debug_level;
    }

    void set_debug_level (int level)
    {
        debug_level = level;
    }

    int debug_level_ge(int level) // return 1 if debug_level >= level
    {
        return (debug_level >= level);
    }

    static double log_internal_time_save = 0;

    void cmsg (const std::string &txt, int color)
    {
        time_t mytime;
        struct tm *tm =0;
        struct timeval tv;
        char stm[65];
        int ms;
        double tnow, dt;
        std::ostringstream s;

        stm[64]=0;
        gettimeofday (&tv, NULL);
        tnow = tv.tv_sec + 1e-6 * tv.tv_usec;
        if (log_internal_time_save > 0)
            dt = tnow - log_internal_time_save;
        else
            dt = 0;
        log_internal_time_save = tnow;
        ms = (int) (1e-3 * tv.tv_usec);
        mytime = tv.tv_sec;
        tm = localtime (&mytime);
        if (tm) {
#ifdef SHORT_TIME
            strftime (stm, 64, "%H:%M:%S", tm);
#else
            strftime (stm, 64, "%b %d %H:%M:%S", tm);
#endif
        } else {
            strncpy(stm, "error", sizeof(stm)-1);
        }
        s << stm << "." << std::setw(3) << std::setfill('0') << ms;
        //        s << " +" << std::setw(5) << std::setfill('0') << dt;
        s << " ";
        const char *colorBegin = "";
        const char *colorEnd = "";
        /*
#ifdef USER_FOREGR
        colorEnd = ANSI_NORM
        switch (color) {
        case C_MAGENTA:
            colorBegin = ANSI_MAGENTA;
            break;
        case C_RED:
            //      colorBegin = ANSI_RED;
            colorBegin = ANSI_MAGENTA;
            break;
        case C_GREEN:
            colorBegin = ANSI_GREEN;
            break;
        case C_BLUE:
            colorBegin = ANSI_BLUE;
            break;
        case C_YELLOW:
            //      colorBegin = ANSI_YELLOW;
            colorBegin = ANSI_RED;
            break;
        default:
            colorEnd = "";
        }
#else
        colorEnd = ANSI_NORM
        switch (color) {
        case C_MAGENTA:
            colorBegin = ANSI_MAGENTA;
            break;
        case C_RED:
            colorBegin = ANSI_MAGENTA;
            //    colorBegin = ANSI_RED;
            break;
        case C_GREEN:
            colorBegin = ANSI_GREEN;
            break;
        case C_BLUE:
            colorBegin = ANSI_BLUE;
            break;
        case C_YELLOW:
            //    colorBegin = ANSI_YELLOW;
            colorBegin = ANSI_RED;
            break;
        default:
            colorEnd = "";
        }
        s << txt;
#endif
*/
        s << colorBegin;
        //        s << " [" << getpid() << "]: ";
        s << txt;
        s << colorEnd;
        std::cerr << s.str() << std::endl;
        std::cerr.flush();
    }

    void mlog (int type, const char *txt)
    {
        //        fprintf(stderr, "%s\n", txt);
        //        //    fflush(stderr);
        //        return;
        // if (level > LOGLEVEL) return;
        switch (type) {
        case LOG_ALERT:
            cmsg (txt, C_MAGENTA);
            break;
        case LOG_CRIT:
            cmsg (txt, C_MAGENTA);
            break;
        case LOG_ERR:
            cmsg (txt, C_RED);
            break;
        case LOG_WARNING:
            cmsg (txt, C_YELLOW);
            break;
        case LOG_NOTICE:
            cmsg (txt, C_GREEN);
            break;
        case LOG_INFO:
            cmsg (txt, C_BLUE);
            break;
        case LOG_DEBUG:
            cmsg (txt, C_NORM);
            break;
        default:
            cmsg (txt, C_NORM);
        }
    }

    void log_debug (int level, const char *fmt, ...)   // used only with -DDEBUG
    {
#ifdef DEBUG
        static char buf[LOGBUFFER_SIZE+1];
        va_list msg;

        if (level > debug_level)
            return;

        va_start (msg, fmt);
        vsnprintf (buf, LOGBUFFER_SIZE, fmt, msg);
        va_end (msg);
        buf[LOGBUFFER_SIZE] = 0;

        mlog (LOG_DEBUG, buf);
#endif
    }

    void log_debug (int level, const std::string &str)   // used only with -DDEBUG
    {
#ifdef DEBUG
        if (level > debug_level)
            return;
        mlog (LOG_DEBUG, str.c_str());
#endif
    }

    void logm (LOG_PRIORITY priority, const char *fmt, ...)
    {
        static char buf[LOGBUFFER_SIZE+1];
        va_list msg;
        //  int newptr;

        if ((priority == LOG_DEBUG) && (debug_level <= 0))
            return;

        va_start (msg, fmt);
        vsnprintf (buf, sizeof (buf), fmt, msg);
        va_end (msg);

        buf[LOGBUFFER_SIZE] = 0;

        mlog (priority, buf);
    }

    void logm (LOG_PRIORITY priority, const std::string &str)
    {

        if ((priority == LOG_DEBUG) && (debug_level <= 0))
            return;

        mlog (priority, str.c_str());
    }
}
