#ifndef MLINK_COLORLOG_LOG_H
#define MLINK_COLORLOG_LOG_H

#include <string>

namespace colorlog {

    // Log levels
    enum LOG_PRIORITY {
        LOG_ALERT = 10,    /* action must be taken immediately */
        LOG_CRIT = 20,     /* critical conditions              */
        LOG_ERR = 30,      /* error conditions                 */
        LOG_WARNING = 40,  /* warning conditions               */
        LOG_NOTICE = 50,   /* normal but significant condition */
        LOG_INFO = 60,     /* informational                    */
        LOG_DEBUG = 70,    /* debug-level messages             */
    };

#define  log_info(fmt, ...)   logm(colorlog::LOG_INFO, fmt, ##__VA_ARGS__)
#define  log_notice(fmt, ...) logm(colorlog::LOG_NOTICE, fmt, ##__VA_ARGS__)
#define  log_warn(fmt, ...)   logm(colorlog::LOG_WARNING, fmt, ##__VA_ARGS__)
#define  log_err(fmt, ...)    logm(colorlog::LOG_ERR, fmt,  ##__VA_ARGS__)
#define  log_fatal(fmt, ...)  logm(colorlog::LOG_CRIT, fmt, ##__VA_ARGS__)

    int debug_level_ge(int level);
    void get_debug_level (int *level);
    void set_debug_level (int level);
    void logm (LOG_PRIORITY priority, const char *fmt, ...);
    void logm (LOG_PRIORITY priority, const std::string &str);
    void log_pri (LOG_PRIORITY priority, const char *fmt, ...);  // log to system
    void log_pri (LOG_PRIORITY priority, const std::string &str);  // log to system
    void log_debug (int level, const char *fmt, ...);   // used only with -DDEBUG
    void log_debug (int level, const std::string &str);   // used only with -DDEBUG
    //  void log_err(const std::string &str);
    //  void log_err(const char *, ...);
}

#endif /* MLINK_COLORLOG_LOG_H */
