/*
 *
 * M-Link hardware interface
 *
 */



#ifndef FRAME_HW_H
#define FRAME_HW_H 1

#include <cstddef>
#include "mlink.h"

namespace mlink {

#define ETIMERECV 10001
#define ETIMESEND 10002
#define EBADCRC   10003

typedef int mlink_event_t;

const char *ml_strerror(int errnum);

int ml_open (const char *devname);
int ml_get_caps (mlink_caps_t *caps);
int ml_get_driver_status (mlink_status_t *st);
int ml_reset ();
int ml_get_driver_version(unsigned long *n);
int ml_get_driver_revision(unsigned long *n);
int ml_close ();

// *data - pointer to contents
// len - size of contents in 32-bit words
// type - frame type
// src, dst - address
// seq - sequence number
// timeout - milliseconds
int ml_send_data_frame(const int *data, size_t len, int type, int src, int dst, int seq, int timeout);

// *flen - on input, user shall specify the allocated data buffer size, words
// *flen - on output, it contains the actual number of words received
// others same as above
int ml_recv_data_frame(int *fdata, size_t *flen, int *ftype, int *fsrc, int *fdst, int *fseq, int timeout);



enum { ML_FRAMESET_NUMDESC = 1024 }; // 16kByte max rawdata, min frame size 16 bytes // FIXME
struct framedesc_t {
    const int *rawdata;
    size_t len;
    int frame_flags;
};
struct frameset_t {
    //  size_t maxdesc; // number of descriptors allocated
    size_t count; // number of descriptors used
    struct framedesc_t fdesc[ML_FRAMESET_NUMDESC];
};

int ml_recv_data_frame_multi(int *fdata, size_t fsize, struct frameset_t *frameset, int timeout);

int ml_get_frame_seq ();
int ml_adv_frame_seq ();

// rx/tx bytes counter
int ml_get_rxb();
int ml_get_txb();
// rx/tx frames counter
int ml_get_rxf();
int ml_get_txf();
void ml_clear_rxtx_counters();
// event-driven interface
int ml_get_fd();

int ml_io_wr (const int *txbuf, int *rxbuf, size_t txlen, size_t rxlen);

int ml_get_event (mlink_event_t *ev, int timeout_ms);

}

#endif /* FRAME_HW_H */
