#ifndef MLINK_JTAG_ENGINE_H
#define MLINK_JTAG_ENGINE_H

#include <cstddef>

#include "jtag_state.h"

class QString;

namespace mlink {

    class Jtag_Impl;

    class JTAG_Engine {
    public:
        explicit JTAG_Engine(Jtag_Impl *p = 0);
        void setImpl(Jtag_Impl *p) { impl = p; }
        Jtag_Impl *getImpl() { return impl; }

        // JTAG operations
        int jtag_reset ();
        void jtag_pause(unsigned int count); // do 'count' times with TMS=0 TDI=0
        int jtag_drscan (unsigned int count, const void *tdiData, void *tdoData);
        int jtag_drscan_chk (unsigned int count, const void *tdiData, void *tdoData);
        int jtag_irscan (unsigned int count, const void *tdiData, void *tdoData);

    protected:
        bool _perform_scan(size_t count, const char *tdiData, char *tdoData);
        void _internal_jtag_reset ();
        bool _internal_jtag_drscan (const unsigned int count, const char *tdiData, char *tdoData);
        bool _internal_jtag_irscan (unsigned int count, const char *tdiData, char *tdoData);

        void TRACE(int level, const QString &text) const;

    private:
        Jtag_Impl *impl;
        JTAG_STATE jtagState;
        JTAG_STATE irStopState;  // end state after instruction register scan (one of the 4 idle states)
        // JTAG_STATE drStopState;  // end state after data register scan
    };
};

#endif /* MLINK_JTAG_ENGINE_H */
