
#include "jtag_impl.h"

#include <iostream>
#include <sstream>
#include <cassert>

namespace mlink {

Jtag_Impl::~Jtag_Impl()
{
    std::cerr << getTrace() << std::endl;
}

int Jtag_Impl::jtag_io_b32 (int tdi)
{
    int tdo = 0;
    for (int b=0; b<32; b++) {
        int tdobit = jtag_io (0, tdi&(1<<b)?1:0); // TMS=0
        tdo |= tdobit?(1<<b):0;
    }
    return tdo;
}

void Jtag_Impl::jtag_io_b32_blk (unsigned int count, const unsigned int *tdi32Data, unsigned int *tdo32Data)
{
    for(unsigned int n=0;n<count;n++) {
        unsigned int tdi32 = tdi32Data?tdi32Data[n]:0;
        unsigned int tdo32 = jtag_io_b32(tdi32);
        if (tdo32Data)
            tdo32Data[n] = tdo32;
    }
}

std::vector<bool> Jtag_Impl::jtag_io_b(const std::vector<bool> &btms, const std::vector<bool> &btdi)
{
    size_t count = btms.size();
    assert(btms.size() == btdi.size());
    std::vector<bool> btdo(count);
    for(size_t i=0; i<count; i++) {
        btdo[i] = jtag_io(btms[i], btdi[i]);
    }
    return btdo;
}

void Jtag_Impl::addTracePoint(bool tms, bool tdi)
{
    std::ostringstream os;
    os << tms << " " << tdi << std::endl;
    if (traceEnableConsole) {
        std::cerr << os.str();
    }
    if (traceEnable)
        trace += os.str();
}

void Jtag_Impl::addTraceComment(const std::string &s)
{
    if (traceEnableConsole) {
        std::cerr << s;
    }
    if (traceEnable)
        trace += s;
}
};
