#ifndef MLINK_JTAG_IMPL_H
#define MLINK_JTAG_IMPL_H

#include <cstddef>
#include <string>
#include <iosfwd>
#include <vector>

namespace mlink {

    enum { ML_DEFAULT_JTAG_ADDRESS = 1 };

    class Jtag_Impl_Options
    {
    public:
        virtual ~Jtag_Impl_Options() { }
    };

    class Jtag_Impl
    {
    public:
        explicit Jtag_Impl(int destAddress = ML_DEFAULT_JTAG_ADDRESS)
            : p_destAddress(destAddress),
            traceEnable(false), traceEnableConsole(false) {}
        virtual ~Jtag_Impl();
        void setTracing(bool state = true, bool stateConsole = true) { traceEnable = state; traceEnableConsole = stateConsole; }
        std::string getTrace() const { return trace; }
        //  virtual void setOptions(Jtag_Impl_Options *p) = 0;
        //  virtual Jtag_Impl_Options *getOptions() = 0;
        virtual bool jtag_io (bool tms, bool tdi) = 0;
        virtual std::vector<bool> jtag_io_b(const std::vector<bool> &btms, const std::vector<bool> &btdi);
        virtual void jtag_io_b32_blk (unsigned int count, const unsigned int *tdi32Data, unsigned int *tdo32Data);

    protected:
        virtual int jtag_io_b32 (int tdi); // emulate 32-bit i/o with single i/o
        bool isTraceEnabled() const { return traceEnable; }
        void addTracePoint(bool tms, bool tdi);
        void addTraceComment(const std::string &s);

        int p_destAddress;

    private:
        bool traceEnable;
        bool traceEnableConsole;
        std::string trace;
    };

    class Jtag_Impl_Null : public Jtag_Impl
    {
    public:
        virtual bool jtag_io (bool tms, bool tdi) { addTracePoint(tms, tdi); return 0; }
    };
};

#endif /* MLINK_JTAG_IMPL_H */
