
#include "jtag_state.h"

#include <iostream>

namespace mlink {

    const char *jtagStateMap[JTAG_STATE_COUNT] = {
        "TEST_LOGIC_RESET",
        "RUN_TEST_IDLE",
        "SELECT_DR_SCAN",
        "CAPTURE_DR",
        "SHIFT_DR",
        "EXIT1_DR",
        "PAUSE_DR",
        "EXIT2_DR",
        "UPDATE_DR",
        "SELECT_IR_SCAN",
        "CAPTURE_IR",
        "SHIFT_IR",
        "EXIT1_IR",
        "PAUSE_IR",
        "EXIT2_IR",
        "UPDATE_IR"
    };

    const char *jtagStateStr(JTAG_STATE state)
    {
        if (state >= 0 && state < JTAG_STATE_COUNT)
            return jtagStateMap[state];
        else
            return "invalid";
    }

    // The following array stores the transition table for the TAP
    // controller.  It tells us what the next state will be by placing the
    // current state code in the first index and the value of
    // the TMS input in the second index.

    JTAG_STATE jtag_next_state[JTAG_STATE_COUNT][2] = {
        /*                       TMS=0          TMS=1           */
        /* Test-Logic Reset */ { JS_RUN_TEST_IDLE, JS_TEST_LOGIC_RESET },
                               /* Run-Test/Idle    */ { JS_RUN_TEST_IDLE, JS_SELECT_DR_SCAN   },
                                                      /* Select-DR-Scan   */ { JS_CAPTURE_DR,    JS_SELECT_IR_SCAN   },
                                                                             /* Capture-DR       */ { JS_SHIFT_DR,      JS_EXIT1_DR         },
                                                                                                    /* Shift-DR         */ { JS_SHIFT_DR,      JS_EXIT1_DR         },
                                                                                                                           /* Exit1-DR         */ { JS_PAUSE_DR,      JS_UPDATE_DR        },
                                                                                                                                                  /* Pause-DR         */ { JS_PAUSE_DR,      JS_EXIT2_DR         },
                                                                                                                                                                         /* Exit2-DR         */ { JS_SHIFT_DR,      JS_UPDATE_DR        },
                                                                                                                                                                                                /* Update-DR        */ { JS_RUN_TEST_IDLE, JS_SELECT_DR_SCAN   },
                                                                                                                                                                                                                       /* Select-IR-Scan   */ { JS_CAPTURE_IR,    JS_TEST_LOGIC_RESET },
                                                                                                                                                                                                                                              /* Capture-IR       */ { JS_SHIFT_IR,      JS_EXIT1_IR         },
                                                                                                                                                                                                                                                                     /* Shift-IR         */ { JS_SHIFT_IR,      JS_EXIT1_IR         },
                                                                                                                                                                                                                                                                                            /* Exit1-IR         */ { JS_PAUSE_IR,      JS_UPDATE_IR        },
                                                                                                                                                                                                                                                                                                                   /* Pause-IR         */ { JS_PAUSE_IR,      JS_EXIT2_IR         },
                                                                                                                                                                                                                                                                                                                                          /* Exit2-IR         */ { JS_SHIFT_IR,      JS_UPDATE_IR        },
                                                                                                                                                                                                                                                                                                                                                                 /* Update-IR        */ { JS_RUN_TEST_IDLE, JS_SELECT_DR_SCAN   },
                                                                                                                                                                                                                                                                                                                                                                                    };

    std::ostream& operator<< (std::ostream & os, const JTAG_STATE &r)
    {
        os << jtagStateStr(r);
        return os;
    }
};
