
#include "mlink_bitop.h"

#include <cstdio>

#include "colorlog/ansi_col.h"

namespace mlink {

void bitcopy (void *dst, unsigned int doff, const void *src, unsigned int soff, unsigned int len)
{
    const unsigned char *xsrc = (const unsigned char *)src;
    unsigned char *xdst = (unsigned char *)dst;
    if (!dst || !src) return;
    for (unsigned int i = 0; i < len; i++) {
        unsigned int is = i + soff;
        unsigned int id = i + doff;
        (xsrc[is >> 3] & (1 << (is % 8))) ? (xdst[id >> 3] |= (1 << (id % 8))) : (xdst[id >> 3] &= (~(1 << (id % 8))));
    }
}

int bitcmp (const void *in, const void *out, unsigned int len)
{
    const unsigned char *xin  = (const unsigned char *)in;
    const unsigned char *xout = (const unsigned char *)out;

    if (!in && !out) return (0);
    if (!in || !out) return (-1);
    for (unsigned int j = 0; j < len; j++) {
        unsigned int bin = ((xin[j >> 3]) >> (j % 8)) & 1;
        unsigned int bout = ((xout[j >> 3]) >> (j % 8)) & 1;
        if (bin != bout)
            return (-1);
    }
    return 0;
}

void bitprintdiff_groups (const void *in, const void *out, unsigned int len, unsigned int bit_per_group)
{
    const unsigned char *xin  = (const unsigned char *)in;
    const unsigned char *xout = (const unsigned char *)out;
    //            printf("--- bit diff ---\n");
    if (!in) {printf("<null>\n"); return;}
    for (unsigned int j = 0; j < len; j++) {
        unsigned int bin = ((xin[j >> 3]) >> (j % 8)) & 1;
        unsigned int bout = ((xout[j >> 3]) >> (j % 8)) & 1;
        if (bin == bout)
            printf ("%s", ANSI_NORM);
        else
            printf ("%s", ANSI_RED);
        printf ("%u", bout);
	if (j % bit_per_group == (bit_per_group-1)) printf(" ");
    }
    printf ("%s\n", ANSI_NORM);
}

void bitprintdiff (const void *in, const void *out, unsigned int len)
{
    bitprintdiff_groups(in, out, len, 8);
}

void bitprint_groups (const void *in, unsigned int len, unsigned int bit_per_group)
{
    unsigned int p=0, need_cr=0;
    const unsigned char *xin = (const unsigned char *)in;
    //            printf("--- bit dump ---\n");
    if (!in) {printf("<null>\n"); return;}
    for (unsigned int j = 0; j < len; j++) {
        unsigned int bin = ((xin[j >> 3]) >> (j % 8)) & 1;
        printf ("%u", bin);
	if (j % bit_per_group == (bit_per_group-1)) {
            printf(" ");
            if (need_cr) {printf("\n"); need_cr=0;p=0;}
	}
	p++;
	if (bit_per_group < 80 && p+bit_per_group > 80) {
            need_cr = 1;
	}
    }
    printf ("%s\n", ANSI_NORM);
}

void bitprint (const void *in, unsigned int len)
{
    bitprint_groups(in, len, 8);
}

} // namespace mlink
