#ifndef MLINK_REGIOMLINK_H
#define MLINK_REGIOMLINK_H

/*
 * Register-based I/O
 */

#include <cstddef>
#include <string>
#include <vector>
#include <list>
#include <regio/regiobase.h>

namespace mlink {

    const int MlinkHostAddress = 1;

    class RegIOMLink: public RegIOBase {
    public:
        RegIOMLink(const std::string &name = std::string()); //<! Construct M-Link Register I/O object with specified device name
        virtual ~RegIOMLink();
        static std::string defaultDevName();
        virtual void reset();
        //  virtual void setLocalJtagMode(bool state = true);
        //  virtual RegIoPacket io(const RegIoPacket &dataToSend);
        virtual void rawSend(const RegIoPacket &data); //< \deprecated
        //  virtual void sendPackets(const PacketList &);
        virtual unsigned long getMaxDataSize() const;

    protected:
        virtual double getDeviceThroughput() const;
        virtual RawPacketList getRxPackets(double timeout); //<! get packets from hardware
        void devOpen();
        void devClose();
        int nextTxPacketSequenceNumber() { return txPacketSequenceNumber++; }

        unsigned int devcaps; // bit field mlink_caps_t
        std::vector<int> rawRxBuf;
        int txPacketSequenceNumber;
    };
}

#endif /* MLINK_REGIOMLINK_H */
