
#include "regioemu.h"

#include <QDebug>
#include "deviceemu.h"
#include "regio/regiopacket.h"

namespace mlink {

    RegIOEmuBase::RegIOEmuBase(const std::string &name)
        : RegIOBase(name)
    {
        if (name.empty())
            setDevName(defaultDevName());
        bufSizeLimit = 10000;
    }

    RegIOEmuBase::~RegIOEmuBase()
    {
    }

    std::string RegIOEmuBase::defaultDevName()
    {
        return "emulator";
    }

    void RegIOEmuBase::reset()
    {
    }

    //RegIoPacket RegIOEmuBase::io(const RegIoPacket &dataToSend)
    //{
    //  rawSend(dataToSend);
    //  return rawRecv();
    //}

    void RegIOEmuBase::rawSend(const RegIoPacket &data)
    {
        //  pktbuf.clear();
        txBuf.push_back(data);
    }

    RegIoPacket RegIOEmuBase::rawRecv()
    {
        processPackets();
        RegIoPacket pkt;
        if (rxBufCtrl.size() > 0)
            pkt = rxBufCtrl.front();
        rxBufCtrl.pop_front();
        return pkt;
    }

    //PacketList RegIOEmuBase::getData()
    //{
    //  PacketList v(rxBufData);
    //  rxBufData.clear();
    //  return v;
    //}

    RawPacketList RegIOEmuBase::getRxPackets(double timeout)
    {
        Q_UNUSED(timeout);
        processPackets();
        RawPacketList pl;
        Q_FOREACH(RegIoPacket pkt, rxBuf) {
            QByteArray rawpkt = pkt.toByteArray();
            pl.push_back(rawpkt);
        }
        rxBuf.clear();
        return pl;
    }

    void RegIOEmu::processPackets()
    {
        //  log_info("RegIOEmu: TX %zu packets", txBuf.size());
        for(PacketList::const_iterator it = txBuf.begin(); it != txBuf.end(); ++it) {
            const RegIoPacket &pkt = (*it);
            int dst = pkt.dst;
            if (dst == 0xFEFE) {
                qDebug() << "RegIOEmu: Point-to-Point packet";
                for(DeviceList::iterator jt = devices.begin(); jt != devices.end(); ++jt) {
                    if ((*jt).second->isPPAddressEnabled()) {
                        (*jt).second->putPacket(pkt);
                        break; // only first
                    } else {
                        continue;
                    }
                }
            } else {
                if (devices.find(dst) != devices.end()) {
                    //		log_info("RegIOEmu: Packet type %04X #%d to %d", pkt.type, pkt.seq, pkt.dst);
                    devices[dst]->putPacket(pkt);
                } else {
                    qDebug() << QString("RegIOEmu: No such destination %1, packet dropped").arg(dst);
                }
            }
        }
        txBuf.clear();
        for(DeviceList::iterator jt = devices.begin(); jt != devices.end(); ++jt) {
            (*jt).second->processPackets();
        }
        for(DeviceList::iterator jt = devices.begin(); jt != devices.end(); ++jt) {
            const PacketList &devicePackets = (*jt).second->getPackets();
            rxBuf.append(devicePackets);
        }
        //  log_info("RegIOEmu: RX %zu packets", rxBuf.size());
    }

    void RegIOEmu::reset()
    {
        for(DeviceList::iterator jt = devices.begin(); jt != devices.end(); ++jt) {
            (*jt).second->reset();
        }
    }

} // namespace mlink
