

#include <sys/types.h>
#include <sys/stat.h>
//#include <sys/time.h>
#include <unistd.h> // usleep
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cerrno>
#include <climits>
#include <map>
#include <iostream>

#include <QApplication>
#include <QDateTime>
#include <QTimer>

//typedef enum {false=0, true=1} bool;

#include "adcm_df.h"
//#include <mlink/crc32.h>

//#include "cal.C"

const unsigned int a_ch_start = 16;
const unsigned int a_ch_count = 9;
const unsigned int g_ch_start = 1;
const unsigned int g_ch_count = 12;
bool gamma[64];
bool alpha[64];

unsigned int r_ts;
float r_dt;
float r_qa, r_qg;
float r_wa, r_wg;
unsigned short r_ca, r_cg;
unsigned int r_fa, r_fg;


// counters
size_t n_good = 0;
size_t n_bad = 0;
quint64 n_ev_total = 0;
quint64 n_p_total = 0;
quint32 save_ts = 0;
quint32 ec = 0;

//crc32_t crc=0;
//crc32_t save_crc=0;

QDateTime tv, save_tv;

struct stat pst;

typedef std::map<int, quint64> blk_cnt_t;
blk_cnt_t blk_cnt;

void handle_block (const struct stor_packet_hdr_t *ph, const void *buf)
{
  if (!ph) return;
//  crc = crc32_i(buf, ph->size, crc^0xFFFFFFFF);

  if (ph->id == STOR_ID_CMAP) {
    const adcm_cmap_t *chMap = (const adcm_cmap_t *)buf;
    ec=0;
    printf("Channal map:\n");
    for(uint i=0; i<chMap->n;++i){
        printf ("Ch #%d: ", i);
        gamma[i]=chMap->map[i] & 0x2;
        if(gamma[i])
            printf ("gamma ");
        else
            printf ("      ");

        alpha[i]=chMap->map[i] & 0x4;
        if(alpha[i])
            printf ("alpha ");
        else
            printf ("      ");

        if(chMap->map[i] & 0x8)
            printf ("calib");
        printf ("\n");
    }
  } else if (ph->id == STOR_ID_EVNT) {
    n_ev_total++;

    const struct stor_ev_hdr_t *eh = (const struct stor_ev_hdr_t *)buf;
    n_p_total += eh->np;
        quint32 ts = eh->ts;
        quint32 ts_diff = ts-save_ts;
    if (ts_diff > INT_MAX) ts_diff = UINT_MAX-ts_diff;
//    quint64 curTs = (quint64)(1.*QDateTime::currentDateTime().toTime_t()/ADC_REALSTEP);
    if (ts_diff > 1200000000) {
      printf("Timestamp step too large: %u->%u, ev#%u\n", save_ts, ts, ec);
    }
    save_ts = ts;

    const stor_puls_t *ha[32];
    const stor_puls_t *hg[32];
    size_t na=0;
    size_t ng=0;
    for (size_t n = 0; n < eh->np; n++) {
      const stor_puls_t *hit = (const stor_puls_t *)
              ((char *)buf + sizeof (stor_ev_hdr_t) + n * sizeof (stor_puls_t));
//      printf ("%d\t%f\t%f\t%f\n", hit->ch, hit->a, hit->t, hit->w);
//      const bool isa = (hit->ch >= a_ch_start && hit->ch < a_ch_start+a_ch_count);
//      const bool isg = (hit->ch >= g_ch_start && hit->ch < g_ch_start+g_ch_count);
      if (gamma[hit->ch]) { // if (isg) {
        if (ng >= 32) {
          fprintf(stderr, "Hit buffer overflow\n");
          return;
        }
        hg[ng]=hit;
        ng++;
      } else if (alpha[hit->ch]) { // if (isa) {
        if (na >= 32) {
          fprintf(stderr, "Hit buffer overflow\n");
          return;
        }
        ha[na]=hit;
        na++;
      } else {
          fprintf(stderr, "Unknown channel %u\n", hit->ch);
          return;
      }
    }

    if ((na > 1 || ng > 1)|| (na ==0 && ng ==0)) {
      n_bad++;
//  	  fprintf(stderr, "Bad event: na=%u, ng=%u\n", na, ng);
      return;
    }

//	r_ts = ;
        if (na>0 && ng>0) {
        r_dt = hg[0]->t-ha[0]->t;
    } else r_dt=0;
    if (na>0) {
    r_qa = ha[0]->a;
    r_wa = ha[0]->w;
    r_ca = ha[0]->ch - a_ch_start;
    } else {
    r_qa=0; r_wa=0; r_ca=0;
    }
    if (ng>0) {
    r_qg = hg[0]->a;
    r_wg = hg[0]->w;
    r_cg = hg[0]->ch - g_ch_start;
    } else {
    r_qg=0; r_wg=0; r_cg=0;
    }
//  	r_fa = ha[0]->flags;
//  	r_fg = hg[0]->flags;
    n_good++;

    if(ec<10){
        printf("Event: %u(+%u) %s\n", ts, ts_diff, QDateTime::fromTime_t((uint)(1.*ts*ADC_REALSTEP)).toString().toStdString().c_str());
        for(uint i=0;i<na;++i){
            printf("\tAlpha: ch=%2d; t=%+05.2f; w=%+05.2f; a=%05.2f\n",ha[i]->ch,ha[i]->t,ha[i]->w,ha[i]->a);
        }
        for(uint i=0;i<ng;++i){
            printf("\tGamma: ch=%2d; t=%+05.2f; w=%+05.2f; a=%05.2f\n",hg[i]->ch,hg[i]->t,hg[i]->w,hg[i]->a);
        }
    } else if(ec==10)
            printf("\tOthers suppressed...\n");


    ec++;
  }
  else if (ph->id == STOR_ID_CNTR) {
      const struct adcm_counters_t *cnt = (const struct adcm_counters_t *)buf;
      printf("Counters: period=%5.2f, n=%d\n", cnt->time, cnt->n);
      if(cnt->time)
          for(uint i=0;i<cnt->n;++i)
              printf("\t Ch #%d: %6.2fkHz\n", i, 1.*cnt->rawhits[i]/cnt->time/1e3);
      else
          for(uint i=0;i<cnt->n;++i)
              printf("\t Ch #%d: %8d hits\n", i, cnt->rawhits[i]);
  }
}

int read_file(const char *fn)
{
  size_t n_blk_read = 0;
  FILE *fp;

//  crc = CRC32_INIT_VALUE^0xFFFFFFFF;
  tv = QDateTime::currentDateTime();

  fp = fopen(fn, "r");
  if (!fp) {
    fprintf(stderr, "Can't open file %s for reading: %s\n", fn, strerror(errno));
    return (-1);
  }

//  if (fstat(fileno(fp), &pst)) {
//	fprintf(stderr, "Can't stat file %s: %s\n", fn, strerror(errno));
//  }

   while (!feof(fp) && !ferror(fp)) {
      const size_t bsz = 4096;
      quint8 buf[bsz];
      struct stor_packet_hdr_t ph;
      size_t nrd;
      nrd=fread(&ph, sizeof(ph), 1, fp);
      if (nrd==0) break; //EOF
      if (nrd != 1) {
        fprintf(stderr, "Error reading file %s: %s\n", fn, strerror(errno));
        break;
      }
      blk_cnt[ph.id]++;
      bool known = false;
      if (ph.id == STOR_ID_CMAP) known = true;
      if (ph.id == STOR_ID_EVNT) known = true;
      if (ph.id == STOR_ID_CNTR) known = true;
      if (!known) {
        fprintf(stderr, "Unknown block: ID %04X, size: %u\n", ph.id, ph.size);
        break;
      }
      if (ph.size<sizeof(struct stor_packet_hdr_t) || ph.size > bsz) {
        fprintf(stderr, "Bad block size: ID %04X, size: %u\n", ph.id, ph.size);
        break;
      }
      nrd = fread(buf, ph.size-sizeof(struct stor_packet_hdr_t), 1, fp);
      if (nrd != 1) {
        fprintf(stderr, "Error reading file %s: %s\n", fn, strerror(errno));
        break;
      }
      handle_block(&ph, buf);
//  	  printf("Read block: ID %04x, size %u\n", ph.id, ph.size);
//      if (!in.good()) break;
//      if (n_blk_read < 5) printf("x=%8f, y=%8f\n",x,y);
//      if (n_blk_read > 0 && (n_blk_read % 10000) == 0) {
//    	printf("%d blocks read\n", n_blk_read);
//      }
      n_blk_read++;
   }
  fclose(fp);

  printf("\n");
//  if (save_crc == crc) {
//	printf("*** File contents not changed\n");
//  }
//  save_crc = crc;

  printf("Time elapsed: %d sec\n", save_tv.secsTo(tv));
  save_tv = tv;

  printf("File %s: %zu blocks, found %zu good, %zu bad events\n", fn, n_blk_read, n_good, n_bad);
  printf("Events: %llu, pulses:%llu\n", n_ev_total, n_p_total);
  printf("\n");
  std::cout << "Data block count:" << std::endl;
  for(blk_cnt_t::const_iterator it = blk_cnt.begin(); it != blk_cnt.end(); ++it) {
    std::cout << std::hex << it->first << ": " << std::dec << it->second << std::endl;;
  }
  return 0;
}

bool file_changed(const struct stat *st1, const struct stat *st2)
{
//  int da = (st1->st_atime - st2->st_atime);
  int dm = (st1->st_mtime - st2->st_mtime);
//  int dc = (st1->st_ctime - st2->st_ctime);
//  if (da<0 || dm<0 || dc<0) return false; // time doesn't go back!
  if (dm>0) return true;
  if (st1->st_size  != st2->st_size) return true;
  return false;
}

int main(int argc, char *argv[])
{
  const char *fn = "/data/adcm.dat";

  tv = QDateTime::currentDateTime();
  save_tv = tv;

  if (argc==2) {
    fn = argv[1];
  }

  memset(&pst, 0, sizeof(pst));

  while(1) {
    struct stat st;
    if (stat(fn, &st) != 0) {
      fprintf(stderr, "Can't stat file %s: %s\n", fn, strerror(errno));
      goto err;
    }
    if (file_changed(&st, &pst)) {
      printf("Changed file attributes: %s %s %s %s %s\n",
        (pst.st_atime<st.st_atime)?"atime":"",
        (pst.st_mtime<st.st_mtime)?"mtime":"",
        (pst.st_ctime<st.st_ctime)?"ctime":"",
        (pst.st_size!=st.st_size)?"size":"",
        (pst.st_ino!=st.st_ino)?"inode":""
        );
      if (pst.st_atime!=st.st_atime) printf("atime: %lu, %lu (%ld)\n", pst.st_atime, st.st_atime, st.st_atime-pst.st_atime);
      if (pst.st_mtime!=st.st_mtime) printf("mtime: %lu, %lu (%ld)\n", pst.st_mtime, st.st_mtime, st.st_mtime-pst.st_mtime);
      if (pst.st_ctime!=st.st_ctime) printf("ctime: %lu, %lu (%ld)\n", pst.st_ctime, st.st_ctime, st.st_ctime-pst.st_ctime);
      if (pst.st_size !=st.st_size)  printf("size:  %llu, %llu (%lld)\n", (qint64)pst.st_size,  (qint64)st.st_size,  (qint64)(st.st_size-pst.st_size));
      if (pst.st_ino  !=st.st_ino)   printf("ino:   %llu, %llu \n",      (qint64)pst.st_ino,   (qint64)st.st_ino);
      pst = st;
      int ret = read_file(fn);
      if (ret) goto err;
      continue;
    }
        qApp->processEvents();
    continue;
err:
    fprintf(stderr, "Error! (%s)\n", strerror(errno));
        qApp->processEvents();
        usleep(300000);
    continue;
  }
  return 0;
}
