//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef QXW_WAVEFORMVIEWADVANCED_H
#define QXW_WAVEFORMVIEWADVANCED_H

#include "dsplib/fir.h"
#include "qxw/WaveformViewAdvanced.h"
#include "qxw/WaveformAverager.h"
#include "qxw/WaveformFilter.h"
#include "qxw/WaveformModel.h"


class WFChannelMap
{
public:
    int hw_to_plot(int plot_c) {return map_h2p[plot_c];} //!< map hardware channel to plot channel, fast
    //	int plot_to_hw(int plot_c); //!< slow!!

    void set(int hw_c, int plot_c) {map_h2p[hw_c] = plot_c;} //!< establish map
    void clear() {map_h2p.clear();}

private:
    std::map<int, int> map_h2p; // hardware-to-plot channel map
};

class WaveformViewAdvancedPrivate;


#include <vector>

#include <QWidget>

#include "qxw/qxw_global.h"
#include "qxw/WaveformData.h"
#include "qxw/WaveformViewSetup.h"

class WaveformPlot;
class QwtPlotMarker;

namespace Ui {
class WaveformViewAdvancedUi;
}

class WaveformViewAdvanced: public QWidget
{
    Q_OBJECT
public:
    explicit WaveformViewAdvanced(QWidget *parent = 0);
    virtual ~WaveformViewAdvanced();
    void setModel(WaveformModel *p);
    void remap_channels();
    void setFilterData( int channel, const intfir_t &fir );
    void setFullScale( int channel, double value );

    void setCurrentTimeMarker(double v);
    void setXMarkers(const std::vector<double> &v);

    // common with WaveformView
    WaveformPlot *getPlot();
    void setWaveformData( const WaveformDataAll &w );


public slots:
    void setAutoscale( bool b );
    void setBipolarDisplay( bool b );
    void setOffset( int d );
    void update();
    void clear();
    void on_checkBox_OffsetComp_toggled( bool value );
    void on_checkBox_Filter_toggled( bool value );
    void on_checkBox_Averaging_toggled( bool value );
    void on_checkBox_FFT_toggled( bool );

private:
    Ui::WaveformViewAdvancedUi * const ui;
    WaveformViewAdvancedPrivate * const d;
    WaveformViewSetup setup;
    QwtPlotMarker * const currentTimeMarker;
};

#endif // QXW_WAVEFORMVIEWADVANCED_H
